@@ -1,4 +1,3 @@
-#! /usr/bin/env python
 # -*- coding: utf-8 -*-
 
 # file layout2layout.py
@@ -122,6 +121,50 @@ import os, re, string, sys
 # Incremented to format 35, 28 March 2011 by rgh
 # Try to add "Flex:" to any flex insets that don't have it.
 
+# Incremented to format 36, 7 December 2011, by rgh
+# Added HTMLStyles and AddToHTMLStyles tags.
+
+# Incremented to format 37, 29 February 2012 by jrioux
+# Implement the citation engine machinery in layouts.
+# Change CiteFormat to CiteFormat (default|authoryear|numerical).
+
+# Incremented to format 38, 08 April 2012 by gb
+# Introduce LangPreamble and BabelPreamble for InsetLayout.
+
+# Incremented to format 39, 15 April 2012 by sanda
+# Introduce styling of branches via "InsetLayout Branch:".
+
+# Incremented to format 40, 10 October 2012 by rgh
+# Re-do layout names for layout categories
+
+# Incremented to format 41, 20 November 2012 by spitz
+# New Argument syntax
+
+# Incremented to format 42, 22 December 2012 by spitz
+# New Style tag "ItemCommand"
+
+# Incremented to format 43, 30 December 2012 by spitz
+# Extended InsetCaption format
+
+# Incremented to format 44, 9 February 2013 by rgh
+# Remove COUNTER label style; rename as STATIC
+# Rename TOP_ENVIRONMENT to ABOVE and CENTERED_TOP_ENVIRONMENT to CENTERED
+
+# Incremented to format 45, 12 February 2013 by rgh
+# New Tag "NoInsetLayout"
+
+# Incremented to format 46, 15 May 2013 by gb
+# New Tag "ForceLocal"
+
+# Incremented to format 47, 23 May 2013 by rgh
+# Add PackageOptions tag
+
+# Incremented to format 48, 31 May 2013 by rgh
+# Add InitialValue tag for counters
+
+# Incremented to format 49, 10 Feb 2014 by gb
+# Change default of "ResetsFont" tag to false
+
 # Do not forget to document format change in Customization
 # Manual (section "Declaring a new text class").
 
@@ -129,7 +172,7 @@ import os, re, string, sys
 # development/tools/updatelayouts.sh script to update all
 # layout files to the new format.
 
-currentFormat = 35
+currentFormat = 49
 
 
 def usage(prog_name):
@@ -152,7 +195,7 @@ def trim_bom(line):
 
 def read(source):
     " Read input file and strip lineendings."
-    lines = source.read().splitlines()
+    lines = source.read().splitlines() or ['']
     lines[0] = trim_bom(lines[0])
     return lines
 
@@ -203,6 +246,7 @@ def convert(lines):
     re_End = re.compile(r'^(\s*)(End)(\s*)$', re.IGNORECASE)
     re_Provides = re.compile(r'^(\s*)Provides(\S+)(\s+)(\S+)', re.IGNORECASE)
     re_CharStyle = re.compile(r'^(\s*)CharStyle(\s+)(\S+)$', re.IGNORECASE)
+    re_CiteFormat = re.compile(r'^(\s*)(CiteFormat)(?:(\s*)()|(\s+)(default|authoryear|numerical))', re.IGNORECASE)
     re_AMSMaths = re.compile(r'^\s*Input ams(?:math|def)s.inc\s*')
     re_AMSMathsPlain = re.compile(r'^\s*Input amsmaths-plain.inc\s*')
     re_AMSMathsSeq = re.compile(r'^\s*Input amsmaths-seq.inc\s*')
@@ -214,6 +258,7 @@ def convert(lines):
     re_Builtin = re.compile(r'^(\s*)LaTeXBuiltin\s+(\w*)', re.IGNORECASE)
     re_True = re.compile(r'^\s*(?:true|1)\s*$', re.IGNORECASE)
     re_InsetLayout = re.compile(r'^\s*InsetLayout\s+(?:Custom|CharStyle|Element):(\S+)\s*$', re.IGNORECASE)
+    re_ResetsFont = re.compile(r'^(\s*)ResetsFont(\s+)(\S+)$', re.IGNORECASE)
     # with quotes
     re_QInsetLayout = re.compile(r'^\s*InsetLayout\s+"(?:Custom|CharStyle|Element):([^"]+)"\s*$', re.IGNORECASE)
     re_InsetLayout_CopyStyle = re.compile(r'^\s*CopyStyle\s+(?:Custom|CharStyle|Element):(\S+)\s*$', re.IGNORECASE)
@@ -225,6 +270,21 @@ def convert(lines):
     re_QInsetLayout2 = re.compile(r'^\s*InsetLayout\s+"([^"]+)"\s*$', re.IGNORECASE)
     re_IsFlex = re.compile(r'\s*LyXType.*$', re.IGNORECASE)
     re_CopyStyle2 = re.compile(r'(\s*CopyStyle\s+)"?([^"]+)"?\s*$')
+    # for categories
+    re_Declaration = re.compile(r'^#\s*\\Declare\w+Class.*$')
+    re_ExtractCategory = re.compile(r'^(#\s*\\Declare\w+Class(?:\[[^]]*?\])?){([^(]+?)\s+\(([^)]+?)\)\s*}\s*$')
+    ConvDict = {"article": "Articles", "book" : "Books", "letter" : "Letters", "report": "Reports", \
+                "presentation" : "Presentations", "curriculum vitae" : "Curricula Vitae", "handout" : "Handouts"}
+    # Arguments
+    re_OptArgs = re.compile(r'^(\s*)OptionalArgs(\s+)(\d+)\D*$', re.IGNORECASE)
+    re_ReqArgs = re.compile(r'^(\s*)RequiredArgs(\s+)(\d+)\D*$', re.IGNORECASE)
+    
+    # various changes associated with changing how chapters are handled
+    re_LabelTypeIsCounter = re.compile(r'^(\s*)LabelType(\s*)Counter\s*$', re.IGNORECASE)
+    re_TopEnvironment = re.compile(r'^(\s*)LabelType(\s+)Top_Environment\s*$', re.IGNORECASE)
+    re_CenteredEnvironment = re.compile(r'^(\s*)LabelType(\s+)Centered_Top_Environment\s*$', re.IGNORECASE)
+    re_ChapterStyle = re.compile(r'^\s*Style\s+Chapter\s*$', re.IGNORECASE)
+
 
     # counters for sectioning styles (hardcoded in 1.3)
     counters = {"part"          : "\\Roman{part}",
@@ -244,7 +304,7 @@ def convert(lines):
                         "subparagraph"  : "\\arabic{section}.\\arabic{subsection}.\\arabic{subsubsection}.\\arabic{paragraph}.\\arabic{subparagraph}"}
 
     # Value of TocLevel for sectioning styles
-    toclevels = {"part"          : 0,
+    toclevels = {"part"          : -1,
                  "chapter"       : 0,
                  "section"       : 1,
                  "subsection"    : 2,
@@ -271,12 +331,33 @@ def convert(lines):
     formatline = 0
     usemodules = []
     flexstyles = []
+    opts = 0
+    reqs = 0
+    inchapter = False
+    isflexlayout = False         # only used for 48 -> 49
+    # Whether a style is inherited (works only for CopyStyle currently,
+    # not for true inherited styles, see bug 8920
+    inherited = False        # only used for 48 -> 49
+    resetsfont_found = False # only used for 48 -> 49
 
     while i < len(lines):
         # Skip comments and empty lines
-        if re_Comment.match(lines[i]) or re_Empty.match(lines[i]):
-            i += 1
-            continue
+        if (re_Comment.match(lines[i]) or re_Empty.match(lines[i])):
+          # We need to deal with this conversion here, because it happens
+          # inside the initial comment block.
+          if only_comment and format == 39:
+              match = re_ExtractCategory.match(lines[i])
+              if match:
+                  lpre = match.group(1)
+                  lcat = match.group(2)
+                  lnam = match.group(3)
+                  if lcat in ConvDict:
+                      lcat = ConvDict[lcat]
+                  lines[i] = lpre + "{" + lnam + "}"
+                  lines.insert(i+1, "#  \\DeclareCategory{" + lcat + "}")
+                  i += 1 
+          i += 1
+          continue
 
         # insert file format if not already there
         if (only_comment):
@@ -291,7 +372,7 @@ def convert(lines):
                     # nothing to do
                     return format
                 else:
-                    error('Cannot convert file format %s' % format)
+                    error('Cannot convert file format %s to %s' % (format, currentFormat))
             else:
                 lines.insert(i, "Format 2")
                 only_comment = 0
@@ -314,6 +395,171 @@ def convert(lines):
                 i += 1
             continue
 
+        if format == 48:
+            # The default of ResetsFont in LyX changed from true to false,
+            # because it is now used for all InsetLayouts, not only flex ones.
+            # Therefore we need to set it to true for all flex insets which do
+            # do not already have a ResetsFont.
+            match = re_InsetLayout2.match(lines[i])
+            if match:
+                resetsfont_found = False
+                inherited = False
+                name = string.lower(match.group(1))
+                if name == "flex" or name[:5] == "flex:":
+                    isflexlayout = True
+                else:
+                    isflexlayout = False
+            match = re_ResetsFont.match(lines[i])
+            if match:
+                resetsfont_found = True
+            match = re_End.match(lines[i])
+            if match:
+                if isflexlayout and not resetsfont_found and not inherited:
+                    lines.insert(i, "\tResetsFont true")
+                    i += 1
+            match = re_Style.match(lines[i])
+            if match:
+                isflexlayout = False
+                inherited = False
+            match = re_Counter.match(lines[i])
+            if match:
+                isflexlayout = False
+                inherited = False
+            match = re_CopyStyle.match(lines[i])
+            if match:
+                inherited = True
+            i += 1
+            continue
+
+        if format >= 44 and format <= 47:
+            # nothing to do.
+            i += 1
+            continue
+
+        if format == 43:
+          match = re_LabelTypeIsCounter.match(lines[i])
+          if match:
+            if inchapter:
+             lines[i] = match.group(1) + "LabelType" + match.group(2) + "Above"              
+            else:
+              lines[i] = match.group(1) + "LabelType" + match.group(2) + "Static"
+
+          match = re_TopEnvironment.match(lines[i])
+          if match:
+            lines[i] = match.group(1) + "LabelType" + match.group(2) + "Above"
+
+          match = re_CenteredEnvironment.match(lines[i])
+          if match:
+            lines[i] = match.group(1) + "LabelType" + match.group(2) + "Centered"
+
+          if inchapter:
+            match = re_Style.match(lines[i])
+            if match:
+              inchapter = False
+          else:
+            match = re_ChapterStyle.match(lines[i])
+            if match:
+              inchapter = True
+
+          i += 1
+          continue
+
+        if format == 42:
+          if lines[i] == "InsetLayout Caption":
+            lines[i] = "InsetLayout Caption:Standard"
+          i += 1
+          continue
+        
+        if format == 41:
+            # nothing to do.
+            i += 1
+            continue
+
+        if format == 40:
+            # reset counters on Style beginning
+            match = re_Style.match(lines[i])
+            if match:
+                opts = 0
+                reqs = 0
+                i += 1
+                continue
+            match = re_OptArgs.match(lines[i])
+            if match:
+                # Save number of optional arguments
+                space1 = match.group(1)
+                opts = int(match.group(3))
+                # OptionalArgs 0 > ResetArgs 1
+                if opts == 0:
+                    lines[i] = space1 + "ResetArgs\t1"
+                    i += 1
+                else:
+                    del lines[i]
+                continue
+            match = re_ReqArgs.match(lines[i])
+            if match:
+                # Save number of required arguments
+                space1 = match.group(1)
+                reqs = int(match.group(3))
+                del lines[i]
+                continue
+            # Insert the required number of arguments at the end of the style definition
+            match = re_End.match(lines[i])
+            if match:
+                newarg = ['']
+                # First the optionals (this is the required order pre 2.1)
+                if opts > 0:
+                    if opts == 1:
+                        newarg = [ '%sArgument 1' % (space1),
+                                   '%s\tLabelString\t\"Optional Layout Argument\"' % (space1),
+                                   '%sEndArgument' % (space1)]
+                    elif opts > 1:
+                        actopt = 1
+                        while actopt < (opts + 1):
+                            newarg += [ '%sArgument %d' % (space1, actopt),
+                               '%s\tLabelString\t\"Optional Layout Argument %d\"' % (space1, actopt),
+                               '%sEndArgument' % (space1)]
+                            actopt += 1
+                # Now the mandatories
+                if reqs > 0:
+                    actopt = opts + 1
+                    while actopt < (opts +  reqs + 1):
+                        newarg += [ '%sArgument %d' % (space1, actopt),
+                           '%s\tLabelString\t"Required Layout Argument %d"' % (space1, actopt - opts),
+                           '%s\tMandatory\t1' % (space1),
+                           '%sEndArgument' % (space1)]
+                        actopt += 1
+                # Since we replace the "End" line, re-add this line
+                if len(newarg) > 1:
+                    newarg += ['End']
+                    lines[i:i+1] = newarg
+                    i += len(newarg)
+                # Reset the counters
+                opts = 0
+                reqs = 0
+            i += 1
+            continue
+        
+        if format == 39:
+            # There is a conversion with format 40, but it is done within the
+            # initial comment block and so is above.
+            i += 1
+            continue
+
+        if format == 37 or format == 38:
+            i += 1
+            continue
+
+        if format == 36:
+            match = re_CiteFormat.match(lines[i]);
+            if match and match.group(4) == "":
+                lines[i] = match.group(0) + " default"
+            i += 1
+            continue
+
+        if format == 35:
+          i += 1
+          continue
+
         if format == 34:
           match = re_QInsetLayout2.match(lines[i])
           if not match:
@@ -753,6 +999,8 @@ def convert(lines):
 
         i += 1
 
+    if only_comment:
+        lines.insert(i, "Format 2")
     if usemodules:
         i = formatline + 1
         for mod in usemodules:
