@@ -1,4 +1,3 @@
-#! /usr/bin/env python
 # -*- coding: utf-8 -*-
 
 # file lyxpak.py
@@ -146,8 +145,6 @@ def gather_files(curfile, incfiles, lyx2lyx):
         match = re_options.match(lines[i])
         if match:
             file = match.group(3).strip('"')
-            if file.startswith("bibtotoc,"):
-                file = file[9:]
             if not os.path.isabs(file):
                 file = os.path.join(curdir, file + '.bst')
             if os.path.exists(file):
