@@ -78,30 +78,35 @@ def ui_l10n(input_files, output, base):
 
 def layouts_l10n(input_files, output, base, layouttranslations):
     '''Generate pot file from lib/layouts/*.{layout,inc,module}'''
-    Style = re.compile(r'^\s*Style\s+(.*)\s*$', re.IGNORECASE)
+    ClassDescription = re.compile(r'^\s*#\s*\\Declare(LaTeX|DocBook)Class.*\{(.*)\}$', re.IGNORECASE)
+    ClassCategory = re.compile(r'^\s*#\s*\\DeclareCategory\{(.*)\}$', re.IGNORECASE)
+    Style = re.compile(r'^\s*Style\s+(.*\S)\s*$', re.IGNORECASE)
     # match LabelString, EndLabelString, LabelStringAppendix and maybe others but no comments
-    LabelString = re.compile(r'^[^#]*LabelString\S*\s+(.*)\s*$', re.IGNORECASE)
-    GuiName = re.compile(r'^\s*GuiName\s+(.*)\s*$', re.IGNORECASE)
-    ListName = re.compile(r'^\s*ListName\s+(.*)\s*$', re.IGNORECASE)
-    CategoryName = re.compile(r'^\s*Category\s+(.*)\s*$', re.IGNORECASE)
+    LabelString = re.compile(r'^[^#]*LabelString\S*\s+(.*\S)\s*$', re.IGNORECASE)
+    MenuString = re.compile(r'^[^#]*MenuString\S*\s+(.*\S)\s*$', re.IGNORECASE)
+    Tooltip = re.compile(r'^[^#]*Tooltip\S*\s+(.*\S)\s*$', re.IGNORECASE)
+    GuiName = re.compile(r'^\s*GuiName\s+(.*\S)\s*$', re.IGNORECASE)
+    ListName = re.compile(r'^\s*ListName\s+(.*\S)\s*$', re.IGNORECASE)
+    CategoryName = re.compile(r'^\s*Category\s+(.*\S)\s*$', re.IGNORECASE)
     NameRE = re.compile(r'^\s*#\s*\\DeclareLyXModule.*{(.*)}$', re.IGNORECASE)
     InsetLayout = re.compile(r'^InsetLayout\s+\"?(.*)\"?\s*$', re.IGNORECASE)
     FlexCheck = re.compile(r'^Flex:(.*)', re.IGNORECASE)
+    CaptionCheck = re.compile(r'^Caption:(.*)', re.IGNORECASE)
     DescBegin = re.compile(r'^\s*#DescriptionBegin\s*$', re.IGNORECASE)
     DescEnd = re.compile(r'^\s*#\s*DescriptionEnd\s*$', re.IGNORECASE)
-    Category = re.compile(r'^\s*#\s*Category:\s+(.*)\s*$', re.IGNORECASE)
+    Category = re.compile(r'^\s*#\s*Category:\s+(.*\S)\s*$', re.IGNORECASE)
     I18nPreamble = re.compile(r'^\s*((Lang)|(Babel))Preamble\s*$', re.IGNORECASE)
     EndI18nPreamble = re.compile(r'^\s*End((Lang)|(Babel))Preamble\s*$', re.IGNORECASE)
     I18nString = re.compile(r'_\(([^\)]+)\)')
     CounterFormat = re.compile(r'^\s*PrettyFormat\s+"?(.*)"?\s*$', re.IGNORECASE)
     CiteFormat = re.compile(r'^\s*CiteFormat', re.IGNORECASE)
-    KeyVal = re.compile(r'^\s*_\w+\s+(.*)\s*$')
+    KeyVal = re.compile(r'^\s*_\w+\s+(.*\S)\s*$')
     Float = re.compile(r'^\s*Float\s*$', re.IGNORECASE)
-    UsesFloatPkg = re.compile(r'^\s*UsesFloatPkg\s+(.*)\s*$', re.IGNORECASE)
-    IsPredefined = re.compile(r'^\s*IsPredefined\s+(.*)\s*$', re.IGNORECASE)
+    UsesFloatPkg = re.compile(r'^\s*UsesFloatPkg\s+(.*\S)\s*$', re.IGNORECASE)
+    IsPredefined = re.compile(r'^\s*IsPredefined\s+(.*\S)\s*$', re.IGNORECASE)
     End = re.compile(r'^\s*End', re.IGNORECASE)
     Comment = re.compile(r'^(.*)#')
-    Translation = re.compile(r'^\s*Translation\s+(.*)\s*$', re.IGNORECASE)
+    Translation = re.compile(r'^\s*Translation\s+(.*\S)\s*$', re.IGNORECASE)
     KeyValPair = re.compile(r'\s*"(.*)"\s+"(.*)"')
 
     oldlanguages = []
@@ -174,6 +179,18 @@ def layouts_l10n(input_files, output, base, layouttranslations):
         lineno = 0
         for line in open(src).readlines():
             lineno += 1
+            res = ClassDescription.search(line)
+            if res != None:
+                string = res.group(2)
+                if not layouttranslations:
+                    writeString(out, src, base, lineno + 1, string)
+                continue
+            res = ClassCategory.search(line)
+            if res != None:
+                string = res.group(1)
+                if not layouttranslations:
+                    writeString(out, src, base, lineno + 1, string)
+                continue
             if readingDescription:
                 res = DescEnd.search(line)
                 if res != None:
@@ -222,6 +239,18 @@ def layouts_l10n(input_files, output, base, layouttranslations):
                         writeString(out, src, base, lineno, string)
                 continue
             res = LabelString.search(line)
+            if res != None:
+                string = res.group(1)
+                if not layouttranslations:
+                    writeString(out, src, base, lineno, string)
+                continue
+            res = MenuString.search(line)
+            if res != None:
+                string = res.group(1)
+                if not layouttranslations:
+                    writeString(out, src, base, lineno, string)
+                continue
+            res = Tooltip.search(line)
             if res != None:
                 string = res.group(1)
                 if not layouttranslations:
@@ -259,6 +288,10 @@ def layouts_l10n(input_files, output, base, layouttranslations):
                 #if not layouttranslations:
                 #    writeString(out, src, base, lineno, string)
                 m = FlexCheck.search(string)
+                if m:
+                    if not layouttranslations:
+                        writeString(out, src, base, lineno, m.group(1))
+                m = CaptionCheck.search(string)
                 if m:
                     if not layouttranslations:
                         writeString(out, src, base, lineno, m.group(1))
@@ -301,11 +334,14 @@ def layouts_l10n(input_files, output, base, layouttranslations):
                 continue
             res = End.search(line)
             if res != None:
-                # If a float is predefined by the package and it does not need
-                # the float package then it uses the standard babel translations.
-                # This is even true for MarginFigure, MarginTable (both from
+                # We have four combinations of the flags usesFloatPkg and isPredefined:
+                #     usesFloatPkg and     isPredefined: might use standard babel translations
+                #     usesFloatPkg and not isPredefined: does not use standard babel translations
+                # not usesFloatPkg and     isPredefined: uses standard babel translations
+                # not usesFloatPkg and not isPredefined: not supported by LyX
+                # The third combination is even true for MarginFigure, MarginTable (both from
                 # tufte-book.layout) and Planotable, Plate (both from aguplus.inc).
-                if layouttranslations and readingFloat and usesFloatPkg and not isPredefined:
+                if layouttranslations and readingFloat and usesFloatPkg:
                     if floatname != '':
                         keyset.add(floatname)
                     if listname != '':
@@ -336,6 +372,8 @@ def layouts_l10n(input_files, output, base, layouttranslations):
             keys.append(key)
         keys.sort()
 
+        ContextRe = re.compile(r'(.*)(\[\[.*\]\])')
+
         print >> out, '''# This file has been automatically generated by po/lyx_pot.py.
 # PLEASE MODIFY ONLY THE LAGUAGES HAVING NO .po FILE! If you want to regenerate
 # this file from the translations, run `make ../lib/layouttranslations' in po.
@@ -366,14 +404,22 @@ def layouts_l10n(input_files, output, base, layouttranslations):
             for key in keys:
                 if key in trans.keys():
                     val = trans[key].replace('\\', '\\\\').replace('"', '\\"')
+                    res = ContextRe.search(val)
+                    if res != None:
+                        val = res.group(1)
                     key = key.replace('\\', '\\\\').replace('"', '\\"')
                     print >> out, '\t"%s" "%s"' % \
                              (key.encode('utf-8'), val.encode('utf-8'))
                 # also print untranslated entries to help translators
                 elif not lang in oldlanguages:
                     key = key.replace('\\', '\\\\').replace('"', '\\"')
+                    res = ContextRe.search(key)
+                    if res != None:
+                        val = res.group(1)
+                    else:
+                        val = key
                     print >> out, '\t"%s" "%s"' % \
-                             (key.encode('utf-8'), key.encode('utf-8'))
+                             (key.encode('utf-8'), val.encode('utf-8'))
             print >> out, 'End'
 
     out.close()
@@ -428,13 +474,33 @@ def languages_l10n(input_files, output, base):
     out.close()
 
 
+def latexfonts_l10n(input_files, output, base):
+    '''Generate pot file from lib/latexfonts'''
+    out = open(output, 'w')
+    GuiName = re.compile(r'^[^#]*GuiName\s+(.*)', re.IGNORECASE)
+    
+    for src in input_files:
+        descStartLine = -1
+        descLines = []
+        lineno = 0
+        for line in open(src).readlines():
+            lineno += 1
+            res = GuiName.search(line)
+            if res != None:
+                string = res.group(1)
+                writeString(out, src, base, lineno, string)
+                continue
+               
+    out.close()
+
+
 def external_l10n(input_files, output, base):
     '''Generate pot file from lib/external_templates'''
     output = open(output, 'w')
     Template = re.compile(r'^Template\s+(.*)', re.IGNORECASE)
     GuiName = re.compile(r'\s*GuiName\s+(.*)', re.IGNORECASE)
     HelpTextStart = re.compile(r'\s*HelpText\s', re.IGNORECASE)
-    HelpTextSection = re.compile(r'\s*(\S.*)\s*$')
+    HelpTextSection = re.compile(r'\s*(\S.*\S)\s*$')
     HelpTextEnd = re.compile(r'\s*HelpTextEnd\s', re.IGNORECASE)
     i = -1
     for src in input_files:
@@ -512,7 +578,7 @@ def encodings_l10n(input_files, output, base):
     output = open(output, 'w')
     # assuming only one encodings file
     #                 Encoding utf8      utf8    "Unicode (utf8)" UTF-8    variable inputenc
-    reg = re.compile('Encoding [\w-]+\s+[\w-]+\s+"([\w \-\(\)]+)"\s+[\w-]+\s+(fixed|variable)\s+\w+.*')
+    reg = re.compile('Encoding [\w-]+\s+[\w-]+\s+"([\w \-\(\)]+)"\s+[\w-]+\s+(fixed|variable|variableunsafe)\s+\w+.*')
     input = open(input_files[0])
     for lineno, line in enumerate(input.readlines()):
         if not line.startswith('Encoding'):
@@ -546,6 +612,7 @@ where
         layouttranslations: create lib/layouttranslations from po/*.po and lib/layouts/*
         qt4: qt4 ui files
         languages: file lib/languages
+        latexfonts: file lib/latexfonts
         encodings: file lib/encodings
         external: external templates file
         formats: formats predefined in lib/configure.py
@@ -572,7 +639,7 @@ if __name__ == '__main__':
         elif opt in ['-s', '--src_file']:
             input_files = [f.strip() for f in open(value)]
 
-    if input_type not in ['ui', 'layouts', 'layouttranslations', 'qt4', 'languages', 'encodings', 'external', 'formats'] or output is None:
+    if input_type not in ['ui', 'layouts', 'layouttranslations', 'qt4', 'languages', 'latexfonts', 'encodings', 'external', 'formats'] or output is None:
         print 'Wrong input type or output filename.'
         sys.exit(1)
 
@@ -580,6 +647,8 @@ if __name__ == '__main__':
 
     if input_type == 'ui':
         ui_l10n(input_files, output, base)
+    elif input_type == 'latexfonts':
+        latexfonts_l10n(input_files, output, base)
     elif input_type == 'layouts':
         layouts_l10n(input_files, output, base, False)
     elif input_type == 'layouttranslations':
