@@ -49,7 +49,7 @@ public:
 
 	/// create a new buffer and add it to the buffer list
 	/// \return 0 if the Buffer creation is not possible for whatever reason.
-	Buffer * newBuffer(std::string const & s, bool ronly = false);
+	Buffer * newBuffer(std::string const & s);
 
 	/// create an internal buffer and add it to the internal buffer list
 	/// \return 0 if the Buffer creation is not possible for whatever reason.
@@ -126,7 +126,7 @@ public:
 private:
 	/// create a new buffer
 	/// \return 0 if the Buffer creation is not possible for whatever reason.
-	Buffer * createNewBuffer(std::string const & s, bool ronly);
+	Buffer * createNewBuffer(std::string const & s);
 
 	/// noncopiable
 	BufferList(BufferList const &);
