@@ -51,10 +51,10 @@
 #include "TextMetrics.h"
 #include "TexRow.h"
 #include "TocBackend.h"
-#include "VSpace.h"
 #include "WordLangTuple.h"
 
 #include "insets/InsetBibtex.h"
+#include "insets/InsetCitation.h"
 #include "insets/InsetCommand.h" // ChangeRefs
 #include "insets/InsetExternal.h"
 #include "insets/InsetGraphics.h"
@@ -62,8 +62,6 @@
 #include "insets/InsetRef.h"
 #include "insets/InsetText.h"
 
-#include "mathed/MathData.h"
-
 #include "frontends/alert.h"
 #include "frontends/Application.h"
 #include "frontends/Delegates.h"
@@ -71,13 +69,12 @@
 #include "frontends/Painter.h"
 #include "frontends/Selection.h"
 
-#include "graphics/Previews.h"
-
 #include "support/convert.h"
 #include "support/debug.h"
 #include "support/ExceptionMessage.h"
 #include "support/filetools.h"
 #include "support/gettext.h"
+#include "support/lassert.h"
 #include "support/lstrings.h"
 #include "support/Package.h"
 #include "support/types.h"
@@ -109,6 +106,7 @@ T * getInsetByCode(Cursor const & cur, InsetCode code)
 	return 0;
 }
 
+
 /// Note that comparing contents can only be used for InsetCommand
 bool findNextInset(DocIterator & dit, vector<InsetCode> const & codes,
 	docstring const & contents)
@@ -565,7 +563,7 @@ docstring BufferView::toolTip(int x, int y) const
 }
 
 
-docstring BufferView::contextMenu(int x, int y) const
+string BufferView::contextMenu(int x, int y) const
 {
 	//If there is a selection, return the containing inset menu
 	if (d->cursor_.selection())
@@ -686,10 +684,6 @@ Change const BufferView::getCurrentChange() const
 		return Change(Change::UNCHANGED);
 
 	DocIterator dit = d->cursor_.selectionBegin();
-	// The selected content might have been changed (see #7685)
-	while (dit.inMathed())
-		// Find enclosing text cursor
-		dit.pop_back();
 	return dit.paragraph().lookupChange(dit.pos());
 }
 
@@ -882,7 +876,7 @@ bool BufferView::scrollToCursor(DocIterator const & dit, bool recenter)
 
 	if (tm.contains(bot_pit)) {
 		ParagraphMetrics const & pm = tm.parMetrics(bot_pit);
-		LASSERT(!pm.rows().empty(), /**/);
+		LBUFERR(!pm.rows().empty());
 		// FIXME: smooth scrolling doesn't work in mathed.
 		CursorSlice const & cs = dit.innerTextSlice();
 		int offset = coordOffset(dit).y_;
@@ -949,7 +943,15 @@ bool BufferView::scrollToCursor(DocIterator const & dit, bool recenter)
 }
 
 
-void BufferView::updateDocumentClass(DocumentClass const * const olddc)
+void BufferView::makeDocumentClass()
+{
+	DocumentClassConstPtr olddc = buffer_.params().documentClassPtr();
+	buffer_.params().makeDocumentClass();
+	updateDocumentClass(olddc);
+}
+
+
+void BufferView::updateDocumentClass(DocumentClassConstPtr olddc)
 {
 	message(_("Converting document to new document class..."));
 	
@@ -964,6 +966,7 @@ void BufferView::updateDocumentClass(DocumentClass const * const olddc)
 	buffer_.errors("Class Switch");
 }
 
+
 /** Return the change status at cursor position, taking in account the
  * status at each level of the document iterator (a table in a deleted
  * footnote is deleted).
@@ -1067,7 +1070,6 @@ bool BufferView::getStatus(FuncRequest const & cmd, FuncStatus & flag)
 	case LFUN_WORD_FIND:
 	case LFUN_WORD_FIND_FORWARD:
 	case LFUN_WORD_FIND_BACKWARD:
-	case LFUN_WORD_FINDADV:
 	case LFUN_WORD_REPLACE:
 	case LFUN_MARK_OFF:
 	case LFUN_MARK_ON:
@@ -1077,7 +1079,6 @@ bool BufferView::getStatus(FuncRequest const & cmd, FuncStatus & flag)
 	case LFUN_BIBTEX_DATABASE_ADD:
 	case LFUN_BIBTEX_DATABASE_DEL:
 	case LFUN_STATISTICS:
-	case LFUN_BRANCH_ADD_INSERT:
 	case LFUN_KEYMAP_OFF:
 	case LFUN_KEYMAP_PRIMARY:
 	case LFUN_KEYMAP_SECONDARY:
@@ -1085,6 +1086,14 @@ bool BufferView::getStatus(FuncRequest const & cmd, FuncStatus & flag)
 		flag.setEnabled(true);
 		break;
 
+	case LFUN_WORD_FINDADV: {
+		FindAndReplaceOptions opt;
+		istringstream iss(to_utf8(cmd.argument()));
+		iss >> opt;
+		flag.setEnabled(opt.repl_buf_name.empty()
+				|| !buffer_.isReadonly());
+	}
+
 	case LFUN_LABEL_GOTO: {
 		flag.setEnabled(!cmd.argument().empty()
 		    || getInsetByCode<InsetRef>(cur, REF_CODE));
@@ -1144,6 +1153,10 @@ bool BufferView::getStatus(FuncRequest const & cmd, FuncStatus & flag)
 		flag.setEnabled(cur.inset().allowParagraphCustomization(cur.idx()));
 		break;
 
+	case LFUN_BRANCH_ADD_INSERT:
+		flag.setEnabled(!(cur.inTexted() && cur.paragraph().isPassThru()));
+		break;
+
 	case LFUN_DIALOG_SHOW_NEW_INSET:
 		// FIXME: this is wrong, but I do not understand the
 		// intent (JMarc)
@@ -1217,7 +1230,7 @@ void BufferView::dispatch(FuncRequest const & cmd, DispatchResult & dr)
 	switch (act) {
 
 	case LFUN_BUFFER_PARAMS_APPLY: {
-		DocumentClass const * const oldClass = buffer_.params().documentClassPtr();
+		DocumentClassConstPtr olddc = buffer_.params().documentClassPtr();
 		cur.recordUndoFullDocument();
 		istringstream ss(to_utf8(cmd.argument()));
 		Lexer lex;
@@ -1228,7 +1241,7 @@ void BufferView::dispatch(FuncRequest const & cmd, DispatchResult & dr)
 						<< unknown_tokens << " unknown token"
 						<< (unknown_tokens == 1 ? "" : "s"));
 		}
-		updateDocumentClass(oldClass);
+		updateDocumentClass(olddc);
 			
 		// We are most certainly here because of a change in the document
 		// It is then better to make sure that all dialogs are in sync with
@@ -1239,12 +1252,9 @@ void BufferView::dispatch(FuncRequest const & cmd, DispatchResult & dr)
 	}
 		
 	case LFUN_LAYOUT_MODULES_CLEAR: {
-		DocumentClass const * const oldClass =
-			buffer_.params().documentClassPtr();
 		cur.recordUndoFullDocument();
 		buffer_.params().clearLayoutModules();
-		buffer_.params().makeDocumentClass();
-		updateDocumentClass(oldClass);
+		makeDocumentClass();
 		dr.screenUpdate(Update::Force);
 		dr.forceBufferUpdate();
 		break;
@@ -1252,17 +1262,15 @@ void BufferView::dispatch(FuncRequest const & cmd, DispatchResult & dr)
 
 	case LFUN_LAYOUT_MODULE_ADD: {
 		BufferParams const & params = buffer_.params();
-		if (!params.moduleCanBeAdded(argument)) {
+		if (!params.layoutModuleCanBeAdded(argument)) {
 			LYXERR0("Module `" << argument << 
 				"' cannot be added due to failed requirements or "
 				"conflicts with installed modules.");
 			break;
 		}
-		DocumentClass const * const oldClass = params.documentClassPtr();
 		cur.recordUndoFullDocument();
 		buffer_.params().addLayoutModule(argument);
-		buffer_.params().makeDocumentClass();
-		updateDocumentClass(oldClass);
+		makeDocumentClass();
 		dr.screenUpdate(Update::Force);
 		dr.forceBufferUpdate();
 		break;
@@ -1289,12 +1297,9 @@ void BufferView::dispatch(FuncRequest const & cmd, DispatchResult & dr)
 			break;
 
 		// Save the old, possibly modular, layout for use in conversion.
-		DocumentClass const * const oldDocClass =
-			buffer_.params().documentClassPtr();
 		cur.recordUndoFullDocument();
 		buffer_.params().setBaseClass(argument);
-		buffer_.params().makeDocumentClass();
-		updateDocumentClass(oldDocClass);
+		makeDocumentClass();
 		dr.screenUpdate(Update::Force);
 		dr.forceBufferUpdate();
 		break;
@@ -1315,12 +1320,10 @@ void BufferView::dispatch(FuncRequest const & cmd, DispatchResult & dr)
 	}
 
 	case LFUN_LAYOUT_RELOAD: {
-		DocumentClass const * const oldClass = buffer_.params().documentClassPtr();
 		LayoutFileIndex bc = buffer_.params().baseClassID();
 		LayoutFileList::get().reset(bc);
 		buffer_.params().setBaseClass(bc);
-		buffer_.params().makeDocumentClass();
-		updateDocumentClass(oldClass);
+		makeDocumentClass();
 		dr.screenUpdate(Update::Force);
 		dr.forceBufferUpdate();
 		break;
@@ -1513,8 +1516,14 @@ void BufferView::dispatch(FuncRequest const & cmd, DispatchResult & dr)
 		docstring const data =
 			find2string(searched_string, true, false, fw);
 		bool found = lyxfind(this, FuncRequest(LFUN_WORD_FIND, data));
-		if (found)
+		if (found) {
 			dr.screenUpdate(Update::Force | Update::FitCursor);
+			cur.dispatched();
+			dispatched = true;
+		} else {
+			cur.undispatched();
+			dispatched = false;
+		}
 		break;
 	}
 
@@ -1526,10 +1535,14 @@ void BufferView::dispatch(FuncRequest const & cmd, DispatchResult & dr)
 			lyx::dispatch(FuncRequest(LFUN_DIALOG_SHOW, "findreplace"));
 			break;
 		}
-		if (lyxfind(this, req))
+		if (lyxfind(this, req)) {
 			dr.screenUpdate(Update::Force | Update::FitCursor);
-		else
-			message(_("String not found!"));
+			cur.dispatched();
+			dispatched = true;
+		} else {
+			cur.undispatched();
+			dispatched = false;
+		}
 		d->search_request_cache_ = req;
 		break;
 	}
@@ -1551,6 +1564,11 @@ void BufferView::dispatch(FuncRequest const & cmd, DispatchResult & dr)
 		if (lyxreplace(this, cmd, has_deleted)) {
 			dr.forceBufferUpdate();
 			dr.screenUpdate(Update::Force | Update::FitCursor);
+			cur.dispatched();
+			dispatched = true;
+		} else {
+			cur.undispatched();
+			dispatched = false;
 		}
 		break;
 	}
@@ -1714,10 +1732,30 @@ void BufferView::dispatch(FuncRequest const & cmd, DispatchResult & dr)
 		break;
 	}
 
-	case LFUN_SCROLL:
-		lfunScroll(cmd);
+	case LFUN_SCROLL: {
+		string const scroll_type = cmd.getArg(0);
+		int scroll_step = 0;
+		if (scroll_type == "line")
+			scroll_step = d->scrollbarParameters_.single_step;
+		else if (scroll_type == "page")
+			scroll_step = d->scrollbarParameters_.page_step;
+		else
+			return;
+		string const scroll_quantity = cmd.getArg(1);
+		if (scroll_quantity == "up")
+			scrollUp(scroll_step);
+		else if (scroll_quantity == "down")
+			scrollDown(scroll_step);
+		else {
+			int const scroll_value = convert<int>(scroll_quantity);
+			if (scroll_value)
+				scroll(scroll_step * scroll_value);
+		}
+		buffer_.changed(true);
+		updateHoveredInset();
 		dr.forceBufferUpdate();
 		break;
+	}
 
 	case LFUN_SCREEN_UP_SELECT: {
 		// FIXME: why is the algorithm different from LFUN_SCREEN_UP?
@@ -1874,6 +1912,23 @@ void BufferView::dispatch(FuncRequest const & cmd, DispatchResult & dr)
 			arg = token(argument, '|', 0);
 			opt1 = token(argument, '|', 1);
 		}
+
+		// if our cursor is directly in front of or behind a citation inset,
+		// we will instead add the new key to it.
+		Inset * inset = cur.nextInset();
+		if (!inset || inset->lyxCode() != CITE_CODE)
+			inset = cur.prevInset();
+		if (inset && inset->lyxCode() == CITE_CODE) {
+			InsetCitation * icite = static_cast<InsetCitation *>(inset);
+			if (icite->addKey(arg)) {
+				dr.forceBufferUpdate();
+				dr.screenUpdate(Update::FitCursor | Update::SinglePar);
+				if (!opt1.empty())
+					LYXERR0("Discarding optional argument to citation-insert.");
+			}
+			dispatched = true;
+			break;
+		}
 		InsetCommandParams icp(CITE_CODE);
 		icp["key"] = from_utf8(arg);
 		if (!opt1.empty())
@@ -1905,6 +1960,11 @@ void BufferView::dispatch(FuncRequest const & cmd, DispatchResult & dr)
 		break;
 	}
 
+	// FIXME:
+	// The change of language of buffer belongs to the Buffer class.
+	// We have to do it here because we need a cursor for Undo.
+	// When Undo::recordUndoBufferParams() is implemented someday
+	// LFUN_BUFFER_LANGUAGE should be handled by the Buffer class.
 	case LFUN_BUFFER_LANGUAGE: {
 		Language const * oldL = buffer_.params().language;
 		Language const * newL = languages.getLanguage(argument);
@@ -1930,7 +1990,6 @@ void BufferView::dispatch(FuncRequest const & cmd, DispatchResult & dr)
 		break;
 	}
 
-
 	default:
 		// OK, so try the Buffer itself...
 		buffer_.dispatch(cmd, dr);
@@ -2070,7 +2129,7 @@ void BufferView::clearLastInset(Inset * inset) const
 {
 	if (d->last_inset_ != inset) {
 		LYXERR0("Wrong last_inset!");
-		LASSERT(false, /**/);
+		LATTEST(false);
 	}
 	d->last_inset_ = 0;
 }
@@ -2147,31 +2206,6 @@ void BufferView::mouseEventDispatch(FuncRequest const & cmd0)
 }
 
 
-void BufferView::lfunScroll(FuncRequest const & cmd)
-{
-	string const scroll_type = cmd.getArg(0);
-	int scroll_step = 0;
-	if (scroll_type == "line")
-		scroll_step = d->scrollbarParameters_.single_step;
-	else if (scroll_type == "page")
-		scroll_step = d->scrollbarParameters_.page_step;
-	else
-		return;
-	string const scroll_quantity = cmd.getArg(1);
-	if (scroll_quantity == "up")
-		scrollUp(scroll_step);
-	else if (scroll_quantity == "down")
-		scrollDown(scroll_step);
-	else {
-		int const scroll_value = convert<int>(scroll_quantity);
-		if (scroll_value)
-			scroll(scroll_step * scroll_value);
-	}
-	buffer_.changed(true);
-	updateHoveredInset();
-}
-
-
 int BufferView::minVisiblePart()
 {
 	return 2 * defaultRowHeight();
@@ -2327,7 +2361,7 @@ TextMetrics const & BufferView::textMetrics(Text const * t) const
 
 TextMetrics & BufferView::textMetrics(Text const * t)
 {
-	LASSERT(t, /**/);
+	LBUFERR(t);
 	TextMetricsCache::iterator tmc_it  = d->text_metrics_.find(t);
 	if (tmc_it == d->text_metrics_.end()) {
 		tmc_it = d->text_metrics_.insert(
@@ -2384,13 +2418,11 @@ bool BufferView::checkDepm(Cursor & cur, Cursor & old)
 		return false;
 
 	d->cursor_ = cur;
-
-	// we would rather not do this here, but it needs to be done before
-	// the changed() signal is sent. a riskier strategy has been tried
-	// in trunk (at r).
+	
+	// we would rather not do this here, but it needs to be done before 
+ 	// the changed() signal is sent.
 	buffer_.updateBuffer();
-	// if it was requested, we've already done it
-	cur.clearBufferUpdate();
+
 	buffer_.changed(true);
 	return true;
 }
@@ -2398,7 +2430,7 @@ bool BufferView::checkDepm(Cursor & cur, Cursor & old)
 
 bool BufferView::mouseSetCursor(Cursor & cur, bool select)
 {
-	LASSERT(&cur.bv() == this, /**/);
+	LASSERT(&cur.bv() == this, return false);
 
 	if (!select)
 		// this event will clear selection so we save selection for
@@ -2462,8 +2494,7 @@ void BufferView::putSelectionAt(DocIterator const & cur,
 
 bool BufferView::selectIfEmpty(DocIterator & cur)
 {
-	if ((cur.inTexted() && !cur.paragraph().empty())
-	    || (cur.inMathed() && !cur.cell().empty()))
+	if (!cur.paragraph().empty())
 		return false;
 
 	pit_type const beg_pit = cur.pit();
@@ -2640,7 +2671,7 @@ void BufferView::updateMetrics()
 
 void BufferView::insertLyXFile(FileName const & fname)
 {
-	LASSERT(d->cursor_.inTexted(), /**/);
+	LASSERT(d->cursor_.inTexted(), return);
 
 	// Get absolute path of file and add ".lyx"
 	// to the filename if necessary
@@ -2723,7 +2754,8 @@ Point BufferView::coordOffset(DocIterator const & dit) const
 	CursorSlice const & sl = dit[0];
 	TextMetrics const & tm = textMetrics(sl.text());
 	ParagraphMetrics const & pm = tm.parMetrics(sl.pit());
-	LASSERT(!pm.rows().empty(), /**/);
+
+	LBUFERR(!pm.rows().empty());
 	y -= pm.rows()[0].ascent();
 #if 1
 	// FIXME: document this mess
@@ -3011,7 +3043,7 @@ bool samePar(DocIterator const & a, DocIterator const & b)
 }
 
 
-void BufferView::setInlineCompletion(Cursor & cur, DocIterator const & pos, 
+void BufferView::setInlineCompletion(Cursor const & cur, DocIterator const & pos, 
 	docstring const & completion, size_t uniqueChars)
 {
 	uniqueChars = min(completion.size(), uniqueChars);
