@@ -340,6 +340,7 @@ void Changes::merge()
 
 
 namespace {
+
 docstring getLaTeXMarkup(docstring const & macro, docstring const & author,
 			 docstring const & chgTime,
 			 OutputParams const & runparams)
@@ -354,40 +355,32 @@ docstring getLaTeXMarkup(docstring const & macro, docstring const & author,
 	ods << macro;
 	// convert utf8 author name to something representable
 	// in the current encoding
-	docstring author_latexed;
-	for (size_t n = 0; n < author.size(); ++n) {
-		try {
-			author_latexed += runparams.encoding->latexChar(author[n]);
-		} catch (EncodingException & /* e */) {
-			if (runparams.dryrun) {
-				ods << "<" << _("LyX Warning: ")
-				    << _("uncodable character") << " '";
-				ods.put(author[n]);
-				ods << "'>";
-			} else {
-				LYXERR0("Omitting uncodable character '"
-					<< docstring(1, author[n])
-					<< "' in change author name!");
-				uncodable_author = author;
-			}
-		}
+	pair<docstring, docstring> author_latexed =
+		runparams.encoding->latexString(author, runparams.dryrun);
+	if (!author_latexed.second.empty()) {
+		LYXERR0("Omitting uncodable characters '"
+			<< author_latexed.second
+			<< "' in change author name!");
+		uncodable_author = author;
 	}
-	ods << author_latexed << "}{" << chgTime << "}{";
+	ods << author_latexed.first << "}{" << chgTime << "}{";
 
 	// warn user (once) if we found uncodable glyphs.
 	if (uncodable_author != warned_author) {
 		frontend::Alert::warning(_("Uncodable character in author name"),
 				support::bformat(_("The author name '%1$s',\n"
-				  "used for change tracking, contains glyphs that cannot be\n"
-				  "represented in the current encoding. The respective glyphs\n"
-				  "will be omitted in the exported LaTeX file.\n\n"
+				  "used for change tracking, contains the following glyphs that\n"
+				  "cannot be represented in the current encoding: %2$s.\n"
+				  "These glyphs will be omitted in the exported LaTeX file.\n\n"
 				  "Choose an appropriate document encoding (such as utf8)\n"
-				  "or change the spelling of the author name."), uncodable_author));
+				  "or change the spelling of the author name."),
+				uncodable_author, author_latexed.second));
 		warned_author = uncodable_author;
 	}
 
 	return ods.str();
 }
+
 } //namespace anon
 
 
@@ -407,7 +400,7 @@ int Changes::latexMarkChange(otexstream & os, BufferParams const & bparams,
 	}
 
 	docstring chgTime;
-	chgTime += ctime(&change.changetime);
+	chgTime += asctime(gmtime(&change.changetime));
 	// remove trailing '\n'
 	chgTime.erase(chgTime.end() - 1);
 
@@ -466,7 +459,8 @@ void Changes::checkAuthors(AuthorList const & authorList)
 }
 
 
-void Changes::addToToc(DocIterator const & cdit, Buffer const & buffer) const
+void Changes::addToToc(DocIterator const & cdit, Buffer const & buffer,
+        bool output_active) const
 {
 	if (table_.empty())
 		return;
@@ -488,8 +482,7 @@ void Changes::addToToc(DocIterator const & cdit, Buffer const & buffer) const
 			break;
 		case Change::INSERTED:
 			// 0x270d is the hand writting symbol in the Dingbats unicode group.
-			str.push_back(0x270d);
-			break;
+			str.push_back(0x270d); break;
 		}
 		dit.pos() = it->range.start;
 		Paragraph const & par = dit.paragraph();
@@ -500,8 +493,8 @@ void Changes::addToToc(DocIterator const & cdit, Buffer const & buffer) const
 		docstring const & author = author_list.get(it->change.author).name();
 		Toc::iterator it = change_list.item(0, author);
 		if (it == change_list.end()) {
-			change_list.push_back(TocItem(dit, 0, author));
-			change_list.push_back(TocItem(dit, 1, str,
+			change_list.push_back(TocItem(dit, 0, author, output_active));
+			change_list.push_back(TocItem(dit, 1, str, output_active,
 				support::wrapParas(str, 4)));
 			continue;
 		}
@@ -509,7 +502,7 @@ void Changes::addToToc(DocIterator const & cdit, Buffer const & buffer) const
 			if (it->depth() == 0 && it->str() != author)
 				break;
 		}
-		change_list.insert(it, TocItem(dit, 1, str,
+		change_list.insert(it, TocItem(dit, 1, str, output_active,
 			support::wrapParas(str, 4)));
 	}
 }
