@@ -14,15 +14,10 @@
 
 #include "Encoding.h"
 
-#include "Buffer.h"
-#include "BufferList.h"
-#include "InsetIterator.h"
-#include "LaTeXFeatures.h"
 #include "Lexer.h"
-#include "LyXRC.h"
 
 #include "support/debug.h"
-#include "support/FileName.h"
+#include "support/gettext.h"
 #include "support/lstrings.h"
 #include "support/textutils.h"
 #include "support/unicode.h"
@@ -36,6 +31,8 @@ using namespace lyx::support;
 
 namespace lyx {
 
+int const Encoding::any = -1;
+
 Encodings encodings;
 
 Encodings::MathCommandSet Encodings::mathcmd;
@@ -142,7 +139,7 @@ char_type arabic_table[172][4] = {
 	{0, 0, 0, 0}, // 0x067b
 	{0, 0, 0, 0}, // 0x067c
 	{0, 0, 0, 0}, // 0x067d
-	{0xfb56, 0xfb57, 0xfb58, 0xfb59}, // 0x067e = peh 
+	{0xfb56, 0xfb57, 0xfb58, 0xfb59}, // 0x067e = peh
 	{0, 0, 0, 0}, // 0x067f
 	{0, 0, 0, 0}, // 0x0680
 	{0, 0, 0, 0}, // 0x0681
@@ -150,7 +147,7 @@ char_type arabic_table[172][4] = {
 	{0, 0, 0, 0}, // 0x0683
 	{0, 0, 0, 0}, // 0x0684
 	{0, 0, 0, 0}, // 0x0685
-	{0xfb7a, 0xfb7b, 0xfb7c, 0xfb7d}, // 0x0686 = tcheh 
+	{0xfb7a, 0xfb7b, 0xfb7c, 0xfb7d}, // 0x0686 = tcheh
 	{0, 0, 0, 0}, // 0x0687
 	{0, 0, 0, 0}, // 0x0688
 	{0, 0, 0, 0}, // 0x0689
@@ -185,13 +182,13 @@ char_type arabic_table[172][4] = {
 	{0, 0, 0, 0}, // 0x06a6
 	{0, 0, 0, 0}, // 0x06a7
 	{0, 0, 0, 0}, // 0x06a8
-	{0xfb8e, 0xfb8f, 0xfb90, 0xfb91}, // 0x06a9 = farsi kaf 
+	{0xfb8e, 0xfb8f, 0xfb90, 0xfb91}, // 0x06a9 = farsi kaf
 	{0, 0, 0, 0}, // 0x06aa
 	{0, 0, 0, 0}, // 0x06ab
 	{0, 0, 0, 0}, // 0x06ac
 	{0, 0, 0, 0}, // 0x06ad
 	{0, 0, 0, 0}, // 0x06ae
-	{0xfb92, 0xfb93, 0xfb94, 0xfb95}, // 0x06af = gaf 
+	{0xfb92, 0xfb93, 0xfb94, 0xfb95}, // 0x06af = gaf
 	{0, 0, 0, 0}, // 0x06b0
 	{0, 0, 0, 0}, // 0x06b1
 	{0, 0, 0, 0}, // 0x06b2
@@ -220,7 +217,7 @@ char_type arabic_table[172][4] = {
 	{0, 0, 0, 0}, // 0x06c9
 	{0, 0, 0, 0}, // 0x06ca
 	{0, 0, 0, 0}, // 0x06cb
-	{0xfbfc, 0xfbfd, 0xfbfe, 0xfbff} // 0x06cc = farsi yeh	
+	{0xfbfc, 0xfbfd, 0xfbfe, 0xfbff} // 0x06cc = farsi yeh
 };
 
 
@@ -228,37 +225,15 @@ char_type const arabic_start = 0x0621;
 char_type const arabic_end = 0x06cc;
 
 
-/// Information about a single UCS4 character
-struct CharInfo {
-	/// LaTeX command (text mode) for this character
-	docstring textcommand;
-	/// LaTeX command (math mode) for this character
-	docstring mathcommand;
-	/// Needed LaTeX preamble (or feature) for text mode
-	string textpreamble;
-	/// Needed LaTeX preamble (or feature) for math mode
-	string mathpreamble;
-	/// Is this a combining character?
-	bool combining;
-	/// Is \c textpreamble a feature known by LaTeXFeatures, or a raw LaTeX
-	/// command?
-	bool textfeature;
-	/// Is \c mathpreamble a feature known by LaTeXFeatures, or a raw LaTeX
-	/// command?
-	bool mathfeature;
-	/// Always force the LaTeX command, even if the encoding contains
-	/// this character?
-	bool force;
-};
-
-
 typedef map<char_type, CharInfo> CharInfoMap;
 CharInfoMap unicodesymbols;
 
-typedef std::set<char_type> CharSet;
+typedef set<char_type> CharSet;
+typedef map<string, CharSet> CharSetMap;
 CharSet forced;
+CharSetMap forcedselected;
 
-typedef std::set<char_type> MathAlphaSet;
+typedef set<char_type> MathAlphaSet;
 MathAlphaSet mathalpha;
 
 
@@ -280,9 +255,20 @@ const char * EncodingException::what() const throw()
 }
 
 
+CharInfo::CharInfo(
+	docstring const textcommand, docstring const mathcommand,
+	std::string const textpreamble, std::string const mathpreamble,
+	std::string const tipashortcut, unsigned int flags)
+	: textcommand_(textcommand), mathcommand_(mathcommand),
+	  textpreamble_(textpreamble), mathpreamble_(mathpreamble),
+	  tipashortcut_(tipashortcut), flags_(flags)
+{
+}
+
 Encoding::Encoding(string const & n, string const & l, string const & g,
-		   string const & i, bool f, Encoding::Package p)
-	: name_(n), latexName_(l), guiName_(g), iconvName_(i), fixedwidth_(f), package_(p)
+		   string const & i, bool f, bool u, Encoding::Package p)
+	: name_(n), latexName_(l), guiName_(g), iconvName_(i), fixedwidth_(f),
+	  unsafe_(u), forced_(&forcedselected[n]), package_(p)
 {
 	if (n == "ascii") {
 		// ASCII can encode 128 code points and nothing else
@@ -293,6 +279,7 @@ Encoding::Encoding(string const & n, string const & l, string const & g,
 		start_encodable_ = max_ucs4;
 		complete_ = true;
 	} else {
+		start_encodable_ = 0;
 		complete_ = false;
 	}
 }
@@ -316,8 +303,12 @@ void Encoding::init() const
 				continue;
 			char_type const uc = ucs4[0];
 			CharInfoMap::const_iterator const it = unicodesymbols.find(uc);
-			if (it == unicodesymbols.end() || !it->second.force)
+			if (it == unicodesymbols.end())
 				encodable_.insert(uc);
+			else if (!it->second.force()) {
+				if (forced_->empty() || forced_->find(uc) == forced_->end())
+					encodable_.insert(uc);
+			}
 		}
 	} else {
 		// We do not know how many code points this encoding has, and
@@ -328,8 +319,12 @@ void Encoding::init() const
 			vector<char> const eightbit = ucs4_to_eightbit(&c, 1, iconvName_);
 			if (!eightbit.empty()) {
 				CharInfoMap::const_iterator const it = unicodesymbols.find(c);
-				if (it == unicodesymbols.end() || !it->second.force)
+				if (it == unicodesymbols.end())
 					encodable_.insert(c);
+				else if (!it->second.force()) {
+					if (forced_->empty() || forced_->find(c) == forced_->end())
+						encodable_.insert(c);
+				}
 			}
 		}
 	}
@@ -344,28 +339,81 @@ void Encoding::init() const
 }
 
 
-docstring Encoding::latexChar(char_type c, bool no_commands) const
+bool Encoding::isForced(char_type c) const
+{
+	if (!forced.empty() && forced.find(c) != forced.end())
+		return true;
+	return !forced_->empty() && forced_->find(c) != forced_->end();
+}
+
+
+bool Encoding::encodable(char_type c) const
 {
 	// assure the used encoding is properly initialized
 	init();
 
 	if (iconvName_ == "UTF-8" && package_ == none)
-		return docstring(1, c);
-	if (c < start_encodable_ && !encodings.isForced(c))
-		return docstring(1, c);
+		return true;
+	if (c < start_encodable_ && !isForced(c))
+		return true;
 	if (encodable_.find(c) != encodable_.end())
-		return docstring(1, c);
-	if (no_commands)
-		return docstring();
+		return true;
+	return false;
+}
+
+
+pair<docstring, bool> Encoding::latexChar(char_type c) const
+{
+	if (encodable(c))
+		return make_pair(docstring(1, c), false);
 
 	// c cannot (or should not) be encoded in this encoding
 	CharInfoMap::const_iterator const it = unicodesymbols.find(c);
 	if (it == unicodesymbols.end())
 		throw EncodingException(c);
 	// at least one of mathcommand and textcommand is nonempty
-	if (it->second.textcommand.empty())
-		return "\\ensuremath{" + it->second.mathcommand + '}';
-	return it->second.textcommand;
+	if (it->second.textcommand().empty())
+		return make_pair(
+			"\\ensuremath{" + it->second.mathcommand() + '}', false);
+	return make_pair(it->second.textcommand(), !it->second.textnotermination());
+}
+
+
+pair<docstring, docstring> Encoding::latexString(docstring const input, bool dryrun) const
+{
+	docstring result;
+	docstring uncodable;
+	bool terminate = false;
+	for (size_t n = 0; n < input.size(); ++n) {
+		try {
+			char_type const c = input[n];
+			pair<docstring, bool> latex_char = latexChar(c);
+			docstring const latex = latex_char.first;
+			if (terminate && !prefixIs(latex, '\\')
+			    && !prefixIs(latex, '{')
+			    && !prefixIs(latex, '}')) {
+					// Prevent eating of a following
+					// space or command corruption by
+					// following characters
+					if (latex == " ")
+						result += "{}";
+					else
+						result += " ";
+				}
+			result += latex;
+			terminate = latex_char.second;
+		} catch (EncodingException & /* e */) {
+			LYXERR0("Uncodable character in latexString!");
+			if (dryrun) {
+				result += "<" + _("LyX Warning: ")
+					   + _("uncodable character") + " '";
+				result += docstring(1, input[n]);
+				result += "'>";
+			} else
+				uncodable += input[n];
+		}
+	}
+	return make_pair(result, uncodable);
 }
 
 
@@ -389,10 +437,14 @@ vector<char_type> Encoding::symbolsList() const
 
 
 bool Encodings::latexMathChar(char_type c, bool mathmode,
-			Encoding const * encoding, docstring & command)
+			Encoding const * encoding, docstring & command,
+			bool & needsTermination)
 {
+	command = empty_docstring();
 	if (encoding)
-		command = encoding->latexChar(c, true);
+		if (encoding->encodable(c))
+			command = docstring(1, c);
+	needsTermination = false;
 
 	CharInfoMap::const_iterator const it = unicodesymbols.find(c);
 	if (it == unicodesymbols.end()) {
@@ -403,14 +455,16 @@ bool Encodings::latexMathChar(char_type c, bool mathmode,
 		return false;
 	}
 	// at least one of mathcommand and textcommand is nonempty
-	bool use_math = (mathmode && !it->second.mathcommand.empty()) ||
-			(!mathmode && it->second.textcommand.empty());
+	bool use_math = (mathmode && !it->second.mathcommand().empty()) ||
+			(!mathmode && it->second.textcommand().empty());
 	if (use_math) {
-		command = it->second.mathcommand;
+		command = it->second.mathcommand();
+		needsTermination = !it->second.mathnotermination();
 		addMathCmd(c);
 	} else {
 		if (!encoding || command.empty()) {
-			command = it->second.textcommand;
+			command = it->second.textcommand();
+			needsTermination = !it->second.textnotermination();
 			addTextCmd(c);
 		}
 		if (mathmode)
@@ -420,36 +474,66 @@ bool Encodings::latexMathChar(char_type c, bool mathmode,
 }
 
 
-char_type Encodings::fromLaTeXCommand(docstring const & cmd, bool & combining)
+char_type Encodings::fromLaTeXCommand(docstring const & cmd, int cmdtype,
+		bool & combining, bool & needsTermination, set<string> * req)
 {
 	CharInfoMap::const_iterator const end = unicodesymbols.end();
 	CharInfoMap::const_iterator it = unicodesymbols.begin();
 	for (combining = false; it != end; ++it) {
-		docstring const math = it->second.mathcommand;
-		docstring const text = it->second.textcommand;
-		if (math == cmd || text == cmd) {
-			combining = it->second.combining;
+		docstring const math = it->second.mathcommand();
+		docstring const text = it->second.textcommand();
+		if ((cmdtype & MATH_CMD) && math == cmd) {
+			combining = it->second.combining();
+			needsTermination = !it->second.mathnotermination();
+			if (req && it->second.mathfeature() &&
+			    !it->second.mathpreamble().empty())
+				req->insert(it->second.mathpreamble());
+			return it->first;
+		}
+		if ((cmdtype & TEXT_CMD) && text == cmd) {
+			combining = it->second.combining();
+			needsTermination = !it->second.textnotermination();
+			if (req && it->second.textfeature() &&
+			    !it->second.textpreamble().empty())
+				req->insert(it->second.textpreamble());
 			return it->first;
 		}
 	}
+	needsTermination = false;
 	return 0;
 }
 
 
-docstring Encodings::fromLaTeXCommand(docstring const & cmd, docstring & rem,
-				      int cmdtype)
+docstring Encodings::fromLaTeXCommand(docstring const & cmd, int cmdtype,
+		bool & needsTermination, docstring & rem, set<string> * req)
 {
+	needsTermination = false;
+	rem = empty_docstring();
 	bool const mathmode = cmdtype & MATH_CMD;
 	bool const textmode = cmdtype & TEXT_CMD;
 	docstring symbols;
-	size_t i = 0;
 	size_t const cmdend = cmd.size();
+	size_t prefix = 0;
 	CharInfoMap::const_iterator const uniend = unicodesymbols.end();
-	for (size_t j = 0; j < cmdend; ++j) {
+	for (size_t i = 0, j = 0; j < cmdend; ++j) {
 		// Also get the char after a backslash
-		if (j + 1 < cmdend && cmd[j] == '\\')
+		if (j + 1 < cmdend && cmd[j] == '\\') {
 			++j;
+			prefix = 1;
+			// Detect things like \=*{e} as well
+			if (j + 3 < cmdend && cmd[j+1] == '*' &&
+			    cmd[j+2] == '{') {
+				++j;
+				prefix = 2;
+			}
+		}
+		// position of the last character before a possible macro
+		// argument
+		size_t m = j;
 		// If a macro argument follows, get it, too
+		// Do it here only for single character commands. Other
+		// combining commands need this too, but they are handled in
+		// the loop below for performance reasons.
 		if (j + 1 < cmdend && cmd[j + 1] == '{') {
 			size_t k = j + 1;
 			int count = 1;
@@ -462,19 +546,29 @@ docstring Encodings::fromLaTeXCommand(docstring const & cmd, docstring & rem,
 			}
 			if (k != docstring::npos)
 				j = k;
+		} else if (m + 1 < cmdend && isAlphaASCII(cmd[m])) {
+			while (m + 2 < cmdend && isAlphaASCII(cmd[m+1]))
+				m++;
 		}
 		// Start with this substring and try augmenting it when it is
 		// the prefix of some command in the unicodesymbols file
-		docstring const subcmd = cmd.substr(i, j - i + 1);
+		docstring subcmd = cmd.substr(i, j - i + 1);
 
 		CharInfoMap::const_iterator it = unicodesymbols.begin();
+		// First part of subcmd which might be a combining character
+		docstring combcmd = (m == j) ? docstring() : cmd.substr(i, m - i + 1);
+		// The combining character of combcmd if it exists
+		CharInfoMap::const_iterator combining = uniend;
 		size_t unicmd_size = 0;
 		char_type c = 0;
 		for (; it != uniend; ++it) {
-			docstring const math = mathmode ? it->second.mathcommand
+			docstring const math = mathmode ? it->second.mathcommand()
 							: docstring();
-			docstring const text = textmode ? it->second.textcommand
+			docstring const text = textmode ? it->second.textcommand()
 							: docstring();
+			if (!combcmd.empty() && it->second.combining() &&
+			    (math == combcmd || text == combcmd))
+				combining = it;
 			size_t cur_size = max(math.size(), text.size());
 			// The current math or text unicode command cannot
 			// match, or we already matched a longer one
@@ -501,6 +595,26 @@ docstring Encodings::fromLaTeXCommand(docstring const & cmd, docstring & rem,
 
 			// If this is an exact match, we found a (longer)
 			// matching entry in the unicodesymbols file.
+			if (math != tmp && text != tmp)
+				continue;
+			// If we found a combining command, we need to append
+			// the macro argument if this has not been done above.
+			if (tmp == combcmd && combining != uniend &&
+			    k < cmdend && cmd[k] == '{') {
+				size_t l = k;
+				int count = 1;
+				while (l < cmdend && count && l != docstring::npos) {
+					l = cmd.find_first_of(from_ascii("{}"), l + 1);
+					if (cmd[l] == '{')
+						++count;
+					else
+						--count;
+				}
+				if (l != docstring::npos) {
+					j = l;
+					subcmd = cmd.substr(i, j - i + 1);
+				}
+			}
 			// If the entry doesn't start with '\', we take note
 			// of the match and continue (this is not a ultimate
 			// acceptance, as some other entry may match a longer
@@ -510,116 +624,86 @@ docstring Encodings::fromLaTeXCommand(docstring const & cmd, docstring & rem,
 			// (nonletter) char macro, or nothing else follows,
 			// or what follows is a nonletter char, or the last
 			// character is a }.
-			if ((math == tmp || text == tmp)
-			    && (tmp[0] != '\\'
-				   || (tmp.size() == 2 && !isAlphaASCII(tmp[1]))
+			else if (tmp[0] != '\\'
+				   || (tmp.size() == prefix + 1 &&
+				       !isAlphaASCII(tmp[1]) &&
+				       (prefix == 1 || !isAlphaASCII(tmp[2])))
 				   || k == cmdend 
 				   || !isAlphaASCII(cmd[k])
-				   || tmp[tmp.size() - 1] == '}')
+				   || tmp[tmp.size() - 1] == '}'
 				 ) {
 				c = it->first;
 				j = k - 1;
 				i = j + 1;
 				unicmd_size = cur_size;
+				if (math == tmp)
+					needsTermination = !it->second.mathnotermination();
+				else
+					needsTermination = !it->second.textnotermination();
+				if (req) {
+					if (math == tmp && it->second.mathfeature() &&
+					    !it->second.mathpreamble().empty())
+						req->insert(it->second.mathpreamble());
+					if (text == tmp && it->second.textfeature() &&
+					    !it->second.textpreamble().empty())
+						req->insert(it->second.textpreamble());
+				}
 			}
 		}
 		if (unicmd_size)
 			symbols += c;
-		else if (j + 1 == cmdend)
+		else if (combining != uniend &&
+		         prefixIs(subcmd, combcmd + '{')) {
+			// We know that subcmd starts with combcmd and
+			// contains an argument in braces.
+			docstring const arg = subcmd.substr(
+				combcmd.length() + 1,
+				subcmd.length() - combcmd.length() - 2);
+			// If arg is a single character we can construct a
+			// combining sequence.
+			char_type a;
+			bool argcomb = false;
+			if (arg.size() == 1 && isAlnumASCII(arg[0]))
+				a = arg[0];
+			else {
+				// Use the version of fromLaTeXCommand() that
+				// parses only one command, since we cannot
+				// use more than one character.
+				bool dummy = false;
+				set<string> r;
+				a = fromLaTeXCommand(arg, cmdtype, argcomb,
+				                     dummy, &r);
+				if (a && req && !argcomb)
+					req->insert(r.begin(), r.end());
+			}
+			if (a && !argcomb) {
+				// In unicode the combining character comes
+				// after its base
+				symbols += a;
+				symbols += combining->first;
+				i = j + 1;
+				unicmd_size = 2;
+			}
+		}
+		if (j + 1 == cmdend && !unicmd_size) {
 			// No luck. Return what remains
 			rem = cmd.substr(i);
+			if (needsTermination && !rem.empty()) {
+				if (rem.substr(0, 2) == "{}") {
+					rem = rem.substr(2);
+					needsTermination = false;
+				} else if (rem[0] == ' ') {
+					needsTermination = false;
+					// LaTeX would swallow all spaces
+					rem = ltrim(rem);
+				}
+			}
+		}
 	}
 	return symbols;
 }
 
 
-void Encodings::initUnicodeMath(Buffer const & buffer, bool for_master)
-{
-#ifdef TEX2LYX
-	// The code below is not needed in tex2lyx and requires additional stuff
-	(void)buffer;
-	(void)for_master;
-#else
-	if (for_master) {
-		mathcmd.clear();
-		textcmd.clear();
-		mathsym.clear();
-	}
-
-	// Check this buffer
-	Inset & inset = buffer.inset();
-	InsetIterator it = inset_iterator_begin(inset);
-	InsetIterator const end = inset_iterator_end(inset);
-	for (; it != end; ++it)
-		it->initUnicodeMath();
-
-	if (!for_master)
-		return;
-
-	// Check children
-	ListOfBuffers blist = buffer.getDescendents();
-	ListOfBuffers::const_iterator bit = blist.begin();
-	ListOfBuffers::const_iterator const bend = blist.end();
-	for (; bit != bend; ++bit)
-		initUnicodeMath(**bit, false);
-#endif
-}
-
-
-void Encodings::validate(char_type c, LaTeXFeatures & features, bool for_mathed)
-{
-#ifdef TEX2LYX
-	// The code below is not needed in tex2lyx and requires additional stuff
-	(void)c;
-	(void)features;
-	(void)for_mathed;
-#else
-	CharInfoMap::const_iterator const it = unicodesymbols.find(c);
-	if (it != unicodesymbols.end()) {
-		// In mathed, c could be used both in textmode and mathmode
-		bool const math_mode = for_mathed && isMathCmd(c);
-		bool const use_math = math_mode ||
-		                      (!for_mathed && it->second.textcommand.empty());
-		bool const use_text = (for_mathed && isTextCmd(c)) ||
-		                      (!for_mathed && !it->second.textcommand.empty());
-		bool const plain_utf8 = (features.runparams().encoding->name() == "utf8-plain");
-		// with utf8-plain, we only load packages when in mathed (see #7766)
-		if (math_mode || (use_math && !plain_utf8)) {
-			if (!it->second.mathpreamble.empty()) {
-				if (it->second.mathfeature) {
-					string feats = it->second.mathpreamble;
-					while (!feats.empty()) {
-						string feat;
-						feats = split(feats, feat, ',');
-						features.require(feat);
-					}
-				} else
-					features.addPreambleSnippet(it->second.mathpreamble);
-			}
-		}
-		// with utf8-plain, we do not load packages (see #7766)
-		if (use_text && !plain_utf8) {
-			if (!it->second.textpreamble.empty()) {
-				if (it->second.textfeature) {
-					string feats = it->second.textpreamble;
-					while (!feats.empty()) {
-						string feat;
-						feats = split(feats, feat, ',');
-						features.require(feat);
-					}
-				} else
-					features.addPreambleSnippet(it->second.textpreamble);
-			}
-		}
-	}
-	if (for_mathed && isMathSym(c)) {
-		features.require("amstext");
-		features.require("lyxmathsym");
-	}
-#endif
-}
-
-
 bool Encodings::isHebrewComposeChar(char_type c)
 {
 	return c <= 0x05c2 && c >= 0x05b0 && c != 0x05be && c != 0x05c0;
@@ -651,6 +735,14 @@ bool Encodings::isArabicChar(char_type c)
 }
 
 
+CharInfo const & Encodings::unicodeCharInfo(char_type c)
+{
+	static CharInfo empty;
+	CharInfoMap::const_iterator const it = unicodesymbols.find(c);
+	return it != unicodesymbols.end() ? it->second : empty;
+}
+
+
 char_type Encodings::transformChar(char_type c, Encodings::LetterForm form)
 {
 	return isArabicChar(c) ? arabic_table[c-arabic_start][form] : c;
@@ -661,11 +753,20 @@ bool Encodings::isCombiningChar(char_type c)
 {
 	CharInfoMap::const_iterator const it = unicodesymbols.find(c);
 	if (it != unicodesymbols.end())
-		return it->second.combining;
+		return it->second.combining();
 	return false;
 }
 
 
+string const Encodings::TIPAShortcut(char_type c)
+{
+	CharInfoMap::const_iterator const it = unicodesymbols.find(c);
+	if (it != unicodesymbols.end())
+		return it->second.tipashortcut();
+	return string();
+}
+
+
 bool Encodings::isKnownScriptChar(char_type const c, string & preamble)
 {
 	CharInfoMap::const_iterator const it = unicodesymbols.find(c);
@@ -673,20 +774,14 @@ bool Encodings::isKnownScriptChar(char_type const c, string & preamble)
 	if (it == unicodesymbols.end())
 		return false;
 
-	if (it->second.textpreamble != "textgreek" && it->second.textpreamble != "textcyr")
+	if (it->second.textpreamble() != "textgreek" && it->second.textpreamble() != "textcyr")
 		return false;
 
 	if (preamble.empty()) {
-		preamble = it->second.textpreamble;
+		preamble = it->second.textpreamble();
 		return true;
 	}
-	return it->second.textpreamble == preamble;
-}
-
-
-bool Encodings::isForced(char_type c)
-{
-	return (!forced.empty() && forced.find(c) != forced.end());
+	return it->second.textpreamble() == preamble;
 }
 
 
@@ -696,14 +791,18 @@ bool Encodings::isMathAlpha(char_type c)
 }
 
 
-Encoding const * Encodings::fromLyXName(string const & name) const
+Encoding const *
+Encodings::fromLyXName(string const & name, bool allowUnsafe) const
 {
 	EncodingList::const_iterator const it = encodinglist.find(name);
+	if (!allowUnsafe && it->second.unsafe())
+		return 0;
 	return it != encodinglist.end() ? &it->second : 0;
 }
 
 
-Encoding const * Encodings::fromLaTeXName(string const & n) const
+Encoding const *
+Encodings::fromLaTeXName(string const & n, int const & p, bool allowUnsafe) const
 {
 	string name = n;
 	// FIXME: if we have to test for too many of these synonyms,
@@ -718,28 +817,39 @@ Encoding const * Encodings::fromLaTeXName(string const & n) const
 	// most at the top of lib/encodings.
 	EncodingList::const_iterator const end = encodinglist.end();
 	for (EncodingList::const_iterator it = encodinglist.begin(); it != end; ++it)
-		if (it->second.latexName() == name)
+		if ((it->second.latexName() == name) && (it->second.package() & p)
+				&& (!it->second.unsafe() || allowUnsafe))
+			return &it->second;
+	return 0;
+}
+
+
+Encoding const *
+Encodings::fromIconvName(string const & n, int const & p, bool allowUnsafe) const
+{
+	EncodingList::const_iterator const end = encodinglist.end();
+	for (EncodingList::const_iterator it = encodinglist.begin(); it != end; ++it)
+		if ((it->second.iconvName() == n) && (it->second.package() & p)
+				&& (!it->second.unsafe() || allowUnsafe))
 			return &it->second;
 	return 0;
 }
 
 
 Encodings::Encodings()
-{
-}
+{}
 
 
 void Encodings::read(FileName const & encfile, FileName const & symbolsfile)
 {
 	// We must read the symbolsfile first, because the Encoding
 	// constructor depends on it.
+	CharSetMap forcednotselected;
 	Lexer symbolslex;
 	symbolslex.setFile(symbolsfile);
 	bool getNextToken = true;
 	while (symbolslex.isOK()) {
 		char_type symbol;
-		CharInfo info;
-		string flags;
 
 		if (getNextToken) {
 			if (!symbolslex.next(true))
@@ -757,27 +867,53 @@ void Encodings::read(FileName const & encfile, FileName const & symbolsfile)
 
 		if (!symbolslex.next(true))
 			break;
-		info.textcommand = symbolslex.getDocString();
+		docstring textcommand = symbolslex.getDocString();
 		if (!symbolslex.next(true))
 			break;
-		info.textpreamble = symbolslex.getString();
+		string textpreamble = symbolslex.getString();
 		if (!symbolslex.next(true))
 			break;
-		flags = symbolslex.getString();
+		string sflags = symbolslex.getString();
+		
+		string tipashortcut;
+		int flags = 0;
 
-		info.combining = false;
-		info.textfeature = false;
-		info.force = false;
-		while (!flags.empty()) {
+		if (suffixIs(textcommand, '}'))
+			flags |= CharInfoTextNoTermination;
+		while (!sflags.empty()) {
 			string flag;
-			flags = split(flags, flag, ',');
+			sflags = split(sflags, flag, ',');
 			if (flag == "combining") {
-				info.combining = true;
+				flags |= CharInfoCombining;
 			} else if (flag == "force") {
-				info.force = true;
+				flags |= CharInfoForce;
 				forced.insert(symbol);
+			} else if (prefixIs(flag, "force=")) {
+				vector<string> encodings =
+					getVectorFromString(flag.substr(6), ";");
+				for (size_t i = 0; i < encodings.size(); ++i)
+					forcedselected[encodings[i]].insert(symbol);
+				flags |= CharInfoForceSelected;
+			} else if (prefixIs(flag, "force!=")) {
+				vector<string> encodings =
+					getVectorFromString(flag.substr(7), ";");
+				for (size_t i = 0; i < encodings.size(); ++i)
+					forcednotselected[encodings[i]].insert(symbol);
+				flags |= CharInfoForceSelected;
 			} else if (flag == "mathalpha") {
 				mathalpha.insert(symbol);
+			} else if (flag == "notermination=text") {
+				flags |= CharInfoTextNoTermination;
+			} else if (flag == "notermination=math") {
+				flags |= CharInfoMathNoTermination;
+			} else if (flag == "notermination=both") {
+				flags |= CharInfoTextNoTermination;
+				flags |= CharInfoMathNoTermination;
+			} else if (flag == "notermination=none") {
+				flags &= ~CharInfoTextNoTermination;
+				flags &= ~CharInfoMathNoTermination;
+			} else if (contains(flag, "tipashortcut=")) {
+				tipashortcut = split(flag, '=');
 			} else {
 				lyxerr << "Ignoring unknown flag `" << flag
 				       << "' for symbol `0x"
@@ -789,19 +925,23 @@ void Encodings::read(FileName const & encfile, FileName const & symbolsfile)
 		// make them optional so that old files still work.
 		int const lineno = symbolslex.lineNumber();
 		bool breakout = false;
+		docstring mathcommand;
+		string mathpreamble;
 		if (symbolslex.next(true)) {
 			if (symbolslex.lineNumber() != lineno) {
 				// line in old format without mathcommand and mathpreamble
 				getNextToken = false;
 			} else {
-				info.mathcommand = symbolslex.getDocString();
+				mathcommand = symbolslex.getDocString();
+				if (suffixIs(mathcommand, '}'))
+					flags |= CharInfoMathNoTermination;
 				if (symbolslex.next(true)) {
 					if (symbolslex.lineNumber() != lineno) {
 						// line in new format with mathcommand only
 						getNextToken = false;
 					} else {
 						// line in new format with mathcommand and mathpreamble
-						info.mathpreamble = symbolslex.getString();
+						mathpreamble = symbolslex.getString();
 					}
 				} else
 					breakout = true;
@@ -811,23 +951,32 @@ void Encodings::read(FileName const & encfile, FileName const & symbolsfile)
 		}
 
 		// backward compatibility
-		if (info.mathpreamble == "esintoramsmath")
-			info.mathpreamble = "esint|amsmath";
+		if (mathpreamble == "esintoramsmath")
+			mathpreamble = "esint|amsmath";
 
-		if (!info.textpreamble.empty())
-			info.textfeature = info.textpreamble[0] != '\\';
-		if (!info.mathpreamble.empty())
-			info.mathfeature = info.mathpreamble[0] != '\\';
+		if (!textpreamble.empty())
+			if (textpreamble[0] != '\\')
+				flags |= CharInfoTextFeature;
+		if (!mathpreamble.empty())
+			if (mathpreamble[0] != '\\')
+				flags |= CharInfoMathFeature;
 
+		CharInfo info = CharInfo(
+			textcommand, mathcommand,
+			textpreamble, mathpreamble,
+			tipashortcut, flags);
 		LYXERR(Debug::INFO, "Read unicode symbol " << symbol << " '"
-			<< to_utf8(info.textcommand) << "' '" << info.textpreamble
-			<< "' " << info.combining << ' ' << info.textfeature
-			<< " '" << to_utf8(info.mathcommand) << "' '"
-			<< info.mathpreamble << "' " << info.mathfeature);
+			   << to_utf8(info.textcommand()) << "' '" << info.textpreamble()
+			   << " '" << info.textfeature() << ' ' << info.textnotermination()
+			   << ' ' << to_utf8(info.mathcommand()) << "' '" << info.mathpreamble()
+			   << "' " << info.mathfeature() << ' ' << info.mathnotermination()
+			   << ' ' << info.combining() << ' ' << info.force()
+			   << ' ' << info.forceselected());
 
 		// we assume that at least one command is nonempty when using unicodesymbols
-		if (!info.textcommand.empty() || !info.mathcommand.empty())
+		if (info.isUnicodeSymbol()) {
 			unicodesymbols[symbol] = info;
+		}
 
 		if (breakout)
 			break;
@@ -862,10 +1011,15 @@ void Encodings::read(FileName const & encfile, FileName const & symbolsfile)
 			lex.next();
 			string const width = lex.getString();
 			bool fixedwidth = false;
+			bool unsafe = false;
 			if (width == "fixed")
 				fixedwidth = true;
 			else if (width == "variable")
 				fixedwidth = false;
+			else if (width == "variableunsafe") {
+				fixedwidth = false;
+				unsafe = true;
+			}
 			else
 				lex.printError("Unknown width");
 
@@ -885,7 +1039,8 @@ void Encodings::read(FileName const & encfile, FileName const & symbolsfile)
 
 			LYXERR(Debug::INFO, "Reading encoding " << name);
 			encodinglist[name] = Encoding(name, latexname,
-				guiname, iconvname, fixedwidth, package);
+				guiname, iconvname, fixedwidth, unsafe,
+				package);
 
 			if (lex.lex() != et_end)
 				lex.printError("Missing end");
@@ -901,6 +1056,15 @@ void Encodings::read(FileName const & encfile, FileName const & symbolsfile)
 			break;
 		}
 	}
+
+	// Move all information from forcednotselected to forcedselected
+	for (CharSetMap::const_iterator it1 = forcednotselected.begin(); it1 != forcednotselected.end(); ++it1) {
+		for (CharSetMap::iterator it2 = forcedselected.begin(); it2 != forcedselected.end(); ++it2) {
+			if (it2->first != it1->first)
+				it2->second.insert(it1->second.begin(), it1->second.end());
+		}
+	}
+
 }
 
 
