@@ -44,9 +44,9 @@ namespace lyx {
 //
 // These are defined in FontInfo.cpp
 extern char const * LyXFamilyNames[NUM_FAMILIES + 2];
-extern char const * LyXSeriesNames[4];
-extern char const * LyXShapeNames[6];
-extern char const * LyXSizeNames[14];
+extern char const * LyXSeriesNames[NUM_SERIES + 2];
+extern char const * LyXShapeNames[NUM_SHAPE + 2];
+extern char const * LyXSizeNames[NUM_SIZE + 4];
 extern char const * LyXMiscNames[5];
 
 //
@@ -57,17 +57,17 @@ namespace {
 
 char const * GUIFamilyNames[NUM_FAMILIES + 2 /* default & error */] =
 { N_("Roman"), N_("Sans Serif"), N_("Typewriter"), N_("Symbol"),
-  "cmr", "cmsy", "cmm", "cmex", "msa", "msb", "eufrak", "rsfs", "wasy", "esint",
-  N_("Inherit"), N_("Ignore") };
+  "cmr", "cmsy", "cmm", "cmex", "msa", "msb", "eufrak", "rsfs", "stmry",
+  "wasy", "esint", N_("Inherit"), N_("Ignore") };
 
-char const * GUISeriesNames[4] =
+char const * GUISeriesNames[NUM_SERIES + 2 /* default & error */] =
 { N_("Medium"), N_("Bold"), N_("Inherit"), N_("Ignore") };
 
-char const * GUIShapeNames[6] =
+char const * GUIShapeNames[NUM_SHAPE + 2 /* default & error */] =
 { N_("Upright"), N_("Italic"), N_("Slanted"), N_("Smallcaps"), N_("Inherit"),
   N_("Ignore") };
 
-char const * GUISizeNames[14] =
+char const * GUISizeNames[NUM_SIZE + 4 /* increase, decrease, default & error */] =
 { N_("Tiny"), N_("Smallest"), N_("Smaller"), N_("Small"), N_("Normal"), N_("Large"),
   N_("Larger"), N_("Largest"), N_("Huge"), N_("Huger"), N_("Increase"), N_("Decrease"),
   N_("Inherit"), N_("Ignore") };
@@ -78,16 +78,18 @@ char const * GUIMiscNames[5] =
 //
 // Strings used to write LaTeX files
 //
-char const * LaTeXFamilyNames[6] =
-{ "textrm", "textsf", "texttt", "error1", "error2", "error3" };
+char const * LaTeXFamilyNames[NUM_FAMILIES + 2] =
+{ "textrm", "textsf", "texttt", "error1", "error2", "error3", "error4",
+  "error5", "error6", "error7", "error8", "error9", "error10", "error11",
+  "error12", "error13" };
 
-char const * LaTeXSeriesNames[4] =
+char const * LaTeXSeriesNames[NUM_SERIES + 2] =
 { "textmd", "textbf", "error4", "error5" };
 
-char const * LaTeXShapeNames[6] =
+char const * LaTeXShapeNames[NUM_SHAPE + 2] =
 { "textup", "textit", "textsl", "textsc", "error6", "error7" };
 
-char const * LaTeXSizeNames[14] =
+char const * LaTeXSizeNames[NUM_SIZE + 4] =
 { "tiny", "scriptsize", "footnotesize", "small", "normalsize", "large",
   "Large", "LARGE", "huge", "Huge", "error8", "error9", "error10", "error11" };
 
@@ -531,7 +533,7 @@ int Font::latexWriteEndChanges(otexstream & os, BufferParams const & bparams,
 		Encoding const * const ascii = encodings.fromLyXName("ascii");
 		pair<bool, int> const c = switchEncoding(os.os(), bparams,
 				runparams, *ascii);
-		LASSERT(c.first, /**/);
+		LATTEST(c.first);
 		count += c.second;
 		runparams.encoding = ascii;
 		open_encoding_ = false;
@@ -700,9 +702,11 @@ void Font::validate(LaTeXFeatures & features) const
 
 	// FIXME: Do something for background and soul package?
 
-	if (lang_->babel() != doc_language->babel() &&
-		lang_ != ignore_language &&
-		lang_ != latex_language)
+	if (((features.usePolyglossia() && lang_->polyglossia() != doc_language->polyglossia())
+	     || (features.useBabel() && lang_->babel() != doc_language->babel())
+	     || (doc_language->encoding()->package() == Encoding::CJK && lang_ != doc_language))
+	    && lang_ != ignore_language
+	    && lang_ != latex_language)
 	{
 		features.useLanguage(lang_);
 		LYXERR(Debug::LATEX, "Found language " << lang_->lang());
