@@ -33,7 +33,7 @@
 #include "FuncStatus.h"
 #include "HunspellChecker.h"
 #include "KeyMap.h"
-#include "Language.h"
+#include "LaTeXFonts.h"
 #include "LayoutFile.h"
 #include "Lexer.h"
 #include "LyX.h"
@@ -44,12 +44,11 @@
 #include "Server.h"
 #include "ServerSocket.h"
 #include "Session.h"
+#include "WordList.h"
 
 #include "frontends/alert.h"
 #include "frontends/Application.h"
 
-#include "graphics/Previews.h"
-
 #include "support/lassert.h"
 #include "support/debug.h"
 #include "support/environment.h"
@@ -60,7 +59,7 @@
 #include "support/Messages.h"
 #include "support/os.h"
 #include "support/Package.h"
-#include "support/Path.h"
+#include "support/PathChanger.h"
 #include "support/Systemcall.h"
 
 #include "support/bind.h"
@@ -138,14 +137,14 @@ void reconfigureUserLyXDir()
 } // namespace anon
 
 /// The main application class private implementation.
-struct LyX::Impl
-{
-	Impl() : spell_checker_(0), apple_spell_checker_(0), aspell_checker_(0), enchant_checker_(0), hunspell_checker_(0)
-	{
-	}
+struct LyX::Impl {
+	Impl()
+		: latexfonts_(0), spell_checker_(0), apple_spell_checker_(0), aspell_checker_(0), enchant_checker_(0), hunspell_checker_(0)
+	{}
 
 	~Impl()
 	{
+		delete latexfonts_;
 		delete apple_spell_checker_;
 		delete aspell_checker_;
 		delete enchant_checker_;
@@ -190,7 +189,8 @@ struct LyX::Impl
 	vector<string> batch_commands;
 
 	///
-	graphics::Previews preview_;
+	LaTeXFonts * latexfonts_;
+
 	///
 	SpellChecker * spell_checker_;
 	///
@@ -203,6 +203,7 @@ struct LyX::Impl
 	SpellChecker * hunspell_checker_;
 };
 
+
 ///
 frontend::Application * theApp()
 {
@@ -217,6 +218,7 @@ LyX::~LyX()
 {
 	delete pimpl_;
 	singleton_ = 0;
+	WordList::cleanupWordLists();
 }
 
 
@@ -258,7 +260,7 @@ Messages & LyX::messages(string const & language)
 	pair<map<string, Messages>::iterator, bool> result =
 			pimpl_->messages_.insert(make_pair(language, Messages(language)));
 
-	LASSERT(result.second, /**/);
+	LATTEST(result.second);
 	return result.first->second;
 }
 
@@ -267,21 +269,19 @@ int LyX::exec(int & argc, char * argv[])
 {
 	// Minimal setting of locale before parsing command line
 	try {
-		init_package(os::utf8_argv(0), string(), string(),
-			top_build_dir_is_one_level_up);
+		init_package(os::utf8_argv(0), string(), string());
+		// we do not get to this point when init_package throws an exception
+		setLocale();
 	} catch (ExceptionMessage const & message) {
 		LYXERR(Debug::LOCALE, message.title_ + ", " + message.details_);
 	}
-	locale_init();
 
 	// Here we need to parse the command line. At least
 	// we need to parse for "-dbg" and "-help"
 	easyParse(argc, argv);
 
 	try {
-		init_package(os::utf8_argv(0),
-			cl_system_support, cl_user_support,
-			top_build_dir_is_one_level_up);
+		init_package(os::utf8_argv(0), cl_system_support, cl_user_support);
 	} catch (ExceptionMessage const & message) {
 		if (message.type_ == ErrorException) {
 			Alert::error(message.title_, message.details_);
@@ -293,7 +293,7 @@ int LyX::exec(int & argc, char * argv[])
 
 	// Reinit the messages machinery in case package() knows
 	// something interesting about the locale directory.
-	Messages::init();
+	setLocale();
 
 	if (!use_gui) {
 		// FIXME: create a ConsoleApplication
@@ -336,7 +336,7 @@ int LyX::exec(int & argc, char * argv[])
 
 	// Reestablish our defaults, as Qt overwrites them
 	// after createApplication()
-	locale_init();
+	setLocale();//???
 
 	// Parse and remove all known arguments in the LyX singleton
 	// Give an error for all remaining ones.
@@ -429,14 +429,13 @@ void LyX::prepareExit()
 
 	// Kill the application object before exiting. This avoids crashes
 	// when exiting on Linux.
-	if (pimpl_->application_)
-		pimpl_->application_.reset();
+	pimpl_->application_.reset();
 }
 
 
 void LyX::earlyExit(int status)
 {
-	LASSERT(pimpl_->application_.get(), /**/);
+	LATTEST(pimpl_->application_.get());
 	// LyX::pimpl_::application_ is not initialised at this
 	// point so it's safe to just exit after some cleanup.
 	prepareExit();
@@ -468,6 +467,11 @@ int LyX::init(int & argc, char * argv[])
 	for (int argi = 1; argi < argc; ++argi)
 		pimpl_->files_to_load_.push_back(os::utf8_argv(argi));
 
+	if (!use_gui && pimpl_->files_to_load_.empty()) {
+		lyxerr << to_utf8(_("Missing filename for this operation.")) << endl;
+		return EXIT_FAILURE;
+	}
+
 	if (first_start) {
 		pimpl_->files_to_load_.push_back(
 			i18nLibFileSearch("examples", "splash.lyx").absFileName());
@@ -479,7 +483,7 @@ int LyX::init(int & argc, char * argv[])
 
 bool LyX::loadFiles()
 {
-	LASSERT(!use_gui, /**/);
+	LATTEST(!use_gui);
 	bool success = true;
 	vector<string>::const_iterator it = pimpl_->files_to_load_.begin();
 	vector<string>::const_iterator end = pimpl_->files_to_load_.end();
@@ -493,7 +497,7 @@ bool LyX::loadFiles()
 		if (fname.empty())
 			continue;
 
-		Buffer * buf = pimpl_->buffer_list_.newBuffer(fname.absFileName(), false);
+		Buffer * buf = pimpl_->buffer_list_.newBuffer(fname.absFileName());
 		if (buf->loadLyXFile() == Buffer::ReadSuccess) {
 			ErrorList const & el = buf->errorList("Parse");
 			if (!el.empty())
@@ -502,6 +506,10 @@ bool LyX::loadFiles()
 		}
 		else {
 			pimpl_->buffer_list_.release(buf);
+			docstring const error_message =
+				bformat(_("LyX failed to load the following file: %1$s"),
+				from_utf8(fname.absFileName()));
+			lyxerr << to_utf8(error_message) << endl;
 			success = false;
 		}
 	}
@@ -511,7 +519,7 @@ bool LyX::loadFiles()
 
 void execBatchCommands()
 {
-	LASSERT(singleton_, /**/);
+	LAPPERR(singleton_);
 	singleton_->execCommands();
 }
 
@@ -727,6 +735,18 @@ bool LyX::init()
 	if (!setEnv("LyXDir", lyx_dir))
 		LYXERR(Debug::INIT, "\t... failed!");
 
+	if (package().explicit_user_support() && getEnv(LYX_USERDIR_VER).empty()) {
+		// -userdir was given on the command line.
+		// Make it available to child processes, otherwise tex2lyx
+		// would not find all layout files, and other converters might
+		// use it as well.
+		string const user_dir = package().user_support().absFileName();
+		LYXERR(Debug::INIT, "Setting " LYX_USERDIR_VER "... to \""
+		                    << user_dir << '"');
+		if (!setEnv(LYX_USERDIR_VER, user_dir))
+			LYXERR(Debug::INIT, "\t... failed!");
+	}
+
 	//
 	// Read configuration files
 	//
@@ -746,8 +766,15 @@ bool LyX::init()
 		prependEnvPath("PATH", replaceEnvironmentPath(lyxrc.path_prefix));
 
 	// Check that user LyX directory is ok.
-	if (queryUserLyXDir(package().explicit_user_support()))
-		reconfigureUserLyXDir();
+	{
+		string const lock_file = package().user_support().absFileName() + ".lyx_configure_lock";
+		int fd = fileLock(lock_file.c_str());
+
+		if (queryUserLyXDir(package().explicit_user_support())) {
+			reconfigureUserLyXDir();
+		}
+		fileUnlock(fd, lock_file.c_str());
+	}
 
 	if (!use_gui) {
 		// No need for a splash when there is no GUI
@@ -785,6 +812,9 @@ bool LyX::init()
 	if (!readRcFile("preferences", true))
 		return false;
 
+	// The language may have been set to someting useful through prefs
+	setLocale();
+
 	if (!readEncodingsFile("encodings", "unicodesymbols"))
 		return false;
 	if (!readLanguagesFile("languages"))
@@ -814,6 +844,7 @@ bool LyX::init()
 		lyxrc.print();
 
 	os::windows_style_tex_paths(lyxrc.windows_style_tex_paths);
+	// Prepend path prefix a second time to take the user preferences into a account
 	if (!lyxrc.path_prefix.empty())
 		prependEnvPath("PATH", replaceEnvironmentPath(lyxrc.path_prefix));
 
@@ -871,7 +902,7 @@ void emergencyCleanup()
 static bool needsUpdate(string const & file)
 {
 	// We cannot initialize configure_script directly because the package
-	// is not initialized yet when  static objects are constructed.
+	// is not initialized yet when static objects are constructed.
 	static FileName configure_script;
 	static bool firstrun = true;
 	if (firstrun) {
@@ -997,8 +1028,8 @@ typedef boost::function<int(string const &, string const &, string &)> cmd_helpe
 int parse_dbg(string const & arg, string const &, string &)
 {
 	if (arg.empty()) {
-		lyxerr << to_utf8(_("List of supported debug flags:")) << endl;
-		Debug::showTags(lyxerr);
+		cout << to_utf8(_("List of supported debug flags:")) << endl;
+		Debug::showTags(cout);
 		exit(0);
 	}
 	lyxerr << to_utf8(bformat(_("Setting debug level to %1$s"), from_utf8(arg))) << endl;
@@ -1011,7 +1042,7 @@ int parse_dbg(string const & arg, string const &, string &)
 
 int parse_help(string const &, string const &, string &)
 {
-	lyxerr <<
+	cout <<
 		to_utf8(_("Usage: lyx [ command line switches ] [ name.lyx ... ]\n"
 		  "Command line switches (case sensitive):\n"
 		  "\t-help              summarize LyX usage\n"
@@ -1024,10 +1055,14 @@ int parse_help(string const &, string const &, string &)
 		  "\t-x [--execute] command\n"
 		  "                  where command is a lyx command.\n"
 		  "\t-e [--export] fmt\n"
-		  "                  where fmt is the export format of choice.\n"
-		  "                  Look on Tools->Preferences->File formats->Format\n"
-		  "                  to get an idea which parameters should be passed.\n"
+		  "                  where fmt is the export format of choice. Look in\n"
+		  "                  Tools->Preferences->File Handling->File Formats->Short Name\n"
+		  "                  to see which parameter (which differs from the format name\n"
+		  "                  in the File->Export menu) should be passed.\n"
 		  "                  Note that the order of -e and -x switches matters.\n"
+		  "\t-E [--export-to] fmt filename\n"
+		  "                  where fmt is the export format of choice (see --export),\n"
+		  "                  and filename is the destination filename.\n"
 		  "\t-i [--import] fmt file.xxx\n"
 		  "                  where fmt is the import format of choice\n"
 		  "                  and file.xxx is the file to be imported.\n"
@@ -1051,11 +1086,12 @@ int parse_help(string const &, string const &, string &)
 
 int parse_version(string const &, string const &, string &)
 {
-	lyxerr << "LyX " << lyx_version
+	cout << "LyX " << lyx_version
 	       << " (" << lyx_release_date << ")" << endl;
-	lyxerr << "Built on " << __DATE__ << ", " << __TIME__ << endl;
+	cout << to_utf8(bformat(_("Built on %1$s[[date]], %2$s[[time]]"),
+		from_ascii(__DATE__), from_ascii(__TIME__))) << endl;
 
-	lyxerr << lyx_version_info << endl;
+	cout << lyx_version_info << endl;
 	exit(0);
 	return 0;
 }
@@ -1097,6 +1133,24 @@ int parse_execute(string const & arg, string const &, string & batch)
 }
 
 
+int parse_export_to(string const & type, string const & output_file, string & batch)
+{
+	if (type.empty()) {
+		lyxerr << to_utf8(_("Missing file type [eg latex, ps...] after "
+					 "--export-to switch")) << endl;
+		exit(1);
+	}
+	if (output_file.empty()) {
+		lyxerr << to_utf8(_("Missing destination filename after "
+					 "--export-to switch")) << endl;
+		exit(1);
+	}
+	batch = "buffer-export " + type + " " + output_file;
+	use_gui = false;
+	return 2;
+}
+
+
 int parse_export(string const & type, string const &, string & batch)
 {
 	if (type.empty()) {
@@ -1121,7 +1175,6 @@ int parse_import(string const & type, string const & file, string & batch)
 		lyxerr << to_utf8(_("Missing filename for --import")) << endl;
 		exit(1);
 	}
-
 	batch = "buffer-import " + type + ' ' + file;
 	return 2;
 }
@@ -1190,8 +1243,10 @@ void LyX::easyParse(int & argc, char * argv[])
 	cmdmap["-userdir"] = parse_userdir;
 	cmdmap["-x"] = parse_execute;
 	cmdmap["--execute"] = parse_execute;
-	cmdmap["-e"] = parse_export;
+ 	cmdmap["-e"] = parse_export;
 	cmdmap["--export"] = parse_export;
+	cmdmap["-E"] = parse_export_to;
+	cmdmap["--export-to"] = parse_export_to;
 	cmdmap["-i"] = parse_import;
 	cmdmap["--import"] = parse_import;
 	cmdmap["-geometry"] = parse_geometry;
@@ -1236,35 +1291,35 @@ void LyX::easyParse(int & argc, char * argv[])
 
 FuncStatus getStatus(FuncRequest const & action)
 {
-	LASSERT(theApp(), /**/);
+	LAPPERR(theApp());
 	return theApp()->getStatus(action);
 }
 
 
 void dispatch(FuncRequest const & action)
 {
-	LASSERT(theApp(), /**/);
+	LAPPERR(theApp());
 	return theApp()->dispatch(action);
 }
 
 
 void dispatch(FuncRequest const & action, DispatchResult & dr)
 {
-	LASSERT(theApp(), /**/);
+	LAPPERR(theApp());
 	return theApp()->dispatch(action, dr);
 }
 
 
 vector<string> & theFilesToLoad()
 {
-	LASSERT(singleton_, /**/);
+	LAPPERR(singleton_);
 	return singleton_->pimpl_->files_to_load_;
 }
 
 
 BufferList & theBufferList()
 {
-	LASSERT(singleton_, /**/);
+	LAPPERR(singleton_);
 	return singleton_->pimpl_->buffer_list_;
 }
 
@@ -1272,8 +1327,8 @@ BufferList & theBufferList()
 Server & theServer()
 {
 	// FIXME: this should not be use_gui dependent
-	LASSERT(use_gui, /**/);
-	LASSERT(singleton_, /**/);
+	LWARNIF(use_gui);
+	LAPPERR(singleton_);
 	return *singleton_->pimpl_->lyx_server_.get();
 }
 
@@ -1281,104 +1336,94 @@ Server & theServer()
 ServerSocket & theServerSocket()
 {
 	// FIXME: this should not be use_gui dependent
-	LASSERT(use_gui, /**/);
-	LASSERT(singleton_, /**/);
+	LWARNIF(use_gui);
+	LAPPERR(singleton_);
 	return *singleton_->pimpl_->lyx_socket_.get();
 }
 
 
 KeyMap & theTopLevelKeymap()
 {
-	LASSERT(singleton_, /**/);
+	LAPPERR(singleton_);
 	return singleton_->pimpl_->toplevel_keymap_;
 }
 
 
 Converters & theConverters()
 {
-	LASSERT(singleton_, /**/);
+	LAPPERR(singleton_);
 	return  singleton_->pimpl_->converters_;
 }
 
 
 Converters & theSystemConverters()
 {
-	LASSERT(singleton_, /**/);
+	LAPPERR(singleton_);
 	return  singleton_->pimpl_->system_converters_;
 }
 
 
 Movers & theMovers()
 {
-	LASSERT(singleton_, /**/);
+	LAPPERR(singleton_);
 	return singleton_->pimpl_->movers_;
 }
 
 
 Mover const & getMover(string  const & fmt)
 {
-	LASSERT(singleton_, /**/);
+	LAPPERR(singleton_);
 	return singleton_->pimpl_->movers_(fmt);
 }
 
 
 void setMover(string const & fmt, string const & command)
 {
-	LASSERT(singleton_, /**/);
+	LAPPERR(singleton_);
 	singleton_->pimpl_->movers_.set(fmt, command);
 }
 
 
 Movers & theSystemMovers()
 {
-	LASSERT(singleton_, /**/);
+	LAPPERR(singleton_);
 	return singleton_->pimpl_->system_movers_;
 }
 
 
 Messages const & getMessages(string const & language)
 {
-	LASSERT(singleton_, /**/);
+	LAPPERR(singleton_);
 	return singleton_->messages(language);
 }
 
 
 Messages const & getGuiMessages()
 {
-	LASSERT(singleton_, /**/);
-	// A cache to translate full language name to language code
-	static string last_language = "auto";
-	static string code;
-	if (lyxrc.gui_language != last_language) {
-		if (lyxrc.gui_language == "auto")
-			code.clear();
-		else {
-			Language const * l = languages.getLanguage(lyxrc.gui_language);
-			code = l ? l->code() : string();
-		}
-		last_language = lyxrc.gui_language;
-	}
-	return singleton_->messages(code);
-}
-
-
-graphics::Previews & thePreviews()
-{
-	LASSERT(singleton_, /**/);
-	return singleton_->pimpl_->preview_;
+	LAPPERR(singleton_);
+	return singleton_->messages(Messages::guiLanguage());
 }
 
 
 Session & theSession()
 {
-	LASSERT(singleton_, /**/);
+	LAPPERR(singleton_);
 	return *singleton_->pimpl_->session_.get();
 }
 
 
+LaTeXFonts & theLaTeXFonts()
+{
+	LAPPERR(singleton_);
+	if (!singleton_->pimpl_->latexfonts_)
+		singleton_->pimpl_->latexfonts_ = new LaTeXFonts;
+	return *singleton_->pimpl_->latexfonts_;
+}
+
+
 CmdDef & theTopLevelCmdDef()
 {
-	LASSERT(singleton_, /**/);
+	LAPPERR(singleton_);
 	return singleton_->pimpl_->toplevel_cmddef_;
 }
 
@@ -1399,7 +1444,7 @@ void setSpellChecker()
 	if (lyxrc.spellchecker == "native") {
 #if defined(USE_MACOSX_PACKAGING)
 		if (!singleton_->pimpl_->apple_spell_checker_)
-			singleton_->pimpl_->apple_spell_checker_ = new AppleSpellChecker();
+			singleton_->pimpl_->apple_spell_checker_ = new AppleSpellChecker;
 		singleton_->pimpl_->spell_checker_ = singleton_->pimpl_->apple_spell_checker_;
 #else
 		singleton_->pimpl_->spell_checker_ = 0;
@@ -1407,7 +1452,7 @@ void setSpellChecker()
 	} else if (lyxrc.spellchecker == "aspell") {
 #if defined(USE_ASPELL)
 		if (!singleton_->pimpl_->aspell_checker_)
-			singleton_->pimpl_->aspell_checker_ = new AspellChecker();
+			singleton_->pimpl_->aspell_checker_ = new AspellChecker;
 		singleton_->pimpl_->spell_checker_ = singleton_->pimpl_->aspell_checker_;
 #else
 		singleton_->pimpl_->spell_checker_ = 0;
@@ -1415,7 +1460,7 @@ void setSpellChecker()
 	} else if (lyxrc.spellchecker == "enchant") {
 #if defined(USE_ENCHANT)
 		if (!singleton_->pimpl_->enchant_checker_)
-			singleton_->pimpl_->enchant_checker_ = new EnchantChecker();
+			singleton_->pimpl_->enchant_checker_ = new EnchantChecker;
 		singleton_->pimpl_->spell_checker_ = singleton_->pimpl_->enchant_checker_;
 #else
 		singleton_->pimpl_->spell_checker_ = 0;
@@ -1423,7 +1468,7 @@ void setSpellChecker()
 	} else if (lyxrc.spellchecker == "hunspell") {
 #if defined(USE_HUNSPELL)
 		if (!singleton_->pimpl_->hunspell_checker_)
-			singleton_->pimpl_->hunspell_checker_ = new HunspellChecker();
+			singleton_->pimpl_->hunspell_checker_ = new HunspellChecker;
 		singleton_->pimpl_->spell_checker_ = singleton_->pimpl_->hunspell_checker_;
 #else
 		singleton_->pimpl_->spell_checker_ = 0;
