@@ -46,7 +46,6 @@
 #include "ParIterator.h"
 #include "TextClass.h"
 #include "TextMetrics.h"
-#include "VSpace.h"
 #include "WordLangTuple.h"
 #include "WordList.h"
 
@@ -56,10 +55,12 @@
 #include "insets/InsetNewline.h"
 #include "insets/InsetNewpage.h"
 #include "insets/InsetArgument.h"
+#include "insets/InsetIPAMacro.h"
 #include "insets/InsetSpace.h"
 #include "insets/InsetSpecialChar.h"
 #include "insets/InsetTabular.h"
 
+#include "support/convert.h"
 #include "support/debug.h"
 #include "support/docstream.h"
 #include "support/gettext.h"
@@ -72,6 +73,31 @@
 #include <limits>
 #include <sstream>
 
+
+// TODO: replace if in Text::readParToken() with compile time switch
+#if 0
+
+#include "support/metahash.h"
+
+typedef boost::mpl::string<'\\end','_lay','out'> end_layout;
+typedef boost::mpl::string<'\\end','in','set'>   end_inset;
+
+void foo()
+{
+	std::string token = "\\end_layout";
+
+	switch (boost::hash_value(token)) {
+		case lyx::support::hash_string<end_layout>::value:
+			return;
+		case lyx::support::hash_string<end_inset>::value:
+			return;
+		default: ;
+	};
+
+}
+#endif
+
+
 using namespace std;
 using namespace lyx::support;
 
@@ -118,7 +144,7 @@ void breakParagraphConservative(BufferParams const & bparams,
 	tmp.setInsetOwner(&par.inInset());
 	tmp.makeSameLayout(par);
 
-	LASSERT(pos <= par.size(), /**/);
+	LASSERT(pos <= par.size(), return);
 
 	if (pos < par.size()) {
 		// move everything behind the break position to the new paragraph
@@ -239,6 +265,17 @@ bool Text::isFirstInSequence(pit_type par_offset) const
 }
 
 
+int Text::getTocLevel(pit_type par_offset) const
+{
+	Paragraph const & par = pars_[par_offset];
+
+	if (par.layout().isEnvironment() && !isFirstInSequence(par_offset))
+		return Layout::NOT_IN_TOC;
+
+	return par.layout().toclevel;
+}
+
+
 Font const Text::outerFont(pit_type par_offset) const
 {
 	depth_type par_depth = pars_[par_offset].getDepth();
@@ -320,6 +357,7 @@ InsetText const & Text::inset() const
 }
 
 
+
 void Text::readParToken(Paragraph & par, Lexer & lex,
 	string const & token, Font & font, Change & change, ErrorList & errorList)
 {
@@ -346,10 +384,10 @@ void Text::readParToken(Paragraph & par, Lexer & lex,
 			// in this case only the empty layout is allowed
 			layoutname = tclass.plainLayoutName();
 		} else if (par.usePlainLayout()) {
-			// in this case, default layout maps to empty layout 
+			// in this case, default layout maps to empty layout
 			if (layoutname == tclass.defaultLayoutName())
 				layoutname = tclass.plainLayoutName();
-		} else { 
+		} else {
 			// otherwise, the empty layout maps to the default
 			if (layoutname == tclass.plainLayoutName())
 				layoutname = tclass.defaultLayoutName();
@@ -359,7 +397,6 @@ void Text::readParToken(Paragraph & par, Lexer & lex,
 		// of this document. For example, when you apply class article to a beamer document,
 		// all unknown layouts such as frame will be added to document class article so that
 		// these layouts can keep their original names.
-		tclass.addLayoutIfNeeded(layoutname);
 		bool const added_one = tclass.addLayoutIfNeeded(layoutname);
 		if (added_one) {
 			// Warn the user.
@@ -455,6 +492,12 @@ void Text::readParToken(Paragraph & par, Lexer & lex,
 		inset->read(lex);
 		inset->setBuffer(*buf);
 		par.insertInset(par.size(), inset.release(), font, change);
+	} else if (token == "\\IPAChar") {
+		auto_ptr<Inset> inset;
+		inset.reset(new InsetIPAChar);
+		inset->read(lex);
+		inset->setBuffer(*buf);
+		par.insertInset(par.size(), inset.release(), font, change);
 	} else if (token == "\\backslash") {
 		par.appendChar('\\', font, change);
 	} else if (token == "\\LyXTable") {
@@ -529,7 +572,7 @@ void Text::readParagraph(Paragraph & par, Lexer & lex,
 
 	// Initialize begin_of_body_ on load; redoParagraph maintains
 	par.setBeginOfBody();
-	
+
 	// mark paragraph for spell checking on load
 	// par.requestSpellCheck();
 }
@@ -540,11 +583,11 @@ class TextCompletionList : public CompletionList
 public:
 	///
 	TextCompletionList(Cursor const & cur, WordList const * list)
-		: buffer_(cur.buffer()), pos_(0), list_(list)
+		: buffer_(cur.buffer()), list_(list)
 	{}
 	///
 	virtual ~TextCompletionList() {}
-	
+
 	///
 	virtual bool sorted() const { return true; }
 	///
@@ -557,13 +600,11 @@ public:
 	{
 		return list_->word(idx);
 	}
-	
+
 private:
 	///
 	Buffer const * buffer_;
 	///
-	size_t pos_;
-	///
 	WordList const * list_;
 };
 
@@ -571,7 +612,7 @@ private:
 bool Text::empty() const
 {
 	return pars_.empty() || (pars_.size() == 1 && pars_[0].empty()
-		// FIXME: Should we consider the labeled type as empty too? 
+		// FIXME: Should we consider the labeled type as empty too?
 		&& pars_[0].layout().labeltype == LABEL_NO_LABEL);
 }
 
@@ -588,10 +629,10 @@ double Text::spacing(Paragraph const & par) const
  * This breaks a paragraph at the specified position.
  * The new paragraph will:
  * - Decrease depth by one (or change layout to default layout) when
- *    keep_layout == false  
+ *    keep_layout == false
  * - keep current depth and layout when keep_layout == true
  */
-static void breakParagraph(Text & text, pit_type par_offset, pos_type pos, 
+static void breakParagraph(Text & text, pit_type par_offset, pos_type pos,
 		    bool keep_layout)
 {
 	BufferParams const & bparams = text.inset().buffer().params();
@@ -684,7 +725,7 @@ static void breakParagraph(Text & text, pit_type par_offset, pos_type pos,
 
 void Text::breakParagraph(Cursor & cur, bool inverse_logic)
 {
-	LASSERT(this == cur.text(), /**/);
+	LBUFERR(this == cur.text());
 
 	Paragraph & cpar = cur.paragraph();
 	pit_type cpit = cur.pit();
@@ -711,7 +752,7 @@ void Text::breakParagraph(Cursor & cur, bool inverse_logic)
 		cpar.eraseChar(cur.pos(), cur.buffer()->params().trackChanges);
 
 	// What should the layout for the new paragraph be?
-	bool keep_layout = layout.isEnvironment() 
+	bool keep_layout = layout.isEnvironment()
 		|| (layout.isParagraph() && layout.parbreak_is_newline);
 	if (inverse_logic)
 		keep_layout = !keep_layout;
@@ -844,7 +885,7 @@ void Text::insertStringAsParagraphs(Cursor & cur, docstring const & str,
 // same Paragraph one to the right and make a rebreak
 void Text::insertChar(Cursor & cur, char_type c)
 {
-	LASSERT(this == cur.text(), /**/);
+	LBUFERR(this == cur.text());
 
 	cur.recordUndo(INSERT_UNDO);
 
@@ -895,40 +936,40 @@ void Text::insertChar(Cursor & cur, char_type c)
 	}
 
 	// In Bidi text, we want spaces to be treated in a special way: spaces
-	// which are between words in different languages should get the 
-	// paragraph's language; otherwise, spaces should keep the language 
+	// which are between words in different languages should get the
+	// paragraph's language; otherwise, spaces should keep the language
 	// they were originally typed in. This is only in effect while typing;
 	// after the text is already typed in, the user can always go back and
 	// explicitly set the language of a space as desired. But 99.9% of the
 	// time, what we're doing here is what the user actually meant.
-	// 
+	//
 	// The following cases are the ones in which the language of the space
 	// should be changed to match that of the containing paragraph. In the
-	// depictions, lowercase is LTR, uppercase is RTL, underscore (_) 
+	// depictions, lowercase is LTR, uppercase is RTL, underscore (_)
 	// represents a space, pipe (|) represents the cursor position (so the
 	// character before it is the one just typed in). The different cases
 	// are depicted logically (not visually), from left to right:
-	// 
+	//
 	// 1. A_a|
 	// 2. a_A|
 	//
 	// Theoretically, there are other situations that we should, perhaps, deal
-	// with (e.g.: a|_A, A|_a). In practice, though, there really isn't any 
+	// with (e.g.: a|_A, A|_a). In practice, though, there really isn't any
 	// point (to understand why, just try to create this situation...).
 
 	if ((cur.pos() >= 2) && (par.isLineSeparator(cur.pos() - 1))) {
-		// get font in front and behind the space in question. But do NOT 
+		// get font in front and behind the space in question. But do NOT
 		// use getFont(cur.pos()) because the character c is not inserted yet
 		Font const pre_space_font  = tm.displayFont(cur.pit(), cur.pos() - 2);
 		Font const & post_space_font = cur.real_current_font;
 		bool pre_space_rtl  = pre_space_font.isVisibleRightToLeft();
 		bool post_space_rtl = post_space_font.isVisibleRightToLeft();
-		
+
 		if (pre_space_rtl != post_space_rtl) {
-			// Set the space's language to match the language of the 
+			// Set the space's language to match the language of the
 			// adjacent character whose direction is the paragraph's
 			// direction; don't touch other properties of the font
-			Language const * lang = 
+			Language const * lang =
 				(pre_space_rtl == par.isRTL(buffer.params())) ?
 				pre_space_font.language() : post_space_font.language();
 
@@ -937,7 +978,7 @@ void Text::insertChar(Cursor & cur, char_type c)
 			par.setFont(cur.pos() - 1, space_font);
 		}
 	}
-	
+
 	// Next check, if there will be two blanks together or a blank at
 	// the beginning of a paragraph.
 	// I decided to handle blanks like normal characters, the main
@@ -953,6 +994,7 @@ void Text::insertChar(Cursor & cur, char_type c)
 					"beginning of a paragraph. Please read the Tutorial."));
 			return;
 		}
+		// LASSERT: Is it safe to continue here?
 		LASSERT(cur.pos() > 0, /**/);
 		if ((par.isLineSeparator(cur.pos() - 1) || par.isNewline(cur.pos() - 1))
 				&& !par.isDeleted(cur.pos() - 1)) {
@@ -963,6 +1005,16 @@ void Text::insertChar(Cursor & cur, char_type c)
 		}
 	}
 
+	// Prevent to insert uncodable characters in verbatim and ERT
+	// (workaround for bug 9012)
+	if (cur.paragraph().isPassThru() && cur.current_font.language()) {
+		Encoding const * e = cur.current_font.language()->encoding();
+		if (!e->encodable(c)) {
+			cur.message(_("Character is uncodable in verbatim paragraphs."));
+			return;
+		}
+	}
+
 	par.insertChar(cur.pos(), c, cur.current_font,
 		cur.buffer()->params().trackChanges);
 	cur.checkBufferStructure();
@@ -993,7 +1045,7 @@ void Text::charInserted(Cursor & cur)
 	    && !par.isWordSeparator(cur.pos() - 2)
 	    && par.isWordSeparator(cur.pos() - 1)) {
 		// get the word in front of cursor
-		LASSERT(this == cur.text(), /**/);
+		LBUFERR(this == cur.text());
 		cur.paragraph().updateWords();
 	}
 }
@@ -1004,7 +1056,7 @@ void Text::charInserted(Cursor & cur)
 
 bool Text::cursorForwardOneWord(Cursor & cur)
 {
-	LASSERT(this == cur.text(), /**/);
+	LBUFERR(this == cur.text());
 
 	pos_type const lastpos = cur.lastpos();
 	pit_type pit = cur.pit();
@@ -1019,7 +1071,7 @@ bool Text::cursorForwardOneWord(Cursor & cur)
 			return false;
 	}
 
-	if (lyxrc.mac_like_word_movement) {
+	if (lyxrc.mac_like_cursor_movement) {
 		// Skip through trailing punctuation and spaces.
 		while (pos != lastpos && (par.isChar(pos) || par.isSpace(pos)))
                         ++pos;
@@ -1030,7 +1082,7 @@ bool Text::cursorForwardOneWord(Cursor & cur)
 		else while (pos != lastpos && !par.isWordSeparator(pos))
 			     ++pos;
 	} else {
-		LASSERT(pos < lastpos, /**/); // see above
+		LASSERT(pos < lastpos, return false); // see above
 		if (!par.isWordSeparator(pos))
 			while (pos != lastpos && !par.isWordSeparator(pos))
 				++pos;
@@ -1042,7 +1094,7 @@ bool Text::cursorForwardOneWord(Cursor & cur)
 
 		// Skip over white space
 		while (pos != lastpos && par.isSpace(pos))
-			     ++pos;		
+			     ++pos;
 	}
 
 	return setCursor(cur, pit, pos);
@@ -1051,7 +1103,7 @@ bool Text::cursorForwardOneWord(Cursor & cur)
 
 bool Text::cursorBackwardOneWord(Cursor & cur)
 {
-	LASSERT(this == cur.text(), /**/);
+	LBUFERR(this == cur.text());
 
 	pit_type pit = cur.pit();
 	pos_type pos = cur.pos();
@@ -1061,7 +1113,7 @@ bool Text::cursorBackwardOneWord(Cursor & cur)
 	if (pos == 0 && pit != 0)
 		return setCursor(cur, pit - 1, getPar(pit - 1).size());
 
-	if (lyxrc.mac_like_word_movement) {
+	if (lyxrc.mac_like_cursor_movement) {
 		// Skip through punctuation and spaces.
 		while (pos != 0 && (par.isChar(pos - 1) || par.isSpace(pos - 1)))
 			--pos;
@@ -1092,7 +1144,7 @@ bool Text::cursorBackwardOneWord(Cursor & cur)
 
 bool Text::cursorVisLeftOneWord(Cursor & cur)
 {
-	LASSERT(this == cur.text(), /**/);
+	LBUFERR(this == cur.text());
 
 	pos_type left_pos, right_pos;
 	bool left_is_letter, right_is_letter;
@@ -1104,9 +1156,9 @@ bool Text::cursorVisLeftOneWord(Cursor & cur)
 
 		// collect some information about current cursor position
 		temp_cur.getSurroundingPos(left_pos, right_pos);
-		left_is_letter = 
+		left_is_letter =
 			(left_pos > -1 ? !temp_cur.paragraph().isWordSeparator(left_pos) : false);
-		right_is_letter = 
+		right_is_letter =
 			(right_pos > -1 ? !temp_cur.paragraph().isWordSeparator(right_pos) : false);
 
 		// if we're not at a letter/non-letter boundary, continue moving
@@ -1122,14 +1174,14 @@ bool Text::cursorVisLeftOneWord(Cursor & cur)
 			break;
 	}
 
-	return setCursor(cur, temp_cur.pit(), temp_cur.pos(), 
+	return setCursor(cur, temp_cur.pit(), temp_cur.pos(),
 					 true, temp_cur.boundary());
 }
 
 
 bool Text::cursorVisRightOneWord(Cursor & cur)
 {
-	LASSERT(this == cur.text(), /**/);
+	LBUFERR(this == cur.text());
 
 	pos_type left_pos, right_pos;
 	bool left_is_letter, right_is_letter;
@@ -1141,9 +1193,9 @@ bool Text::cursorVisRightOneWord(Cursor & cur)
 
 		// collect some information about current cursor position
 		temp_cur.getSurroundingPos(left_pos, right_pos);
-		left_is_letter = 
+		left_is_letter =
 			(left_pos > -1 ? !temp_cur.paragraph().isWordSeparator(left_pos) : false);
-		right_is_letter = 
+		right_is_letter =
 			(right_pos > -1 ? !temp_cur.paragraph().isWordSeparator(right_pos) : false);
 
 		// if we're not at a letter/non-letter boundary, continue moving
@@ -1153,22 +1205,22 @@ bool Text::cursorVisRightOneWord(Cursor & cur)
 		// we should stop when we have an LTR word on our right or an RTL word
 		// on our left
 		if ((left_is_letter && temp_cur.paragraph().getFontSettings(
-				temp_cur.buffer()->params(), 
+				temp_cur.buffer()->params(),
 				left_pos).isRightToLeft())
 			|| (right_is_letter && !temp_cur.paragraph().getFontSettings(
-				temp_cur.buffer()->params(), 
+				temp_cur.buffer()->params(),
 				right_pos).isRightToLeft()))
 			break;
 	}
 
-	return setCursor(cur, temp_cur.pit(), temp_cur.pos(), 
+	return setCursor(cur, temp_cur.pit(), temp_cur.pos(),
 					 true, temp_cur.boundary());
 }
 
 
 void Text::selectWord(Cursor & cur, word_location loc)
 {
-	LASSERT(this == cur.text(), /**/);
+	LBUFERR(this == cur.text());
 	CursorSlice from = cur.top();
 	CursorSlice to = cur.top();
 	getWord(from, to, loc);
@@ -1186,14 +1238,14 @@ void Text::selectWord(Cursor & cur, word_location loc)
 
 void Text::selectAll(Cursor & cur)
 {
-	LASSERT(this == cur.text(), /**/);
+	LBUFERR(this == cur.text());
 	if (cur.lastpos() == 0 && cur.lastpit() == 0)
 		return;
 	// If the cursor is at the beginning, make sure the cursor ends there
 	if (cur.pit() == 0 && cur.pos() == 0) {
 		setCursor(cur, cur.lastpit(), getPar(cur.lastpit()).size());
 		cur.resetAnchor();
-		setCursor(cur, 0, 0);		
+		setCursor(cur, 0, 0);
 	} else {
 		setCursor(cur, 0, 0);
 		cur.resetAnchor();
@@ -1207,7 +1259,7 @@ void Text::selectAll(Cursor & cur)
 // selection is currently set
 bool Text::selectWordWhenUnderCursor(Cursor & cur, word_location loc)
 {
-	LASSERT(this == cur.text(), /**/);
+	LBUFERR(this == cur.text());
 	if (cur.selection())
 		return false;
 	selectWord(cur, loc);
@@ -1217,7 +1269,7 @@ bool Text::selectWordWhenUnderCursor(Cursor & cur, word_location loc)
 
 void Text::acceptOrRejectChanges(Cursor & cur, ChangeOp op)
 {
-	LASSERT(this == cur.text(), /**/);
+	LBUFERR(this == cur.text());
 
 	if (!cur.selection()) {
 		bool const changed = cur.paragraph().isChanged(cur.pos());
@@ -1256,11 +1308,11 @@ void Text::acceptOrRejectChanges(Cursor & cur, ChangeOp op)
 
 		pos_type left  = (pit == begPit ? begPos : 0);
 		pos_type right = (pit == endPit ? endPos : parSize);
-		
+
 		if (left == right)
 			// there is no change here
 			continue;
-		
+
 		if (op == ACCEPT) {
 			pars_[pit].acceptChanges(left, right);
 		} else {
@@ -1372,7 +1424,7 @@ void Text::rejectChanges()
 
 void Text::deleteWordForward(Cursor & cur)
 {
-	LASSERT(this == cur.text(), /**/);
+	LBUFERR(this == cur.text());
 	if (cur.lastpos() == 0)
 		cursorForward(cur);
 	else {
@@ -1388,7 +1440,7 @@ void Text::deleteWordForward(Cursor & cur)
 
 void Text::deleteWordBackward(Cursor & cur)
 {
-	LASSERT(this == cur.text(), /**/);
+	LBUFERR(this == cur.text());
 	if (cur.lastpos() == 0)
 		cursorBackward(cur);
 	else {
@@ -1405,7 +1457,7 @@ void Text::deleteWordBackward(Cursor & cur)
 // Kill to end of line.
 void Text::changeCase(Cursor & cur, TextCase action)
 {
-	LASSERT(this == cur.text(), /**/);
+	LBUFERR(this == cur.text());
 	CursorSlice from;
 	CursorSlice to;
 
@@ -1477,7 +1529,7 @@ bool Text::handleBibitems(Cursor & cur)
 		setCursorIntern(cur, prevcur.pit(), prevcur.pos());
 		cur.screenUpdateFlags(Update::Force);
 		return true;
-	} 
+	}
 
 	// otherwise reset to default
 	cur.paragraph().setPlainOrDefaultLayout(bufparams.documentClass());
@@ -1534,7 +1586,7 @@ bool Text::erase(Cursor & cur)
 
 bool Text::backspacePos0(Cursor & cur)
 {
-	LASSERT(this == cur.text(), /**/);
+	LBUFERR(this == cur.text());
 	if (cur.pit() == 0)
 		return false;
 
@@ -1587,7 +1639,7 @@ bool Text::backspacePos0(Cursor & cur)
 
 bool Text::backspace(Cursor & cur)
 {
-	LASSERT(this == cur.text(), /**/);
+	LBUFERR(this == cur.text());
 	bool needsUpdate = false;
 	if (cur.pos() == 0) {
 		if (cur.pit() == 0)
@@ -1663,9 +1715,13 @@ bool Text::dissolveInset(Cursor & cur)
 
 	if (!plist.empty()) {
 		// see bug 7319
-		// we need to update here since we need to refresh
-		// the reference cache.
-		cur.buffer()->updateBuffer();
+		// we clear the cache so that we won't get conflicts with labels
+		// that get pasted into the buffer. we should update this before
+		// its being empty matters. if not (i.e., if we encounter bugs),
+		// then this should instead be:
+		//	  cur.buffer().updateBuffer();
+		// but we'll try the cheaper solution here.
+		cur.buffer()->clearReferenceCache();
 
 		// ERT paragraphs have the Language latex_language.
 		// This is invalid outside of ERT, so we need to
@@ -1715,7 +1771,7 @@ void Text::write(ostream & os) const
 }
 
 
-bool Text::read(Lexer & lex, 
+bool Text::read(Lexer & lex,
 		ErrorList & errorList, InsetText * insetPtr)
 {
 	Buffer const & buf = owner_->buffer();
@@ -1772,19 +1828,20 @@ bool Text::read(Lexer & lex,
 		Paragraph par;
 		par.setInsetOwner(insetPtr);
 		par.params().depth(depth);
-		par.setFont(0, Font(inherit_font, 
+		par.setFont(0, Font(inherit_font,
 				    buf.params().language));
 		par.setPlainOrDefaultLayout(buf.params().documentClass());
 		pars_.push_back(par);
 	}
-	
+
 	return res;
 }
 
+
 // Returns the current font and depth as a message.
 docstring Text::currentState(Cursor const & cur) const
 {
-	LASSERT(this == cur.text(), /**/);
+	LBUFERR(this == cur.text());
 	Buffer & buf = *cur.buffer();
 	Paragraph const & par = cur.paragraph();
 	odocstringstream os;
@@ -1883,7 +1940,7 @@ docstring Text::getPossibleLabel(Cursor const & cur) const
 			text += '-';
 		text += head;
 	}
-	
+
 	// Make sure it isn't too long
 	unsigned int const max_label_length = 32;
 	if (text.size() > max_label_length)
@@ -1906,7 +1963,7 @@ docstring Text::getPossibleLabel(Cursor const & cur) const
 	// For captions, we just take the caption type
 	Inset * caption_inset = cur.innerInsetOfType(CAPTION_CODE);
 	if (caption_inset) {
-		string const & ftype = static_cast<InsetCaption *>(caption_inset)->type();
+		string const & ftype = static_cast<InsetCaption *>(caption_inset)->floattype();
 		FloatList const & fl = cur.buffer()->params().documentClass().floats();
 		if (fl.typeExist(ftype)) {
 			Floating const & flt = fl.getType(ftype);
@@ -1925,7 +1982,15 @@ docstring Text::getPossibleLabel(Cursor const & cur) const
 	if (!name.empty())
 		text = name + ':' + text;
 
-	return text;
+	// We need a unique label
+	docstring label = text;
+	int i = 1;
+	while (cur.buffer()->insetLabel(label)) {
+			label = text + '-' + convert<docstring>(i);
+			++i;
+		}
+
+	return label;
 }
 
 
@@ -1962,7 +2027,7 @@ void Text::forToc(docstring & os, size_t maxlen, bool shorten) const
 
 void Text::charsTranspose(Cursor & cur)
 {
-	LASSERT(this == cur.text(), /**/);
+	LBUFERR(this == cur.text());
 
 	pos_type pos = cur.pos();
 
@@ -2038,7 +2103,7 @@ docstring Text::previousWord(CursorSlice const & sl) const
 	getWord(from, to, PREVIOUS_WORD);
 	if (sl == from || to == from)
 		return docstring();
-	
+
 	Paragraph const & par = sl.paragraph();
 	return par.asString(from.pos(), to.pos());
 }
@@ -2061,16 +2126,16 @@ CompletionList const * Text::createCompletionList(Cursor const & cur) const
 
 
 bool Text::insertCompletion(Cursor & cur, docstring const & s, bool /*finished*/)
-{	
-	LASSERT(cur.bv().cursor() == cur, /**/);
+{
+	LBUFERR(cur.bv().cursor() == cur);
 	cur.insert(s);
 	cur.bv().cursor() = cur;
 	if (!(cur.result().screenUpdate() & Update::Force))
 		cur.screenUpdateFlags(cur.result().screenUpdate() | Update::SinglePar);
 	return true;
 }
-	
-	
+
+
 docstring Text::completionPrefix(Cursor const & cur) const
 {
 	return previousWord(cur.top());
