@@ -51,6 +51,7 @@
 
 #include "support/debug.h"
 #include "support/docstring_list.h"
+#include "support/gettext.h"
 #include "support/lassert.h"
 
 #include <cstdlib>
@@ -128,7 +129,7 @@ static int numberOfHfills(Paragraph const & par, Row const & row)
 TextMetrics::TextMetrics(BufferView * bv, Text * text)
 	: bv_(bv), text_(text)
 {
-	LASSERT(bv_, /**/);
+	LBUFERR(bv_);
 	max_width_ = bv_->workWidth();
 	dim_.wid = max_width_;
 	dim_.asc = 10;
@@ -158,7 +159,7 @@ pair<pit_type, ParagraphMetrics const *> TextMetrics::first() const
 
 pair<pit_type, ParagraphMetrics const *> TextMetrics::last() const
 {
-	LASSERT(!par_metrics_.empty(), /**/);
+	LBUFERR(!par_metrics_.empty());
 	ParMetricsCache::const_reverse_iterator it = par_metrics_.rbegin();
 	return make_pair(it->first, &it->second);
 }
@@ -179,7 +180,7 @@ ParagraphMetrics & TextMetrics::parMetrics(pit_type pit, bool redo)
 
 bool TextMetrics::metrics(MetricsInfo & mi, Dimension & dim, int min_width)
 {
-	LASSERT(mi.base.textwidth > 0, /**/);
+	LBUFERR(mi.base.textwidth > 0);
 	max_width_ = mi.base.textwidth;
 	// backup old dimension.
 	Dimension const old_dim = dim_;
@@ -233,13 +234,13 @@ void TextMetrics::applyOuterFont(Font & font) const
 {
 	FontInfo lf(font_.fontInfo());
 	lf.reduce(bv_->buffer().params().getFont().fontInfo());
-	font.fontInfo().realize(lf); 
+	font.fontInfo().realize(lf);
 }
 
 
 Font TextMetrics::displayFont(pit_type pit, pos_type pos) const
 {
-	LASSERT(pos >= 0, /**/);
+	LASSERT(pos >= 0, { static Font f; return f; });
 
 	ParagraphList const & pars = text_->paragraphs();
 	Paragraph const & par = pars[pit];
@@ -345,19 +346,19 @@ bool TextMetrics::isRTLBoundary(pit_type pit, pos_type pos,
 	// no RTL boundary at line break:
 	// abc|\n    -> move right ->   abc\n       (and not:    abc\n|
 	// FED                          FED|                     FED     )
-	if (startpos == pos && endpos == pos && endpos != par.size() 
-		&& (par.isNewline(pos - 1) 
-			|| par.isLineSeparator(pos - 1) 
+	if (startpos == pos && endpos == pos && endpos != par.size()
+		&& (par.isNewline(pos - 1)
+			|| par.isLineSeparator(pos - 1)
 			|| par.isSeparator(pos - 1)))
 		return false;
-	
+
 	bool left = font.isVisibleRightToLeft();
 	bool right;
 	if (pos == par.size())
 		right = par.isRTL(bv_->buffer().params());
 	else
 		right = displayFont(pit, pos).isVisibleRightToLeft();
-	
+
 	return left != right;
 }
 
@@ -416,7 +417,7 @@ bool TextMetrics::redoParagraph(pit_type const pit)
 		// should be.
 		bv_->buffer().updateBuffer();
 		parPos = text_->macrocontextPosition();
-		LASSERT(!parPos.empty(), /**/);
+		LBUFERR(!parPos.empty());
 		parPos.pit() = pit;
 	}
 
@@ -580,7 +581,7 @@ void TextMetrics::computeRowMetrics(pit_type const pit,
 			case LYX_ALIGN_CENTER:
 			case LYX_ALIGN_LEFT:
 			case LYX_ALIGN_RIGHT:
-				if (align == LYX_ALIGN_NONE 
+				if (align == LYX_ALIGN_NONE
 				    || align == LYX_ALIGN_BLOCK)
 					align = owner.contentAlignment();
 				break;
@@ -607,6 +608,11 @@ void TextMetrics::computeRowMetrics(pit_type const pit,
 			}
 		}
 
+		// Has the user requested we not justify stuff?
+		if (!bv_->buffer().params().justification
+		    && align == LYX_ALIGN_BLOCK)
+			align = LYX_ALIGN_LEFT;
+
 		switch (align) {
 		case LYX_ALIGN_BLOCK: {
 			int const ns = numberOfSeparators(par, row);
@@ -687,7 +693,7 @@ int TextMetrics::labelFill(pit_type const pit, Row const & row) const
 	Paragraph const & par = text_->getPar(pit);
 
 	pos_type last = par.beginOfBody();
-	LASSERT(last > 0, /**/);
+	LBUFERR(last > 0);
 
 	// -1 because a label ends with a space that is in the label
 	--last;
@@ -712,7 +718,7 @@ int TextMetrics::labelFill(pit_type const pit, Row const & row) const
 
 
 #if 0
-// Not used, see TextMetrics::rowBreakPoint. 
+// Not used, see TextMetrics::rowBreakPoint.
 // this needs special handling - only newlines count as a break point
 static pos_type addressBreakPoint(pos_type i, Paragraph const & par)
 {
@@ -765,7 +771,7 @@ public:
 	FontIterator & operator++()
 	{
 		++pos_;
-		if (pos_ > endspan_ || pos_ == bodypos_) {
+		if (pos_ < par_.size() && (pos_ > endspan_ || pos_ == bodypos_)) {
 			font_ = tm_.displayFont(pit_, pos_);
 			endspan_ = par_.fontSpan(pos_).last;
 		}
@@ -806,7 +812,7 @@ pos_type TextMetrics::rowBreakPoint(int width, pit_type const pit,
 	Layout const & layout = par.layout();
 
 #if 0
-	//FIXME: As long as leftMargin() is not correctly implemented for 
+	//FIXME: As long as leftMargin() is not correctly implemented for
 	// MARGIN_RIGHT_ADDRESS_BOX, we should also not do this here.
 	// Otherwise, long rows will be painted off the screen.
 	if (layout.margintype == MARGIN_RIGHT_ADDRESS_BOX)
@@ -847,10 +853,10 @@ pos_type TextMetrics::rowBreakPoint(int width, pit_type const pit,
 	ParagraphList const & pars_ = text_->paragraphs();
 	bool const draw_par_end_marker = lyxrc.paragraph_markers
 		&& size_type(pit + 1) < pars_.size();
-				
+
 	for ( ; i < end; ++i, ++fi) {
 		int thiswidth = pm.singleWidth(i, *fi);
-		
+
 		if (draw_par_end_marker && i == end - 1)
 			// enlarge the last character to hold the end-of-par marker
 			thiswidth += par_marker_width;
@@ -967,8 +973,8 @@ int TextMetrics::rowWidth(int right_margin, pit_type const pit,
 					w -= singleWidth(pit, i - 1);
 				w = max(w, label_end);
 			}
-			
-			// a line separator at the end of a line (but not at the end of a 
+
+			// a line separator at the end of a line (but not at the end of a
 			// paragraph) will not be drawn and should therefore not count for
 			// the row width.
 			if (!par.isLineSeparator(i) || i != end - 1 || end == par.size())
@@ -1097,20 +1103,9 @@ Dimension TextMetrics::rowHeight(pit_type const pit, pos_type const first,
 		if (par.params().startOfAppendix())
 			maxasc += int(3 * dh);
 
-		// This is special code for the chapter, since the label of this
-		// layout is printed in an extra row
-		if (layout.counter == "chapter"
-		    && !par.params().labelString().empty()) {
-			labeladdon = int(labelfont_metrics.maxHeight()
-				     * layout.spacing.getValue()
-				     * text_->spacing(par));
-		}
-
 		// special code for the top label
-		if ((layout.labeltype == LABEL_TOP_ENVIRONMENT
-		     || layout.labeltype == LABEL_BIBLIO
-		     || layout.labeltype == LABEL_CENTERED_TOP_ENVIRONMENT)
-		    && text_->isFirstInSequence(pit)
+		if (layout.labelIsAbove()
+		    && (!layout.isParagraphGroup() || text_->isFirstInSequence(pit))
 		    && !par.labelString().empty())
 		{
 			labeladdon = int(
@@ -1243,7 +1238,7 @@ pos_type TextMetrics::getColumnNearX(pit_type const pit,
 
 	// If lastrow is false, we don't need to compute
 	// the value of rtl.
-	bool const rtl = lastrow ? text_->isRTL(par) : false;
+	bool const rtl_on_lastrow = lastrow ? text_->isRTL(par) : false;
 
 	while (vc < end && tmpx <= x) {
 		c = bidi.vis2log(vc);
@@ -1267,13 +1262,14 @@ pos_type TextMetrics::getColumnNearX(pit_type const pit,
 		left_side = true;
 	}
 
-	LASSERT(vc <= end, /**/);  // This shouldn't happen.
+	// This shouldn't happen. But we can reset and try to continue.
+	LASSERT(vc <= end, vc = end);
 
 	boundary = false;
 
 	if (lastrow &&
-	    ((rtl  &&  left_side && vc == row.pos() && x < tmpx - 5) ||
-	     (!rtl && !left_side && vc == end  && x > tmpx + 5))) {
+	    ((rtl_on_lastrow  &&  left_side && vc == row.pos() && x < tmpx - 5) ||
+	     (!rtl_on_lastrow && !left_side && vc == end  && x > tmpx + 5))) {
 		if (!par.isNewline(end - 1))
 			c = end;
 	} else if (vc == row.pos()) {
@@ -1340,7 +1336,7 @@ pos_type TextMetrics::x2pos(pit_type pit, int row, int x) const
 	// upDownInText() while in selection mode.
 	ParagraphMetrics const & pm = parMetrics(pit);
 
-	LASSERT(row < int(pm.rows().size()), /**/);
+	LBUFERR(row < int(pm.rows().size()));
 	bool bound = false;
 	Row const & r = pm.rows()[row];
 	return r.pos() + getColumnNearX(pit, r, x, bound);
@@ -1443,7 +1439,7 @@ Row const & TextMetrics::getPitAndRowNearY(int & y, pit_type & pit,
 	ParagraphMetrics const & pm = par_metrics_[pit];
 
 	int yy = pm.position() - pm.ascent();
-	LASSERT(!pm.rows().empty(), /**/);
+	LBUFERR(!pm.rows().empty());
 	RowList::const_iterator rit = pm.rows().begin();
 	RowList::const_iterator rlast = pm.rows().end();
 	--rlast;
@@ -1492,7 +1488,7 @@ Inset * TextMetrics::editXY(Cursor & cur, int x, int y,
 	}
 	pit_type pit = getPitNearY(y);
 	LASSERT(pit != -1, return 0);
-	
+
 	int yy = y; // is modified by getPitAndRowNearY
 	Row const & row = getPitAndRowNearY(yy, pit, assert_in_view, up);
 
@@ -1509,10 +1505,6 @@ Inset * TextMetrics::editXY(Cursor & cur, int x, int y,
 	Inset * inset = checkInsetHit(x, yy);
 	//lyxerr << "inset " << inset << " hit at x: " << x << " y: " << y << endl;
 	if (!inset) {
-		// Either we deconst editXY or better we move current_font
-		// and real_current_font to Cursor
-		// FIXME: what is needed now that current_font and real_current_font
-		// are transferred?
 		cur.setCurrentFont();
 		return 0;
 	}
@@ -1522,7 +1514,7 @@ Inset * TextMetrics::editXY(Cursor & cur, int x, int y,
 
 	// This should be just before or just behind the
 	// cursor position set above.
-	LASSERT(inset == inset_before 
+	LASSERT(inset == inset_before
 		|| inset == pars[pit].getInset(pos), return 0);
 
 	// Make sure the cursor points to the position before
@@ -1543,8 +1535,8 @@ Inset * TextMetrics::editXY(Cursor & cur, int x, int y,
 
 void TextMetrics::setCursorFromCoordinates(Cursor & cur, int const x, int const y)
 {
-	LASSERT(text_ == cur.text(), /**/);
-	pit_type pit = getPitNearY(y);
+	LASSERT(text_ == cur.text(), return);
+	pit_type const pit = getPitNearY(y);
 	LASSERT(pit != -1, return);
 
 	ParagraphMetrics const & pm = par_metrics_[pit];
@@ -1554,7 +1546,7 @@ void TextMetrics::setCursorFromCoordinates(Cursor & cur, int const x, int const
 		" pit: " << pit << " yy: " << yy);
 
 	int r = 0;
-	LASSERT(pm.rows().size(), /**/);
+	LBUFERR(pm.rows().size());
 	for (; r < int(pm.rows().size()) - 1; ++r) {
 		Row const & row = pm.rows()[r];
 		if (int(yy + row.height()) > y)
@@ -1624,7 +1616,7 @@ Inset * TextMetrics::checkInsetHit(int x, int y)
 int TextMetrics::cursorX(CursorSlice const & sl,
 		bool boundary) const
 {
-	LASSERT(sl.text() == text_, /**/);
+	LASSERT(sl.text() == text_, return 0);
 	pit_type const pit = sl.pit();
 	Paragraph const & par = text_->paragraphs()[pit];
 	ParagraphMetrics const & pm = par_metrics_[pit];
@@ -1799,7 +1791,7 @@ int TextMetrics::cursorY(CursorSlice const & sl, bool boundary) const
 
 bool TextMetrics::cursorHome(Cursor & cur)
 {
-	LASSERT(text_ == cur.text(), /**/);
+	LASSERT(text_ == cur.text(), return false);
 	ParagraphMetrics const & pm = par_metrics_[cur.pit()];
 	Row const & row = pm.getRow(cur.pos(),cur.boundary());
 	return text_->setCursor(cur, cur.pit(), row.pos());
@@ -1808,7 +1800,7 @@ bool TextMetrics::cursorHome(Cursor & cur)
 
 bool TextMetrics::cursorEnd(Cursor & cur)
 {
-	LASSERT(text_ == cur.text(), /**/);
+	LASSERT(text_ == cur.text(), return false);
 	// if not on the last row of the par, put the cursor before
 	// the final space exept if I have a spanning inset or one string
 	// is so long that we force a break.
@@ -1830,7 +1822,7 @@ bool TextMetrics::cursorEnd(Cursor & cur)
 
 void TextMetrics::deleteLineForward(Cursor & cur)
 {
-	LASSERT(text_ == cur.text(), /**/);
+	LASSERT(text_ == cur.text(), return);
 	if (cur.lastpos() == 0) {
 		// Paragraph is empty, so we just go forward
 		text_->cursorForward(cur);
@@ -1865,8 +1857,6 @@ bool TextMetrics::isFirstRow(pit_type pit, Row const & row) const
 
 int TextMetrics::leftMargin(int max_width, pit_type pit) const
 {
-	LASSERT(pit >= 0, /**/);
-	LASSERT(pit < int(text_->paragraphs().size()), /**/);
 	return leftMargin(max_width, pit, text_->paragraphs()[pit].size());
 }
 
@@ -1876,11 +1866,11 @@ int TextMetrics::leftMargin(int max_width,
 {
 	ParagraphList const & pars = text_->paragraphs();
 
-	LASSERT(pit >= 0, /**/);
-	LASSERT(pit < int(pars.size()), /**/);
+	LASSERT(pit >= 0, return 0);
+	LASSERT(pit < int(pars.size()), return 0);
 	Paragraph const & par = pars[pit];
-	LASSERT(pos >= 0, /**/);
-	LASSERT(pos <= par.size(), /**/);
+	LASSERT(pos >= 0, return 0);
+	LASSERT(pos <= par.size(), return 0);
 	Buffer const & buffer = bv_->buffer();
 	//lyxerr << "TextMetrics::leftMargin: pit: " << pit << " pos: " << pos << endl;
 	DocumentClass const & tclass = buffer.params().documentClass();
@@ -1969,10 +1959,7 @@ int TextMetrics::leftMargin(int max_width,
 			       && layout.latextype == LATEX_ENVIRONMENT
 			       && !text_->isFirstInSequence(pit))) {
 			l_margin += labelfont_metrics.signedWidth(layout.leftmargin);
-		} else if (layout.labeltype != LABEL_TOP_ENVIRONMENT
-			   && layout.labeltype != LABEL_BIBLIO
-			   && layout.labeltype !=
-			   LABEL_CENTERED_TOP_ENVIRONMENT) {
+		} else if (!layout.labelIsAbove()) {
 			l_margin += labelfont_metrics.signedWidth(layout.labelindent);
 			l_margin += labelfont_metrics.width(layout.labelsep);
 			l_margin += labelfont_metrics.width(par.labelString());
@@ -1981,16 +1968,18 @@ int TextMetrics::leftMargin(int max_width,
 
 	case MARGIN_RIGHT_ADDRESS_BOX: {
 #if 0
-		// ok, a terrible hack. The left margin depends on the widest
-		// row in this paragraph.
-		RowList::iterator rit = par.rows().begin();
-		RowList::iterator end = par.rows().end();
-		// FIXME: This is wrong.
+		// The left margin depends on the widest row in this paragraph.
+		// This code is wrong because it depends on the rows, but at the
+		// same time this function is used in redoParagraph to construct
+		// the rows.
+		ParagraphMetrics const & pm = par_metrics_[pit];
+		RowList::const_iterator rit = pm.rows().begin();
+		RowList::const_iterator end = pm.rows().end();
 		int minfill = max_width;
 		for ( ; rit != end; ++rit)
 			if (rit->fill() < minfill)
 				minfill = rit->fill();
-		l_margin += theFontMetrics(params.getFont()).signedWidth(layout.leftmargin);
+		l_margin += theFontMetrics(buffer.params().getFont()).signedWidth(layout.leftmargin);
 		l_margin += minfill;
 #endif
 		// also wrong, but much shorter.
@@ -2012,8 +2001,8 @@ int TextMetrics::leftMargin(int max_width,
 	// set the correct parindent
 	if (pos == 0
 	    && (layout.labeltype == LABEL_NO_LABEL
-	       || layout.labeltype == LABEL_TOP_ENVIRONMENT
-	       || layout.labeltype == LABEL_CENTERED_TOP_ENVIRONMENT
+	       || layout.labeltype == LABEL_ABOVE
+	       || layout.labeltype == LABEL_CENTERED
 	       || (layout.labeltype == LABEL_STATIC
 	           && layout.latextype == LATEX_ENVIRONMENT
 	           && !text_->isFirstInSequence(pit)))
@@ -2027,7 +2016,7 @@ int TextMetrics::leftMargin(int max_width,
 		    && par.getInset(pos)->display())
 			&& (!(tclass.isDefaultLayout(par.layout())
 	         || tclass.isPlainLayout(par.layout()))
-	        || buffer.params().paragraph_separation 
+	        || buffer.params().paragraph_separation
 				== BufferParams::ParagraphIndentSeparation)
 	    )
 		{
@@ -2039,7 +2028,7 @@ int TextMetrics::leftMargin(int max_width,
 			else
 				l_margin += buffer.params().getIndentation().inPixels(*bv_);
 		}
-	
+
 	return l_margin;
 }
 
@@ -2100,7 +2089,7 @@ void TextMetrics::drawParagraph(PainterInfo & pi, pit_type pit, int x, int y) co
 	// We store the begin and end pos of the selection relative to this par
 	DocIterator sel_beg_par = cur.selectionBegin();
 	DocIterator sel_end_par = cur.selectionEnd();
-	
+
 	// We care only about visible selection.
 	if (selection) {
 		if (pit != sel_beg.pit()) {
@@ -2129,7 +2118,7 @@ void TextMetrics::drawParagraph(PainterInfo & pi, pit_type pit, int x, int y) co
 			row.setSelectionAndMargins(sel_beg_par, sel_end_par);
 		else
 			row.setSelection(-1, -1);
-		
+
 		// The row knows nothing about the paragraph, so we have to check
 		// whether this row is the first or last and update the margins.
 		if (row.selection()) {
@@ -2164,7 +2153,7 @@ void TextMetrics::drawParagraph(PainterInfo & pi, pit_type pit, int x, int y) co
 			pi.pain.fillRectangle(x, y - row.ascent(),
 				width(), row.height(), pi.background_color);
 		}
-		
+
 		// Instrumentation for testing row cache (see also
 		// 12 lines lower):
 		if (lyxerr.debugging(Debug::PAINTING) && inside
