@@ -20,7 +20,7 @@
 #include "Buffer.h"
 #include "BufferParams.h"
 #include "buffer_funcs.h"
-#include "DocIterator.h"
+#include "Cursor.h"
 #include "Paragraph.h"
 #include "ParagraphList.h"
 #include "Text.h"
@@ -30,8 +30,9 @@
 
 #include "insets/Inset.h"
 
-#include "support/lassert.h"
 #include "support/debug.h"
+#include "support/gettext.h"
+#include "support/lassert.h"
 
 #include <algorithm>
 #include <deque>
@@ -60,15 +61,15 @@ structure ('Undo') _before_ it is changed in some edit operation.
 Obviously, the stored range should be as small as possible. However,
 there is a lower limit: The StableDocIterator stored in the undo class
 must be valid after the changes, too, as it will used as a pointer
-where to insert the stored bits when performining undo. 
+where to insert the stored bits when performining undo.
 */
 struct UndoElement
 {
 	///
-	UndoElement(UndoKind kin, StableDocIterator const & cb, 
+	UndoElement(UndoKind kin, CursorData const & cb,
 	            StableDocIterator const & cel,
-	            pit_type fro, pit_type en, ParagraphList * pl, 
-	            MathData * ar, BufferParams const & bp, 
+	            pit_type fro, pit_type en, ParagraphList * pl,
+	            MathData * ar, BufferParams const & bp,
 	            bool ifb, bool lc, size_t gid) :
 	        kind(kin), cur_before(cb), cell(cel), from(fro), end(en),
 	        pars(pl), array(ar), bparams(0), isFullBuffer(ifb),
@@ -103,9 +104,9 @@ struct UndoElement
 	/// Which kind of operation are we recording for?
 	UndoKind kind;
 	/// the position of the cursor before recordUndo
-	StableDocIterator cur_before;
+	CursorData cur_before;
 	/// the position of the cursor at the end of the undo group
-	StableDocIterator cur_after;
+	CursorData cur_after;
 	/// the position of the cell described
 	StableDocIterator cell;
 	/// counted from begin of cell
@@ -126,11 +127,11 @@ struct UndoElement
 	size_t group_id;
 private:
 	/// Protect construction
-	UndoElement();	
+	UndoElement();
 };
 
 
-class UndoElementStack 
+class UndoElementStack
 {
 public:
 	/// limit is the maximum size of the stack
@@ -172,7 +173,7 @@ public:
 	void markDirty() {
 		for (size_t i = 0; i != c_.size(); ++i)
 			c_[i].lyx_clean = false;
-	}		
+	}
 
 private:
 	/// Internal contents.
@@ -184,21 +185,21 @@ private:
 
 struct Undo::Private
 {
-	Private(Buffer & buffer) : buffer_(buffer), undo_finished_(true), 
+	Private(Buffer & buffer) : buffer_(buffer), undo_finished_(true),
 				   group_id(0), group_level(0) {}
-	
+
 	// Do one undo/redo step
-	void doTextUndoOrRedo(DocIterator & cur, UndoElementStack & stack, 
+	void doTextUndoOrRedo(CursorData & cur, UndoElementStack & stack,
 			      UndoElementStack & otherStack);
 	// Apply one undo/redo group. Returns false if no undo possible.
-	bool textUndoOrRedo(DocIterator & cur, bool isUndoOperation);
+	bool textUndoOrRedo(CursorData & cur, bool isUndoOperation);
 
 	///
 	void doRecordUndo(UndoKind kind,
 		DocIterator const & cell,
 		pit_type first_pit,
 		pit_type last_pit,
-		StableDocIterator const & cur,
+		CursorData const & cur,
 		bool isFullBuffer,
 		UndoElementStack & stack);
 	///
@@ -206,7 +207,7 @@ struct Undo::Private
 		DocIterator const & cell,
 		pit_type first_pit,
 		pit_type last_pit,
-		DocIterator const & cur,
+		CursorData const & cur,
 		bool isFullBuffer);
 
 	///
@@ -272,7 +273,7 @@ void Undo::markDirty()
 {
 	d->undo_finished_ = true;
 	d->undostack_.markDirty();
-	d->redostack_.markDirty();	
+	d->redostack_.markDirty();
 }
 
 
@@ -293,7 +294,7 @@ static bool samePar(StableDocIterator const & i1, StableDocIterator const & i2)
 void Undo::Private::doRecordUndo(UndoKind kind,
 	DocIterator const & cell,
 	pit_type first_pit, pit_type last_pit,
-	StableDocIterator const & cur_before,
+	CursorData const & cur_before,
 	bool isFullBuffer,
 	UndoElementStack & stack)
 {
@@ -318,7 +319,7 @@ void Undo::Private::doRecordUndo(UndoKind kind,
 	    && stack.top().from == from
 	    && stack.top().end == end) {
 		// reset cur_after; it will be filled correctly by endUndoGroup.
-		stack.top().cur_after = StableDocIterator();
+		stack.top().cur_after = CursorData();
 		return;
 	}
 
@@ -327,7 +328,7 @@ void Undo::Private::doRecordUndo(UndoKind kind,
 	else
 		LYXERR(Debug::UNDO, "Create undo element of group " << group_id);
 	// create the position information of the Undo entry
-	UndoElement undo(kind, cur_before, cell, from, end, 0, 0, 
+	UndoElement undo(kind, cur_before, cell, from, end, 0, 0,
 	                 buffer_.params(), isFullBuffer, buffer_.isClean(), group_id);
 
 	// fill in the real data to be saved
@@ -340,7 +341,7 @@ void Undo::Private::doRecordUndo(UndoKind kind,
 		// main Text _is_ the whole document.
 		// record the relevant paragraphs
 		Text const * text = cell.text();
-		LASSERT(text, /**/);
+		LBUFERR(text);
 		ParagraphList const & plist = text->paragraphs();
 		ParagraphList::const_iterator first = plist.begin();
 		advance(first, first_pit);
@@ -358,11 +359,11 @@ void Undo::Private::doRecordUndo(UndoKind kind,
 void Undo::Private::recordUndo(UndoKind kind,
 			       DocIterator const & cell,
 			       pit_type first_pit, pit_type last_pit,
-			       DocIterator const & cur,
+			       CursorData const & cur,
 			       bool isFullBuffer)
 {
-	LASSERT(first_pit <= cell.lastpit(), /**/);
-	LASSERT(last_pit <= cell.lastpit(), /**/);
+	LASSERT(first_pit <= cell.lastpit(), return);
+	LASSERT(last_pit <= cell.lastpit(), return);
 
 	doRecordUndo(kind, cell, first_pit, last_pit, cur,
 		isFullBuffer, undostack_);
@@ -380,7 +381,7 @@ void Undo::Private::recordUndo(UndoKind kind,
 }
 
 
-void Undo::Private::doTextUndoOrRedo(DocIterator & cur, UndoElementStack & stack, UndoElementStack & otherstack)
+void Undo::Private::doTextUndoOrRedo(CursorData & cur, UndoElementStack & stack, UndoElementStack & otherstack)
 {
 	// Adjust undo stack and get hold of current undo data.
 	UndoElement & undo = stack.top();
@@ -400,7 +401,7 @@ void Undo::Private::doTextUndoOrRedo(DocIterator & cur, UndoElementStack & stack
 	//LYXERR0("undo, performing: " << undo);
 	DocIterator dit = undo.cell.asDocIterator(&buffer_);
 	if (undo.isFullBuffer) {
-		LASSERT(undo.pars, /**/);
+		LBUFERR(undo.pars);
 		// This is a full document
 		delete otherstack.top().bparams;
 		otherstack.top().bparams = new BufferParams(buffer_.params());
@@ -413,15 +414,15 @@ void Undo::Private::doTextUndoOrRedo(DocIterator & cur, UndoElementStack & stack
 		// gained by storing just 'a few' paragraphs (most if not
 		// all math inset cells have just one paragraph!)
 		//LYXERR0("undo.array: " << *undo.array);
-		LASSERT(undo.array, /**/);
+		LBUFERR(undo.array);
 		dit.cell().swap(*undo.array);
 		delete undo.array;
 		undo.array = 0;
 	} else {
 		// Some finer machinery is needed here.
 		Text * text = dit.text();
-		LASSERT(text, /**/);
-		LASSERT(undo.pars, /**/);
+		LBUFERR(text);
+		LBUFERR(undo.pars);
 		ParagraphList & plist = text->paragraphs();
 
 		// remove new stuff between first and last
@@ -445,11 +446,13 @@ void Undo::Private::doTextUndoOrRedo(DocIterator & cur, UndoElementStack & stack
 		delete undo.pars;
 		undo.pars = 0;
 	}
-	LASSERT(undo.pars == 0, /**/);
-	LASSERT(undo.array == 0, /**/);
 
-	if (undo.cur_before.size())
-		cur = undo.cur_before.asDocIterator(&buffer_);
+	// We'll clean up in release mode.
+	LASSERT(undo.pars == 0, undo.pars = 0);
+	LASSERT(undo.array == 0, undo.array = 0);
+
+	if (!undo.cur_before.empty())
+		cur = undo.cur_before;
 	if (undo.lyx_clean)
 		buffer_.markClean();
 	else
@@ -459,7 +462,7 @@ void Undo::Private::doTextUndoOrRedo(DocIterator & cur, UndoElementStack & stack
 }
 
 
-bool Undo::Private::textUndoOrRedo(DocIterator & cur, bool isUndoOperation)
+bool Undo::Private::textUndoOrRedo(CursorData & cur, bool isUndoOperation)
 {
 	undo_finished_ = true;
 
@@ -488,13 +491,13 @@ void Undo::finishUndo()
 }
 
 
-bool Undo::textUndo(DocIterator & cur)
+bool Undo::textUndo(CursorData & cur)
 {
 	return d->textUndoOrRedo(cur, true);
 }
 
 
-bool Undo::textRedo(DocIterator & cur)
+bool Undo::textRedo(CursorData & cur)
 {
 	return d->textUndoOrRedo(cur, false);
 }
@@ -525,10 +528,10 @@ void Undo::endUndoGroup()
 }
 
 
-void Undo::endUndoGroup(DocIterator const & cur)
+void Undo::endUndoGroup(CursorData const & cur)
 {
 	endUndoGroup();
-	if (!d->undostack_.empty() && !d->undostack_.top().cur_after.size())
+	if (!d->undostack_.empty() && d->undostack_.top().cur_after.empty())
 		d->undostack_.top().cur_after = cur;
 }
 
@@ -537,13 +540,13 @@ void Undo::endUndoGroup(DocIterator const & cur)
 // Private::recordUndo public as sole interface. The code in the
 // convenience functions can move to Cursor.cpp.
 
-void Undo::recordUndo(DocIterator const & cur, UndoKind kind)
+void Undo::recordUndo(CursorData const & cur, UndoKind kind)
 {
 	d->recordUndo(kind, cur, cur.pit(), cur.pit(), cur, false);
 }
 
 
-void Undo::recordUndoInset(DocIterator const & cur, UndoKind kind,
+void Undo::recordUndoInset(CursorData const & cur, UndoKind kind,
 			   Inset const * inset)
 {
 	if (!inset || inset == &cur.inset()) {
@@ -557,25 +560,25 @@ void Undo::recordUndoInset(DocIterator const & cur, UndoKind kind,
 }
 
 
-void Undo::recordUndo(DocIterator const & cur, UndoKind kind, pit_type from)
+void Undo::recordUndo(CursorData const & cur, UndoKind kind, pit_type from)
 {
 	d->recordUndo(kind, cur, cur.pit(), from, cur, false);
 }
 
 
-void Undo::recordUndo(DocIterator const & cur, UndoKind kind,
+void Undo::recordUndo(CursorData const & cur, UndoKind kind,
 	pit_type from, pit_type to)
 {
 	d->recordUndo(kind, cur, from, to, cur, false);
 }
 
 
-void Undo::recordUndoFullDocument(DocIterator const & cur)
+void Undo::recordUndoFullDocument(CursorData const & cur)
 {
 	// This one may happen outside of the main undo group, so we
 	// put it in its own subgroup to avoid complaints.
 	beginUndoGroup();
-	d->recordUndo(ATOMIC_UNDO, doc_iterator_begin(&d->buffer_), 
+	d->recordUndo(ATOMIC_UNDO, doc_iterator_begin(&d->buffer_),
 		      0, d->buffer_.paragraphs().size() - 1, cur, true);
 	endUndoGroup();
 }
