@@ -54,7 +54,8 @@ using namespace lyx::support;
 namespace lyx {
 namespace frontend {
 
-static vector<CiteStyle> citeStyles_;
+static vector<string> citeCmds_;
+static vector<CitationStyle> citeStyles_;
 
 
 template<typename String>
@@ -96,17 +97,17 @@ GuiCitation::GuiCitation(GuiView & lv)
 	connect(forceuppercaseCB, SIGNAL(clicked()),
 		this, SLOT(changed()));
 	connect(textBeforeED, SIGNAL(textChanged(QString)),
-		this, SLOT(changed()));
+		this, SLOT(updateStyles()));
 	connect(textAfterED, SIGNAL(textChanged(QString)),
-		this, SLOT(changed()));
-	connect(findLE, SIGNAL(returnPressed()), 
+		this, SLOT(updateStyles()));
+	connect(findLE, SIGNAL(returnPressed()),
 		this, SLOT(on_searchPB_clicked()));
 	connect(textBeforeED, SIGNAL(returnPressed()),
 		this, SLOT(on_okPB_clicked()));
 	connect(textAfterED, SIGNAL(returnPressed()),
 		this, SLOT(on_okPB_clicked()));
 
-	selectionManager = new GuiSelectionManager(availableLV, selectedLV, 
+	selectionManager = new GuiSelectionManager(availableLV, selectedLV,
 			addPB, deletePB, upPB, downPB, &available_model_, &selected_model_);
 	connect(selectionManager, SIGNAL(selectionChanged()),
 		this, SLOT(setCitedKeys()));
@@ -116,9 +117,6 @@ GuiCitation::GuiCitation(GuiView & lv)
 		this, SLOT(on_okPB_clicked()));
 
 	setFocusProxy(availableLV);
-
-	// FIXME: the sizeHint() for this is _way_ too high
-	infoML->setFixedHeight(60);
 }
 
 
@@ -194,86 +192,50 @@ void GuiCitation::updateControls()
 // The main point of separating this out is that the fill*() methods
 // called in update() do not need to be called for INTERNAL updates,
 // such as when addPB is pressed, as the list of fields, entries, etc,
-// will not have changed. At the moment, however, the division between
-// fillStyles() and updateStyle() doesn't lend itself to dividing the
-// two methods, though they should be divisible. That is, we should
-// not have to call fillStyles() every time through here.
+// will not have changed.
 void GuiCitation::updateControls(BiblioInfo const & bi)
 {
 	QModelIndex idx = selectionManager->getSelectedIndex();
 	updateInfo(bi, idx);
-	setButtons();
-
-	textBeforeED->setText(toqstr(params_["before"]));
-	textAfterED->setText(toqstr(params_["after"]));
-	fillStyles(bi);
-	updateStyle();
+	selectionManager->update();
 }
 
 
-void GuiCitation::updateFormatting(CiteStyle currentStyle)
+void GuiCitation::updateFormatting(CitationStyle currentStyle)
 {
-	CiteEngine const engine = citeEngine();
-	bool const natbib_engine =
-		engine == ENGINE_NATBIB_AUTHORYEAR ||
-		engine == ENGINE_NATBIB_NUMERICAL;
-	bool const basic_engine = engine == ENGINE_BASIC;
+	bool const force = currentStyle.forceUpperCase;
+	bool const full = currentStyle.fullAuthorList &&
+		documentBuffer().params().fullAuthorList();
+	bool const textbefore = currentStyle.textBefore;
+	bool const textafter = currentStyle.textAfter;
 
-	bool const haveSelection = 
+	bool const haveSelection =
 		selectedLV->model()->rowCount() > 0;
 
-	bool const isNocite = currentStyle == NOCITE;
-
-	bool const isCiteyear =
-		currentStyle == CITEYEAR ||
-		currentStyle == CITEYEARPAR;
-
-	fulllistCB->setEnabled(natbib_engine && haveSelection && !isNocite
-		&& !isCiteyear);
-	forceuppercaseCB->setEnabled(natbib_engine && haveSelection
-		&& !isNocite && !isCiteyear);
-	textBeforeED->setEnabled(!basic_engine && haveSelection && !isNocite);
-	textBeforeLA->setEnabled(!basic_engine && haveSelection && !isNocite);
-	textAfterED->setEnabled(haveSelection && !isNocite);
-	textAfterLA->setEnabled(haveSelection && !isNocite);
+	forceuppercaseCB->setEnabled(force && haveSelection);
+	fulllistCB->setEnabled(full && haveSelection);
+	textBeforeED->setEnabled(textbefore && haveSelection);
+	textBeforeLA->setEnabled(textbefore && haveSelection);
+	textAfterED->setEnabled(textafter && haveSelection);
+	textAfterLA->setEnabled(textafter && haveSelection);
 	citationStyleCO->setEnabled(haveSelection);
 	citationStyleLA->setEnabled(haveSelection);
 }
 
 
-void GuiCitation::updateStyle()
+// Update the styles for the style combo, citationStyleCO, and mark the
+// settings as changed. Called upon changing the cited keys (including
+// merely reordering the keys) or editing the text before/after fields.
+void GuiCitation::updateStyles()
 {
-	string const & command = params_.getCmdName();
-
-	// Find the style of the citekeys
-	vector<CiteStyle> const & styles = citeStyles_;
-	CitationStyle const cs = citationStyleFromString(command);
-
-	vector<CiteStyle>::const_iterator cit =
-		std::find(styles.begin(), styles.end(), cs.style);
-
-	if (cit != styles.end()) {
-		fulllistCB->setChecked(cs.full);
-		forceuppercaseCB->setChecked(cs.forceUpperCase);
-	} else {
-		// restore the last used natbib style
-		if (style_ >= 0 && style_ < citationStyleCO->count()) {
-			// the necessary update will be performed later
-			citationStyleCO->blockSignals(true);
-			citationStyleCO->setCurrentIndex(style_);
-			citationStyleCO->blockSignals(false);
-		}
-		fulllistCB->setChecked(false);
-		forceuppercaseCB->setChecked(false);
-	}
-	updateFormatting(cs.style);
+	BiblioInfo const & bi = bibInfo();
+	updateStyles(bi);
+	changed();
 }
 
 
-// This one needs to be called whenever citationStyleCO needs
-// to be updated---and this would be on anything that changes the
-// selection in selectedLV, or on a general update.
-void GuiCitation::fillStyles(BiblioInfo const & bi)
+// Update the styles for the style combo, citationStyleCO.
+void GuiCitation::updateStyles(BiblioInfo const & bi)
 {
 	QStringList selected_keys = selected_model_.stringList();
 	int curr = selectedLV->model()->rowCount() - 1;
@@ -288,18 +250,18 @@ void GuiCitation::fillStyles(BiblioInfo const & bi)
 	if (!selectedLV->selectionModel()->selectedIndexes().empty())
 		curr = selectedLV->selectionModel()->selectedIndexes()[0].row();
 
-	QStringList sty = citationStyles(bi, curr);
+	QStringList sty = citationStyles(bi);
 
-	if (sty.isEmpty()) { 
+	if (sty.isEmpty()) {
 		// some error
 		citationStyleCO->setEnabled(false);
 		citationStyleLA->setEnabled(false);
 		citationStyleCO->clear();
 		return;
 	}
-	
+
 	citationStyleCO->blockSignals(true);
-	
+
 	// save old index
 	int const oldIndex = citationStyleCO->currentIndex();
 	citationStyleCO->clear();
@@ -352,7 +314,6 @@ bool GuiCitation::isSelected(QModelIndex const & idx)
 
 void GuiCitation::setButtons()
 {
-	selectionManager->update();
 	int const srows = selectedLV->model()->rowCount();
 	applyPB->setEnabled(srows > 0);
 	okPB->setEnabled(srows > 0);
@@ -367,7 +328,7 @@ void GuiCitation::updateInfo(BiblioInfo const & bi, QModelIndex const & idx)
 	}
 
 	QString const keytxt = toqstr(
-		bi.getInfo(qstring_to_ucs4(idx.data().toString()), buffer(), true));
+		bi.getInfo(qstring_to_ucs4(idx.data().toString()), documentBuffer(), true));
 	infoML->document()->setHtml(keytxt);
 }
 
@@ -375,38 +336,38 @@ void GuiCitation::updateInfo(BiblioInfo const & bi, QModelIndex const & idx)
 void GuiCitation::findText(QString const & text, bool reset)
 {
 	//"All Fields" and "Keys" are the first two
-	int index = fieldsCO->currentIndex() - 2; 
+	int index = fieldsCO->currentIndex() - 2;
 	BiblioInfo const & bi = bibInfo();
 	vector<docstring> const & fields = bi.getFields();
 	docstring field;
-	
+
 	if (index <= -1 || index >= int(fields.size()))
 		//either "All Fields" or "Keys" or an invalid value
 		field = from_ascii("");
 	else
 		field = fields[index];
-	
+
 	//Was it "Keys"?
 	bool const onlyKeys = index == -1;
-	
+
 	//"All Entry Types" is first.
-	index = entriesCO->currentIndex() - 1; 
+	index = entriesCO->currentIndex() - 1;
 	vector<docstring> const & entries = bi.getEntries();
 	docstring entry_type;
 	if (index < 0 || index >= int(entries.size()))
 		entry_type = from_ascii("");
-	else 
+	else
 		entry_type = entries[index];
-	
+
 	bool const case_sentitive = caseCB->checkState();
 	bool const reg_exp = regexCB->checkState();
-	findKey(bi, text, onlyKeys, field, entry_type, 
+	findKey(bi, text, onlyKeys, field, entry_type,
 	               case_sentitive, reg_exp, reset);
 	//FIXME
-	//It'd be nice to save and restore the current selection in 
+	//It'd be nice to save and restore the current selection in
 	//availableLV. Currently, we get an automatic reset, since the
 	//model is reset.
-	
+
 	updateControls(bi);
 }
 
@@ -426,8 +387,9 @@ void GuiCitation::on_entriesCO_currentIndexChanged(int /*index*/)
 void GuiCitation::on_citationStyleCO_currentIndexChanged(int index)
 {
 	if (index >= 0 && index < citationStyleCO->count()) {
-		vector<CiteStyle> const & styles = citeStyles_;
+		vector<CitationStyle> const & styles = citeStyles_;
 		updateFormatting(styles[index]);
+		changed();
 	}
 }
 
@@ -484,19 +446,18 @@ void GuiCitation::apply(int const choice, bool full, bool force,
 	if (cited_keys_.isEmpty())
 		return;
 
-	vector<CiteStyle> const & styles = citeStyles_;
-	if (styles[choice] == NOCITE) {
-		full = false;
-		force = false;
+	vector<CitationStyle> const & styles = citeStyles_;
+
+	CitationStyle cs = styles[choice];
+
+	if (!cs.textBefore)
 		before.clear();
+	if (!cs.textAfter)
 		after.clear();
-	}
-	
-	CitationStyle s;
-	s.style = styles[choice];
-	s.full = full;
-	s.forceUpperCase = force;
-	string const command = citationStyleToString(s);
+
+	cs.forceUpperCase &= force;
+	cs.fullAuthorList &= full;
+	string const command = citationStyleToString(cs);
 
 	params_.setCmdName(command);
 	params_["key"] = qstring_to_ucs4(cited_keys_.join(","));
@@ -527,33 +488,49 @@ void GuiCitation::init()
 	else
 		cited_keys_ = str.split(",");
 	selected_model_.setStringList(cited_keys_);
+
+	// Initialize the drop downs
+	fillEntries(bi);
+	fillFields(bi);
+
+	// Initialize the citation formatting
+	string const & cmd = params_.getCmdName();
+	CitationStyle const cs = citationStyleFromString(cmd);
+	forceuppercaseCB->setChecked(cs.forceUpperCase);
+	fulllistCB->setChecked(cs.fullAuthorList &&
+		documentBuffer().params().fullAuthorList());
+	textBeforeED->setText(toqstr(params_["before"]));
+	textAfterED->setText(toqstr(params_["after"]));
+
+	// Update the interface
+	updateControls(bi);
+	updateStyles(bi);
 	if (selected_model_.rowCount()) {
 		selectedLV->blockSignals(true);
 		selectedLV->setFocus();
 		QModelIndex idx = selected_model_.index(0, 0);
-		selectedLV->selectionModel()->select(idx, 
+		selectedLV->selectionModel()->select(idx,
 				QItemSelectionModel::ClearAndSelect);
 		selectedLV->blockSignals(false);
-		
-		// set the style combo appropriately
-		string const & command = params_.getCmdName();
-		vector<CiteStyle> const & styles = citeStyles_;
-		CitationStyle const cs = citationStyleFromString(command);
-	
-		vector<CiteStyle>::const_iterator cit =
-			std::find(styles.begin(), styles.end(), cs.style);
-		if (cit != styles.end()) {
-			int const i = int(cit - styles.begin());
-			// the necessary update will be performed later
-			citationStyleCO->blockSignals(true);
-			citationStyleCO->setCurrentIndex(i);
-			citationStyleCO->blockSignals(false);
-		}
+
+		// Find the citation style
+		vector<string> const & cmds = citeCmds_;
+		vector<string>::const_iterator cit =
+			std::find(cmds.begin(), cmds.end(), cs.cmd);
+		int i = 0;
+		if (cit != cmds.end())
+			i = int(cit - cmds.begin());
+
+		// Set the style combo appropriately
+		citationStyleCO->blockSignals(true);
+		citationStyleCO->setCurrentIndex(i);
+		citationStyleCO->blockSignals(false);
+		updateFormatting(citeStyles_[i]);
 	} else
 		availableLV->setFocus();
-	fillFields(bi);
-	fillEntries(bi);
-	updateControls(bi);
+
+	applyPB->setEnabled(false);
+	okPB->setEnabled(false);
 }
 
 
@@ -594,40 +571,47 @@ void GuiCitation::findKey(BiblioInfo const & bi,
 	last_searched_string = str;
 
 	QStringList result;
-	
-	// First, filter by entry_type, which will be faster than 
+
+	// First, filter by entry_type, which will be faster than
 	// what follows, so we may get to do that on less.
 	vector<docstring> keyVector = to_docstring_vector(keys);
 	filterByEntryType(bi, keyVector, entry_type);
-	
+
 	if (str.isEmpty())
 		result = to_qstring_list(keyVector);
 	else
-		result = to_qstring_list(searchKeys(bi, keyVector, only_keys, 
+		result = to_qstring_list(searchKeys(bi, keyVector, only_keys,
 			qstring_to_ucs4(str), field, case_sensitive, reg_exp));
-	
+
 	available_model_.setStringList(result);
 }
 
 
-QStringList GuiCitation::citationStyles(BiblioInfo const & bi, int sel)
+QStringList GuiCitation::citationStyles(BiblioInfo const & bi)
 {
-	docstring const key = qstring_to_ucs4(cited_keys_[sel]);
-	return to_qstring_list(bi.getCiteStrings(key, buffer()));
+	docstring const before = qstring_to_ucs4(textBeforeED->text());
+	docstring const after = qstring_to_ucs4(textAfterED->text());
+	vector<docstring> const keys = to_docstring_vector(cited_keys_);
+	vector<CitationStyle> styles = citeStyles_;
+	// FIXME: pass a dictionary instead of individual before, after, dialog, etc.
+	vector<docstring> ret = bi.getCiteStrings(keys, styles, documentBuffer(),
+		false, before, after, from_utf8("dialog"));
+	return to_qstring_list(ret);
 }
 
 
-void GuiCitation::setCitedKeys() 
+void GuiCitation::setCitedKeys()
 {
 	cited_keys_ = selected_model_.stringList();
+	updateStyles();
 }
 
 
 bool GuiCitation::initialiseParams(string const & data)
 {
 	InsetCommand::string2params(data, params_);
-	CiteEngine const engine = citeEngine();
-	citeStyles_ = citeStyles(engine);
+	citeCmds_ = documentBuffer().params().citeCommands();
+	citeStyles_ = documentBuffer().params().citeStyles();
 	init();
 	return true;
 }
@@ -640,11 +624,11 @@ void GuiCitation::clearParams()
 
 
 void GuiCitation::filterByEntryType(BiblioInfo const & bi,
-	vector<docstring> & keyVector, docstring entry_type) 
+	vector<docstring> & keyVector, docstring entry_type)
 {
 	if (entry_type.empty())
 		return;
-	
+
 	vector<docstring>::iterator it = keyVector.begin();
 	vector<docstring>::iterator end = keyVector.end();
 
@@ -661,12 +645,6 @@ void GuiCitation::filterByEntryType(BiblioInfo const & bi,
 }
 
 
-CiteEngine GuiCitation::citeEngine() const
-{
-	return buffer().params().citeEngine();
-}
-
-
 // Escape special chars.
 // All characters are literals except: '.|*?+(){}[]^$\'
 // These characters are literals when preceded by a "\", which is done here
@@ -690,7 +668,7 @@ static docstring escape_special_chars(docstring const & expr)
 }
 
 
-vector<docstring> GuiCitation::searchKeys(BiblioInfo const & bi,	
+vector<docstring> GuiCitation::searchKeys(BiblioInfo const & bi,
 	vector<docstring> const & keys_to_search, bool only_keys,
  	docstring const & search_expression, docstring field,
 	bool case_sensitive, bool regex)
@@ -710,7 +688,7 @@ vector<docstring> GuiCitation::searchKeys(BiblioInfo const & bi,
 	try {
 		reg_exp.assign(to_utf8(expr), case_sensitive ?
 			lyx::regex_constants::ECMAScript : lyx::regex_constants::icase);
-	} catch (lyx::regex_error & e) {
+	} catch (lyx::regex_error const & e) {
 		// lyx::regex throws an exception if the regular expression is not
 		// valid.
 		LYXERR(Debug::GUI, e.what());
@@ -723,16 +701,16 @@ vector<docstring> GuiCitation::searchKeys(BiblioInfo const & bi,
 		BiblioInfo::const_iterator info = bi.find(*it);
 		if (info == bi.end())
 			continue;
-		
+
 		BibTeXInfo const & kvm = info->second;
 		string data;
 		if (only_keys)
 			data = to_utf8(*it);
 		else if (field.empty())
 			data = to_utf8(*it) + ' ' + to_utf8(kvm.allData());
-		else 
+		else
 			data = to_utf8(kvm[field]);
-		
+
 		if (data.empty())
 			continue;
 
@@ -740,7 +718,7 @@ vector<docstring> GuiCitation::searchKeys(BiblioInfo const & bi,
 			if (lyx::regex_search(data, reg_exp))
 				foundKeys.push_back(*it);
 		}
-		catch (lyx::regex_error & e) {
+		catch (lyx::regex_error const & e) {
 			LYXERR(Debug::GUI, e.what());
 			return vector<docstring>();
 		}
@@ -758,21 +736,12 @@ void GuiCitation::dispatchParams()
 
 BiblioInfo const & GuiCitation::bibInfo() const
 {
-	Buffer const & buf = buffer();
+	Buffer const & buf = documentBuffer();
 	buf.reloadBibInfoCache();
 	return buf.masterBibInfo();
 }
 
 
-Buffer const & GuiCitation::buffer() const
-{
-	if (lyxview().documentBufferView())
-		return lyxview().documentBufferView()->buffer();
-	else
-		return lyxview().currentBufferView()->buffer();
-}
-
-
 void GuiCitation::saveSession() const
 {
 	Dialog::saveSession();
