@@ -15,10 +15,6 @@
 
 #include "support/ProgressInterface.h"
 
-#include "DockView.h"
-
-
-#include <QSplashScreen>
 #include <QTimer>
 
 #include <sstream>
