@@ -40,16 +40,19 @@
 #include "IndicesList.h"
 #include "KeyMap.h"
 #include "Language.h"
+#include "Layout.h"
 #include "Lexer.h"
 #include "LyXAction.h"
 #include "LyX.h"
 #include "LyXRC.h"
 #include "lyxfind.h"
 #include "Paragraph.h"
+#include "ParagraphParameters.h"
 #include "ParIterator.h"
 #include "Session.h"
 #include "SpellChecker.h"
 #include "TextClass.h"
+#include "Text.h"
 #include "TocBackend.h"
 #include "Toolbars.h"
 #include "WordLangTuple.h"
@@ -108,9 +111,9 @@ public:
 		    menuitem is in a submenu, the submenu is enabled to make sure the
 		    user sees the information. */
 		Help,
-		/** This type of item merely shows that there might be a list or 
+		/** This type of item merely shows that there might be a list or
 		    something alike at this position, but the list is still empty.
-		    If this item is in a submenu, the submenu will not always be 
+		    If this item is in a submenu, the submenu will not always be
 		    enabled. */
 		Info,
 		/** This is the list of last opened file,
@@ -172,7 +175,21 @@ public:
 		/// Words suggested by the spellchecker.
 		SpellingSuggestions,
 		/** Used Languages */
-		LanguageSelector
+		LanguageSelector,
+		/** This is the list of arguments available
+		    for insertion into the current layout. */
+		Arguments,
+		/** This is the list of arguments available
+		    in the InsetArgument context menu. */
+		SwitchArguments,
+		/** This is the list of captions available
+		in the current layout. */
+		Captions,
+		/** This is the list of captions available
+		in the InsetCaption context menu. */
+		SwitchCaptions,
+		/** Commands to separate environments. */
+		EnvironmentSeparators
 	};
 
 	explicit MenuItem(Kind kind) : kind_(kind), optional_(false) {}
@@ -185,7 +202,7 @@ public:
 		: kind_(kind), label_(label), submenuname_(submenu),
 		  tooltip_(tooltip), optional_(optional)
 	{
-		LASSERT(kind == Submenu || kind == Help || kind == Info, /**/);
+		LATTEST(kind == Submenu || kind == Help || kind == Info);
 	}
 
 	MenuItem(Kind kind,
@@ -204,8 +221,8 @@ public:
 	~MenuItem() {}
 
 	/// The label of a given menuitem
-	QString label() const 
-	{	
+	QString label() const
+	{
 		int const index = label_.lastIndexOf('|');
 		return index == -1 ? label_ : label_.left(index);
 	}
@@ -214,7 +231,7 @@ public:
 	QString shortcut() const
 	{
 		int const index = label_.lastIndexOf('|');
-		return index == -1 ? QString() : label_.mid(index + 1); 
+		return index == -1 ? QString() : label_.mid(index + 1);
 	}
 	/// The complete label, with label and shortcut separated by a '|'
 	QString fulllabel() const { return label_; }
@@ -241,7 +258,7 @@ public:
 		// Get the keys bound to this action, but keep only the
 		// first one later
 		KeyMap::Bindings bindings = theTopLevelKeymap().findBindings(func_);
-		if (bindings.size())
+		if (!bindings.empty())
 			return toqstr(bindings.begin()->print(KeySequence::ForGui));
 
 		LYXERR(Debug::KBMAP, "No binding for "
@@ -315,7 +332,7 @@ public:
 	void cat(MenuDefinition const & other);
 	///
 	void catSub(docstring const & name);
-	
+
 	// search for func in this menu iteratively, and put menu
 	// names in a stack.
 	bool searchMenu(FuncRequest const & func, docstring_list & names)
@@ -333,7 +350,7 @@ public:
 	void expandLastfiles();
 	void expandDocuments();
 	void expandBookmarks();
-	void expandFormats(MenuItem::Kind kind, Buffer const * buf);
+	void expandFormats(MenuItem::Kind const kind, Buffer const * buf);
 	void expandFloatListInsert(Buffer const * buf);
 	void expandFloatInsert(Buffer const * buf);
 	void expandFlexInsert(Buffer const * buf, InsetLayout::InsetLyXType type);
@@ -348,6 +365,9 @@ public:
 	void expandGraphicsGroups(BufferView const *);
 	void expandSpellingSuggestions(BufferView const *);
 	void expandLanguageSelector(Buffer const * buf);
+	void expandArguments(BufferView const *, bool switcharg = false);
+	void expandCaptions(Buffer const * buf, bool switchcap = false);
+	void expandEnvironmentSeparators(BufferView const *);
 	///
 	ItemList items_;
 	///
@@ -455,18 +475,26 @@ void MenuDefinition::read(Lexer & lex)
 		md_toolbars,
 		md_graphicsgroups,
 		md_spellingsuggestions,
-		md_languageselector
+		md_languageselector,
+		md_arguments,
+		md_switcharguments,
+		md_captions,
+		md_switchcaptions,
+		md_env_separators
 	};
 
 	LexerKeyword menutags[] = {
+		{ "arguments", md_arguments },
 		{ "bookmarks", md_bookmarks },
 		{ "branches", md_branches },
+		{ "captions", md_captions },
 		{ "charstyles", md_charstyles },
 		{ "citestyles", md_citestyles },
 		{ "custom", md_custom },
 		{ "documents", md_documents },
 		{ "elements", md_elements },
 		{ "end", md_endmenu },
+		{ "environmentseparators", md_env_separators },
 		{ "exportformats", md_exportformats },
 		{ "floatinsert", md_floatinsert },
 		{ "floatlistinsert", md_floatlistinsert },
@@ -485,6 +513,8 @@ void MenuDefinition::read(Lexer & lex)
 		{ "separator", md_separator },
 		{ "spellingsuggestions", md_spellingsuggestions },
 		{ "submenu", md_submenu },
+		{ "switcharguments", md_switcharguments },
+		{ "switchcaptions", md_switchcaptions },
 		{ "toc", md_toc },
 		{ "toolbars", md_toolbars },
 		{ "updateformats", md_updateformats },
@@ -494,14 +524,10 @@ void MenuDefinition::read(Lexer & lex)
 	lex.pushTable(menutags);
 	lex.setContext("MenuDefinition::read: ");
 
-	bool quit = false;
-	bool optional = false;
-
-	while (lex.isOK() && !quit) {
-		switch (lex.lex()) {
+	int md_type = 0;
+	while (lex.isOK() && md_type != md_endmenu) {
+		switch (md_type = lex.lex()) {
 		case md_optitem:
-			optional = true;
-			// fallback to md_item
 		case md_item: {
 			lex.next(true);
 			docstring const name = translateIfPossible(lex.getDocString());
@@ -511,8 +537,8 @@ void MenuDefinition::read(Lexer & lex)
 			FuncRequest::Origin origin = FuncRequest::MENU;
 			if (name_.startsWith("context-toc-"))
 				origin = FuncRequest::TOC;
+			bool const optional = (md_type == md_optitem);
 			add(MenuItem(MenuItem::Command, toqstr(name), func, QString(), optional, origin));
-			optional = false;
 			break;
 		}
 
@@ -616,22 +642,39 @@ void MenuDefinition::read(Lexer & lex)
 			add(MenuItem(MenuItem::IndicesListsContext));
 			break;
 
+		case md_arguments:
+			add(MenuItem(MenuItem::Arguments));
+			break;
+
+		case md_switcharguments:
+			add(MenuItem(MenuItem::SwitchArguments));
+			break;
+
+		case md_captions:
+			add(MenuItem(MenuItem::Captions));
+			break;
+
+		case md_switchcaptions:
+			add(MenuItem(MenuItem::SwitchCaptions));
+			break;
+
+		case md_env_separators:
+			add(MenuItem(MenuItem::EnvironmentSeparators));
+			break;
+
 		case md_optsubmenu:
-			optional = true;
-			// fallback to md_submenu
 		case md_submenu: {
 			lex.next(true);
 			docstring const mlabel = translateIfPossible(lex.getDocString());
 			lex.next(true);
 			docstring const mname = lex.getDocString();
+			bool const optional = (md_type == md_optsubmenu);
 			add(MenuItem(MenuItem::Submenu,
 				toqstr(mlabel), toqstr(mname), QString(), optional));
-			optional = false;
 			break;
 		}
 
 		case md_endmenu:
-			quit = true;
 			break;
 
 		default:
@@ -745,7 +788,7 @@ void MenuDefinition::expandGraphicsGroups(BufferView const * bv)
 
 	set<string>::const_iterator it = grp.begin();
 	set<string>::const_iterator end = grp.end();
-	add(MenuItem(MenuItem::Command, qt_("No Group"), 
+	add(MenuItem(MenuItem::Command, qt_("No Group"),
 		     FuncRequest(LFUN_SET_GRAPHICS_GROUP)));
 	for (; it != end; ++it) {
 		addWithStatusCheck(MenuItem(MenuItem::Command, toqstr(*it) + '|',
@@ -781,7 +824,7 @@ void MenuDefinition::expandSpellingSuggestions(BufferView const * bv)
 					docstring const & suggestion = suggestions[i];
 					LYXERR(Debug::GUI, suggestion);
 					MenuItem w(MenuItem::Command, toqstr(suggestion),
-						FuncRequest(LFUN_WORD_REPLACE, 
+						FuncRequest(LFUN_WORD_REPLACE,
 							replace2string(suggestion, selection,
 								true,     // case sensitive
 								true,     // match word
@@ -836,7 +879,7 @@ void MenuDefinition::expandLanguageSelector(Buffer const * buf)
 
 	std::set<Language const *> languages_buffer =
 		buf->masterBuffer()->getLanguages();
-	
+
 	if (languages_buffer.size() < 2)
 		return;
 
@@ -953,9 +996,8 @@ void MenuDefinition::expandDocuments()
 	Buffer * b = first;
 	// We cannot use a for loop as the buffer list cycles.
 	do {
-		bool const shown = guiApp->currentView()
-			? bool(guiApp->currentView()->workArea(*b)) : false;
-		if (!shown) {
+		if (!(guiApp->currentView()
+		    && guiApp->currentView()->workArea(*b))) {
 			QString label = toqstr(b->fileName().displayName(20));
 			if (!b->isClean())
 				label += "*";
@@ -966,8 +1008,8 @@ void MenuDefinition::expandDocuments()
 			++i;
 		}
 		b = theBufferList().next(b);
-	} while (b != first); 
-	
+	} while (b != first);
+
 	if (!item.submenu().empty())
 		add(item);
 }
@@ -993,14 +1035,14 @@ void MenuDefinition::expandBookmarks()
 }
 
 
-void MenuDefinition::expandFormats(MenuItem::Kind kind, Buffer const * buf)
+void MenuDefinition::expandFormats(MenuItem::Kind const kind, Buffer const * buf)
 {
 	if (!buf && kind != MenuItem::ImportFormats)
 		return;
 
 	typedef vector<Format const *> Formats;
 	Formats formats;
-	FuncCode action;
+	FuncCode action = LFUN_NOACTION;
 
 	switch (kind) {
 	case MenuItem::ImportFormats:
@@ -1015,18 +1057,23 @@ void MenuDefinition::expandFormats(MenuItem::Kind kind, Buffer const * buf)
 		formats = buf->params().exportableFormats(true);
 		action = LFUN_BUFFER_UPDATE;
 		break;
-	default:
+	case MenuItem::ExportFormats:
 		formats = buf->params().exportableFormats(false);
 		action = LFUN_BUFFER_EXPORT;
+		break;
+	default:
+		LATTEST(false);
+		return;
 	}
+	sort(formats.begin(), formats.end(), Format::formatSorter);
 
 	bool const view_update = (kind == MenuItem::ViewFormats
 			|| kind == MenuItem::UpdateFormats);
 
 	QString smenue;
 	if (view_update)
-		smenue = (kind == MenuItem::ViewFormats ?
-			qt_("View (Other Formats)|F")
+		smenue = (kind == MenuItem::ViewFormats
+			? qt_("View (Other Formats)|F")
 			: qt_("Update (Other Formats)|p"));
 	MenuItem item(MenuItem::Submenu, smenue);
 	item.setSubmenu(MenuDefinition(smenue));
@@ -1038,59 +1085,56 @@ void MenuDefinition::expandFormats(MenuItem::Kind kind, Buffer const * buf)
 			continue;
 
 		docstring lab = from_utf8((*fit)->prettyname());
-		docstring scut = from_utf8((*fit)->shortcut());
+		docstring const scut = from_utf8((*fit)->shortcut());
 		docstring const tmplab = lab;
- 
+
 		if (!scut.empty())
 			lab += char_type('|') + scut;
-		docstring lab_i18n = translateIfPossible(lab);
+		docstring const lab_i18n = translateIfPossible(lab);
+		docstring const shortcut = split(lab_i18n, lab, '|');
+
 		bool const untranslated = (lab == lab_i18n);
-		QString const shortcut = toqstr(split(lab_i18n, lab, '|'));
-		QString label = toqstr(lab);
-		if (untranslated)
-			// this might happen if the shortcut
-			// has been redefined
-			label = toqstr(translateIfPossible(tmplab));
+		docstring label = untranslated ? translateIfPossible(tmplab) : lab;
 
 		switch (kind) {
 		case MenuItem::ImportFormats:
-			label += "...";
+			label += from_ascii("...");
 			break;
 		case MenuItem::ViewFormats:
 		case MenuItem::UpdateFormats:
 			if ((*fit)->name() == buf->params().getDefaultOutputFormat()) {
-				docstring lbl = (kind == MenuItem::ViewFormats ?
-					bformat(_("View [%1$s]|V"), qstring_to_ucs4(label))
-					: bformat(_("Update [%1$s]|U"), qstring_to_ucs4(label)));
-				MenuItem w(MenuItem::Command, toqstr(lbl),
-					   FuncRequest(action));
-				add(w);
+				docstring lbl = (kind == MenuItem::ViewFormats
+					? bformat(_("View [%1$s]|V"), label)
+					: bformat(_("Update [%1$s]|U"), label));
+				add(MenuItem(MenuItem::Command, toqstr(lbl), FuncRequest(action)));
 				continue;
 			}
+		// fall through
 		case MenuItem::ExportFormats:
 			if (!(*fit)->inExportMenu())
 				continue;
 			break;
 		default:
-			LASSERT(false, /**/);
-			break;
+			// we already asserted earlier in this case
+			// LATTEST(false);
+			continue;
 		}
-		if (!shortcut.isEmpty())
+		if (!shortcut.empty())
 			label += '|' + shortcut;
 
 		if (view_update) {
 			if (buf)
-				item.submenu().addWithStatusCheck(MenuItem(MenuItem::Command, label,
-					FuncRequest(action, (*fit)->name())));
+				item.submenu().addWithStatusCheck(MenuItem(MenuItem::Command,
+					toqstr(label), FuncRequest(action, (*fit)->name())));
 			else
-				item.submenu().add(MenuItem(MenuItem::Command, label,
+				item.submenu().add(MenuItem(MenuItem::Command, toqstr(label),
 					FuncRequest(action, (*fit)->name())));
 		} else {
 			if (buf)
-				addWithStatusCheck(MenuItem(MenuItem::Command, label,
+				addWithStatusCheck(MenuItem(MenuItem::Command, toqstr(label),
 					FuncRequest(action, (*fit)->name())));
 			else
-				add(MenuItem(MenuItem::Command, label,
+				add(MenuItem(MenuItem::Command, toqstr(label),
 					FuncRequest(action, (*fit)->name())));
 		}
 	}
@@ -1165,7 +1209,7 @@ void MenuDefinition::expandFlexInsert(
 			// we remove the "Flex:" prefix, if it is present
 			if (prefixIs(label, from_utf8("Flex:")))
 				label = label.substr(5);
-			addWithStatusCheck(MenuItem(MenuItem::Command, 
+			addWithStatusCheck(MenuItem(MenuItem::Command,
 				toqstr(translateIfPossible(label)),
 				FuncRequest(LFUN_FLEX_INSERT, Lexer::quoteString(label))));
 		}
@@ -1258,7 +1302,7 @@ void MenuDefinition::expandToc(Buffer const * buf)
 	}
 
 	MenuDefinition other_lists;
-	
+
 	FloatList const & floatlist = buf->params().documentClass().floats();
 	TocList const & toc_list = buf->tocBackend().tocs();
 	TocList::const_iterator cit = toc_list.begin();
@@ -1302,7 +1346,7 @@ void MenuDefinition::expandToc(Buffer const * buf)
 	if (cit == end)
 		LYXERR(Debug::GUI, "No table of contents.");
 	else {
-		if (cit->second.size() > 0 ) 
+		if (!cit->second.empty())
 			expandToc2(cit->second, 0, cit->second.size(), 0);
 		else
 			add(MenuItem(MenuItem::Info, qt_("<Empty Table of Contents>")));
@@ -1355,7 +1399,6 @@ void MenuDefinition::expandBranches(Buffer const * buf)
 	BufferParams const & master_params = buf->masterBuffer()->params();
 	BufferParams const & params = buf->params();
 	if (params.branchlist().empty() && master_params.branchlist().empty() ) {
-		LYXERR(Debug::GUI, "No Branches set for Document");
 		add(MenuItem(MenuItem::Help, qt_("No Branches Set for Document!")));
 		return;
 	}
@@ -1369,17 +1412,16 @@ void MenuDefinition::expandBranches(Buffer const * buf)
 			label = convert<docstring>(ii) + ". " + label
 				+ char_type('|') + convert<docstring>(ii);
 		}
-		LYXERR(Debug::GUI, "Add item for branch " << label);
 		addWithStatusCheck(MenuItem(MenuItem::Command, toqstr(label),
 				    FuncRequest(LFUN_BRANCH_INSERT,
 						cit->branch())));
 	}
-	
+
 	if (buf == buf->masterBuffer())
 		return;
-	
+
 	MenuDefinition child_branches;
-	
+
 	BranchList::const_iterator ccit = params.branchlist().begin();
 	BranchList::const_iterator cend = params.branchlist().end();
 
@@ -1395,7 +1437,7 @@ void MenuDefinition::expandBranches(Buffer const * buf)
 				    FuncRequest(LFUN_BRANCH_INSERT,
 						ccit->branch())));
 	}
-	
+
 	if (!child_branches.empty()) {
 		MenuItem item(MenuItem::Submenu, qt_("Child Document"));
 		item.setSubmenu(child_branches);
@@ -1432,12 +1474,12 @@ void MenuDefinition::expandIndices(Buffer const * buf, bool listof)
 
 	for (int ii = 1; cit != end; ++cit, ++ii) {
 		if (listof) {
-			docstring const label = 
+			docstring const label =
 				bformat(_("Index: %1$s"), cit->index());
 			addWithStatusCheck(MenuItem(MenuItem::Command, toqstr(label),
 					   FuncRequest(LFUN_INDEX_PRINT, cit->shortcut())));
 		} else {
-			docstring const label = 
+			docstring const label =
 				bformat(_("Index Entry (%1$s)"), cit->index());
 			addWithStatusCheck(MenuItem(MenuItem::Command, toqstr(label),
 					   FuncRequest(LFUN_INDEX_INSERT, cit->shortcut())));
@@ -1466,7 +1508,7 @@ void MenuDefinition::expandIndicesContext(Buffer const * buf, bool listof)
 			addWithStatusCheck(MenuItem(MenuItem::Command, toqstr(cit->index()),
 					   FuncRequest(LFUN_INSET_MODIFY, data)));
 		} else {
-			docstring const label = 
+			docstring const label =
 					bformat(_("Index Entry (%1$s)"), cit->index());
 			addWithStatusCheck(MenuItem(MenuItem::Command, toqstr(label),
 					   FuncRequest(LFUN_INSET_MODIFY,
@@ -1490,31 +1532,175 @@ void MenuDefinition::expandCiteStyles(BufferView const * bv)
 	}
 	InsetCommand const * citinset =
 				static_cast<InsetCommand const *>(inset);
-	
+
 	Buffer const * buf = &bv->buffer();
-	docstring key = citinset->getParam("key");
-	// we can only handle one key currently
-	if (contains(key, ','))
-		key = qstring_to_ucs4(toqstr(key).split(',')[0]);
+	string const cmd = citinset->params().getCmdName();
 
-	vector<CiteStyle> citeStyleList = citeStyles(buf->params().citeEngine());
-	docstring_list citeStrings =
-		buf->masterBibInfo().getCiteStrings(key, bv->buffer());
+	docstring const & key = citinset->getParam("key");
+	if (key.empty()) {
+		add(MenuItem(MenuItem::Command,
+				    qt_("No citations selected!"),
+				    FuncRequest(LFUN_NOACTION)));
+		return;
+	}
 
-	docstring_list::const_iterator cit = citeStrings.begin();
-	docstring_list::const_iterator end = citeStrings.end();
+	docstring const & before = citinset->getParam("before");
+	docstring const & after = citinset->getParam("after");
+
+	size_t const n = cmd.size();
+	bool const force = cmd[0] == 'C';
+	bool const full = cmd[n] == '*';
+
+	vector<docstring> const keys = getVectorFromString(key);
+
+	vector<CitationStyle> const citeStyleList = buf->params().citeStyles();
+	vector<docstring> citeStrings =
+		buf->masterBibInfo().getCiteStrings(keys, citeStyleList, bv->buffer(),
+		false, before, after, from_utf8("dialog"));
+
+	vector<docstring>::const_iterator cit = citeStrings.begin();
+	vector<docstring>::const_iterator end = citeStrings.end();
 
 	for (int ii = 1; cit != end; ++cit, ++ii) {
 		docstring label = *cit;
-		CitationStyle cs;
-		CiteStyle cst = citeStyleList[ii - 1];
-		cs.style = cst;
+		CitationStyle cs = citeStyleList[ii - 1];
+		cs.forceUpperCase &= force;
+		cs.fullAuthorList &= full;
 		addWithStatusCheck(MenuItem(MenuItem::Command, toqstr(label),
 				    FuncRequest(LFUN_INSET_MODIFY,
 						"changetype " + from_utf8(citationStyleToString(cs)))));
 	}
 }
 
+
+void MenuDefinition::expandArguments(BufferView const * bv, bool switcharg)
+{
+	if (!bv)
+		return;
+
+	if (!bv->cursor().inTexted())
+		return;
+
+	Inset const * inset = &bv->cursor().inset();
+	Layout::LaTeXArgMap args = bv->cursor().paragraph().layout().args();
+	if (inset && args.empty())
+		args = inset->getLayout().args();
+	if (args.empty() || (switcharg && args.size() == 1))
+		return;
+	Layout::LaTeXArgMap::const_iterator lait = args.begin();
+	Layout::LaTeXArgMap::const_iterator const laend = args.end();
+	for (; lait != laend; ++lait) {
+		Layout::latexarg arg = (*lait).second;
+		docstring str = arg.menustring.empty()? arg.labelstring : arg.menustring;
+		QString item = toqstr(translateIfPossible(str));
+		if (switcharg)
+			add(MenuItem(MenuItem::Command, item,
+				     FuncRequest(LFUN_INSET_MODIFY,
+						 from_ascii("changetype ")
+						 + from_ascii((*lait).first))));
+		else
+			add(MenuItem(MenuItem::Command, item,
+				     FuncRequest(LFUN_ARGUMENT_INSERT,
+						 from_ascii((*lait).first))));
+	}
+}
+
+
+void MenuDefinition::expandCaptions(Buffer const * buf, bool switchcap)
+{
+	if (!buf)
+		return;
+
+	vector<docstring> caps;
+	DocumentClass const & dc = buf->params().documentClass();
+	TextClass::InsetLayouts::const_iterator lit = dc.insetLayouts().begin();
+	TextClass::InsetLayouts::const_iterator len = dc.insetLayouts().end();
+	for (; lit != len; ++lit) {
+		if (prefixIs(lit->first, from_ascii("Caption:")))
+			caps.push_back(lit->first);
+	}
+
+	if (caps.empty() || (switchcap && caps.size() == 1))
+		return;
+	if (caps.size() == 1) {
+		docstring dummy;
+		docstring const type = split(*caps.begin(), dummy, ':');
+		add(MenuItem(MenuItem::Command, qt_("Caption"),
+			 FuncRequest(LFUN_CAPTION_INSERT, translateIfPossible(type))));
+		return;
+	}
+
+	MenuDefinition captions;
+
+	vector<docstring>::const_iterator cit = caps.begin();
+	vector<docstring>::const_iterator end = caps.end();
+
+	for (int ii = 1; cit != end; ++cit, ++ii) {
+		docstring dummy;
+		docstring const type = split(*cit, dummy, ':');
+		docstring const trtype = translateIfPossible(type);
+		docstring const cmitem = bformat(_("Caption (%1$s)"), trtype);
+		// make menu item optional, otherwise we would also see
+		// forbidden caption types
+		if (switchcap)
+			addWithStatusCheck(MenuItem(MenuItem::Command, toqstr(cmitem),
+				     FuncRequest(LFUN_INSET_MODIFY,
+						 from_ascii("changetype ")
+						 + type), QString(), true));
+		else
+			captions.addWithStatusCheck(MenuItem(MenuItem::Command,
+							     toqstr(trtype),
+							     FuncRequest(LFUN_CAPTION_INSERT,
+							     type), QString(), true));
+	}
+	if (!captions.empty()) {
+		MenuItem item(MenuItem::Submenu, qt_("Caption"));
+		item.setSubmenu(captions);
+		add(item);
+	}
+}
+
+
+void MenuDefinition::expandEnvironmentSeparators(BufferView const * bv)
+{
+	if (!bv)
+		return;
+
+	pit_type pit = bv->cursor().selBegin().pit();
+	Paragraph const & par = bv->buffer().text().getPar(pit);
+	docstring const curlayout = par.layout().name();
+	docstring outerlayout;
+	depth_type current_depth = par.params().depth();
+	// check if we have an environment in our nesting hierarchy
+	Paragraph cpar = par;
+	while (true) {
+		if (pit == 0 || cpar.params().depth() == 0)
+			break;
+		--pit;
+		cpar = bv->buffer().text().getPar(pit);
+		if (cpar.params().depth() < current_depth
+		    && cpar.layout().isEnvironment()) {
+				outerlayout = cpar.layout().name();
+				current_depth = cpar.params().depth();
+		}
+	}
+	if (par.layout().isEnvironment()) {
+		docstring const label =
+			bformat(_("Start New Environment (%1$s)"),
+				translateIfPossible(curlayout));
+		add(MenuItem(MenuItem::Command, toqstr(label),
+			     FuncRequest(LFUN_ENVIRONMENT_SPLIT)));
+	}
+	if (!outerlayout.empty()) {
+		docstring const label =
+			bformat(_("Start New Parent Environment (%1$s)"),
+				translateIfPossible(outerlayout));
+		add(MenuItem(MenuItem::Command, toqstr(label),
+			     FuncRequest(LFUN_ENVIRONMENT_SPLIT,
+					 from_ascii("outer"))));
+	}
+}
+
 } // namespace anon
 
 
@@ -1562,7 +1748,7 @@ static QString label(MenuItem const & mi)
 void Menu::Impl::populate(QMenu & qMenu, MenuDefinition const & menu)
 {
 	LYXERR(Debug::GUI, "populating menu " << menu.name());
-	if (menu.size() == 0) {
+	if (menu.empty()) {
 		LYXERR(Debug::GUI, "\tERROR: empty menu " << menu.name());
 		return;
 	}
@@ -1578,7 +1764,7 @@ void Menu::Impl::populate(QMenu & qMenu, MenuDefinition const & menu)
 			subMenu->setEnabled(m->status().enabled());
 		} else {
 			// we have a MenuItem::Command
-			qMenu.addAction(new Action(view, QIcon(), label(*m), 
+			qMenu.addAction(new Action(QIcon(), label(*m),
 				m->func(), m->tooltip(), &qMenu));
 		}
 	}
@@ -1588,7 +1774,7 @@ void Menu::Impl::populate(QMenu & qMenu, MenuDefinition const & menu)
 class AlwaysMnemonicStyle : public QProxyStyle {
 public:
 	int styleHint(StyleHint hint, const QStyleOption *opt = 0, const QWidget *widget = 0,
-		QStyleHintReturn *returnData = 0) const 
+		QStyleHintReturn *returnData = 0) const
 	{
 		if (hint == QStyle::SH_UnderlineShortcut)
 			return 1;
@@ -1615,7 +1801,7 @@ Menu::Menu(GuiView * gv, QString const & name, bool top_level, bool keyboard)
 	d->name = name;
 	setTitle(name);
 	if (d->top_level_menu)
-		connect(this, SIGNAL(aboutToShow()), this, SLOT(updateView()));	
+		connect(this, SIGNAL(aboutToShow()), this, SLOT(updateView()));
 }
 
 
@@ -1632,6 +1818,17 @@ void Menu::updateView()
 }
 
 
+void Menu::clear()
+{
+	QList<QAction *> items = actions();
+	for (int i = 0; i != items.size(); ++i) {
+		// QAction::menu() returns 0 if there's no submenu.
+		delete items.at(i)->menu();
+	}
+	QMenu::clear();
+}
+
+
 /////////////////////////////////////////////////////////////////////
 // Menus::Impl definition and implementation
 /////////////////////////////////////////////////////////////////////
@@ -1647,13 +1844,15 @@ struct Menus::Impl {
 	/// Expands some special entries of the menu
 	/** The entries with the following kind are expanded to a
 	    sequence of Command MenuItems: Lastfiles, Documents,
-	    ViewFormats, ExportFormats, UpdateFormats, Branches, Indices
+	    ViewFormats, ExportFormats, UpdateFormats, Branches,
+	    Indices, Arguments, SwitchArguments, Captions, SwitchCaptions,
+	    EnvironmentSeparators
 	*/
 	void expand(MenuDefinition const & frommenu, MenuDefinition & tomenu,
 		BufferView const *) const;
 
 	/// Initialize specific MACOS X menubar
-	void macxMenuBarInit(GuiView * view, QMenuBar * qmb);
+	void macxMenuBarInit(QMenuBar * qmb);
 
 	/// Mac special menu.
 	/** This defines a menu whose entries list the FuncRequests
@@ -1703,7 +1902,7 @@ MenuDefinition Menus::Impl::mac_special_menu_;
   that this menubar will be used also when one of LyX' dialogs has
   focus. (JMarc)
 */
-void Menus::Impl::macxMenuBarInit(GuiView * view, QMenuBar * qmb)
+void Menus::Impl::macxMenuBarInit(QMenuBar * qmb)
 {
 	/* Since Qt 4.2, the qt/mac menu code has special code for
 	   specifying the role of a menu entry. However, it does not
@@ -1731,15 +1930,18 @@ void Menus::Impl::macxMenuBarInit(GuiView * view, QMenuBar * qmb)
 		 QAction::AboutRole},
 		{LFUN_DIALOG_SHOW, "prefs", "Preferences",
 		 QAction::PreferencesRole},
+#if !(defined(QT_MAC_USE_COCOA) || (QT_VERSION >= 0x050000))
+		/* This doesn't work with Cocoa. */
 		{LFUN_RECONFIGURE, "", "Reconfigure",
 		 QAction::ApplicationSpecificRole},
+#endif
 		{LFUN_LYX_QUIT, "", "Quit LyX", QAction::QuitRole}
 	};
 	const size_t num_entries = sizeof(entries) / sizeof(entries[0]);
+	const bool first_call = mac_special_menu_.empty();
 
 	// the special menu for Menus. Fill it up only once.
-	if (mac_special_menu_.size() == 0) {
-		LYXERR(Debug::GUI, "Prepare Mac OS X special menu");
+	if (first_call) {
 		for (size_t i = 0 ; i < num_entries ; ++i) {
 			FuncRequest const func(entries[i].action,
 				from_utf8(entries[i].arg));
@@ -1747,17 +1949,17 @@ void Menus::Impl::macxMenuBarInit(GuiView * view, QMenuBar * qmb)
 				entries[i].label, func));
 		}
 	}
+
 	// add the entries to a QMenu that will eventually be empty
 	// and therefore invisible.
 	QMenu * qMenu = qmb->addMenu("special");
 	MenuDefinition::const_iterator cit = mac_special_menu_.begin();
 	MenuDefinition::const_iterator end = mac_special_menu_.end();
 	for (size_t i = 0 ; cit != end ; ++cit, ++i) {
-		Action * action = new Action(view, QIcon(), cit->label(),
+		Action * action = new Action(QIcon(), cit->label(),
 			cit->func(), QString(), qMenu);
 		action->setMenuRole(entries[i].role);
 		qMenu->addAction(action);
-		LYXERR(Debug::GUI, "Add special menu item " << cit->label());
 	}
 }
 
@@ -1859,6 +2061,26 @@ void Menus::Impl::expand(MenuDefinition const & frommenu,
 			tomenu.expandLanguageSelector(buf);
 			break;
 
+		case MenuItem::Arguments:
+			tomenu.expandArguments(bv, false);
+			break;
+
+		case MenuItem::SwitchArguments:
+			tomenu.expandArguments(bv, true);
+			break;
+
+		case MenuItem::Captions:
+			tomenu.expandCaptions(buf, false);
+			break;
+
+		case MenuItem::SwitchCaptions:
+			tomenu.expandCaptions(buf, true);
+			break;
+
+		case MenuItem::EnvironmentSeparators:
+			tomenu.expandEnvironmentSeparators(bv);
+			break;
+
 		case MenuItem::Submenu: {
 			MenuItem item(*cit);
 			item.setSubmenu(MenuDefinition(cit->submenuname()));
@@ -1899,9 +2121,10 @@ MenuDefinition const & Menus::Impl::getMenu(QString const & name) const
 {
 	const_iterator cit = find_if(menulist_.begin(), menulist_.end(),
 		MenuNamesEqual(name));
-	if (cit == menulist_.end())
+	if (cit == menulist_.end()) {
 		LYXERR0("No submenu named " << name);
-	LASSERT(cit != menulist_.end(), /**/);
+		LASSERT(false, { static const MenuDefinition m; return m; });
+	}
 	return (*cit);
 }
 
@@ -1910,16 +2133,17 @@ MenuDefinition & Menus::Impl::getMenu(QString const & name)
 {
 	iterator it = find_if(menulist_.begin(), menulist_.end(),
 		MenuNamesEqual(name));
-	if (it == menulist_.end())
+	if (it == menulist_.end()) {
 		LYXERR0("No submenu named " << name);
-	LASSERT(it != menulist_.end(), /**/);
+		LASSERT(false, { static MenuDefinition m; return m; });
+	}
 	return (*it);
 }
 
 
 /////////////////////////////////////////////////////////////////////
 //
-// Menus 
+// Menus
 //
 /////////////////////////////////////////////////////////////////////
 
@@ -2009,7 +2233,7 @@ void Menus::fillMenuBar(QMenuBar * qmb, GuiView * view, bool initial)
 		// create duplicate items in the application menu. It seems
 		// that Qt does not remove them when the QMenubar is cleared.
 		LYXERR(Debug::GUI, "Creating Mac OS X special menu bar");
-		d->macxMenuBarInit(view, qmb);
+		d->macxMenuBarInit(qmb);
 #endif
 	} else {
 		// Clear all menubar contents before filling it.
@@ -2018,7 +2242,7 @@ void Menus::fillMenuBar(QMenuBar * qmb, GuiView * view, bool initial)
 
 	LYXERR(Debug::GUI, "populating menu bar" << d->menubar_.name());
 
-	if (d->menubar_.size() == 0) {
+	if (d->menubar_.empty()) {
 		LYXERR(Debug::GUI, "\tERROR: empty menu bar"
 			<< d->menubar_.name());
 		return;
@@ -2053,6 +2277,14 @@ void Menus::fillMenuBar(QMenuBar * qmb, GuiView * view, bool initial)
 
 		Menu * menu = new Menu(view, m->submenuname(), true);
 		menu->setTitle(label(*m));
+
+#if defined(Q_WS_MACX) && (defined(QT_MAC_USE_COCOA) || (QT_VERSION >= 0x050000))
+		// On Mac OS with QT/cocoa, the menu is not displayed if there is no action
+		// so we create a temporary one here
+		QAction * action = new QAction(menu);
+		menu->addAction(action);
+#endif
+
 		qmb->addMenu(menu);
 
 		d->name_map_[view][name] = menu;
@@ -2083,7 +2315,7 @@ void Menus::updateMenu(Menu * qmenu)
 		MenuDefinition cat_menu = d->getMenu(toqstr(menu_name));
 		//FIXME: 50 is a wild guess. We should take into account here
 		//the expansion of menu items, disabled optional items etc.
-		bool const in_sub_menu = fromLyxMenu.size() > 0 
+		bool const in_sub_menu = !fromLyxMenu.empty()
 			&& fromLyxMenu.size() + cat_menu.size() > 50 ;
 		if (in_sub_menu)
 			fromLyxMenu.catSub(menu_name);
