@@ -155,9 +155,9 @@ bool TocWidget::getStatus(Cursor & cur, FuncRequest const & cmd,
 		status.setEnabled(item.dit() != 0);
 		return true;
 
-	case LFUN_LABEL_COPY_AS_REF: {
+	case LFUN_LABEL_COPY_AS_REFERENCE: {
 		// For labels in math, we need to supply the label as a string
-		FuncRequest label_copy(LFUN_LABEL_COPY_AS_REF, item.asString());
+		FuncRequest label_copy(LFUN_LABEL_COPY_AS_REFERENCE, item.asString());
 		if (inset)
 			return inset->getStatus(cur, label_copy, status);
 	}
@@ -187,26 +187,19 @@ void TocWidget::doDispatch(Cursor & cur, FuncRequest const & cmd)
 	{
 	case LFUN_CHANGE_ACCEPT:
 	case LFUN_CHANGE_REJECT:
-		dispatch(item.action());
-		cur.dispatch(tmpcmd);
-		break;
-
 	case LFUN_SECTION_SELECT:
 		dispatch(item.action());
 		cur.dispatch(tmpcmd);
-		// necessary to get the selection drawn.
-		cur.buffer()->changed(true);
-		gui_view_.setFocus();
 		break;
 
-	case LFUN_LABEL_COPY_AS_REF: {
+	case LFUN_LABEL_COPY_AS_REFERENCE: {
 		// For labels in math, we need to supply the label as a string
-		FuncRequest label_copy(LFUN_LABEL_COPY_AS_REF, item.asString());
+		FuncRequest label_copy(LFUN_LABEL_COPY_AS_REFERENCE, item.asString());
 		if (inset)
 			inset->dispatch(cur, label_copy);
 		break;
 	}
-	
+
 	case LFUN_OUTLINE_UP:
 	case LFUN_OUTLINE_DOWN:
 	case LFUN_OUTLINE_IN:
@@ -269,6 +262,7 @@ void TocWidget::on_persistentCB_stateChanged(int state)
 }
 
 
+#if 0
 /* FIXME (Ugras 17/11/06):
 I have implemented a indexDepth function to get the model indices. In my
 opinion, somebody should derive a new qvariant class for tocModelItem
@@ -281,7 +275,7 @@ static int indexDepth(QModelIndex const & index, int depth = -1)
 	return index.parent() == QModelIndex()
 		? depth : indexDepth(index.parent(), depth);
 }
-
+#endif
 
 void TocWidget::on_depthSL_valueChanged(int depth)
 {
@@ -298,27 +292,10 @@ void TocWidget::setTreeDepth(int depth)
 	if (!tocTV->model())
 		return;
 
-#if QT_VERSION >= 0x040300
-	// this should be faster than our own code below
 	if (depth == 0)
 		tocTV->collapseAll();
 	else
 		tocTV->expandToDepth(depth - 1);
-#else
-	// expanding and then collapsing is probably better,
-	// but my qt 4.1.2 doesn't have expandAll()..
-	//tocTV->expandAll();
-	QModelIndexList indices = tocTV->model()->match(
-		tocTV->model()->index(0, 0),
-		Qt::DisplayRole, "*", -1,
-		Qt::MatchFlags(Qt::MatchWildcard|Qt::MatchRecursive));
-
-	int size = indices.size();
-	for (int i = 0; i < size; i++) {
-		QModelIndex index = indices[i];
-		tocTV->setExpanded(index, indexDepth(index) < depth_);
-	}
-#endif
 }
 
 
@@ -503,9 +480,11 @@ void TocWidget::filterContents()
 static QString decodeType(QString const & str)
 {
 	QString type = str;
-	if (type.contains("tableofcontents")) {
+	if (type.contains("tableofcontents"))
 		type = "tableofcontents";
-	} else if (type.contains("floatlist")) {
+	else if (type.contains("lstlistoflistings"))
+		type = "listing";
+	else if (type.contains("floatlist")) {
 		if (type.contains("\"figure"))
 			type = "figure";
 		else if (type.contains("\"table"))
