@@ -27,7 +27,6 @@
 #include "TexRow.h"
 
 #include "frontends/Application.h" // hexName
-#include "frontends/qt4/ColorCache.h"
 
 #include "insets/Inset.h"
 
@@ -37,7 +36,6 @@
 #include "support/filetools.h"
 #include "support/ForkedCalls.h"
 #include "support/lstrings.h"
-#include "support/qstring_helpers.h"
 
 #include "support/bind.h"
 
@@ -390,11 +388,7 @@ namespace graphics {
 PreviewLoader::Impl::Impl(PreviewLoader & p, Buffer const & b)
 	: parent_(p), buffer_(b)
 {
-	if (b.params().bufferFormat() == "lilypond-book")
-		pconverter_ = setConverter("lyxpreview-lytex");
-	else if (b.params().encoding().package() == Encoding::japanese)
-		pconverter_ = setConverter("lyxpreview-platex");
-	else
+	if (!pconverter_)
 		pconverter_ = setConverter("lyxpreview");
 }
 
@@ -553,7 +547,7 @@ void PreviewLoader::Impl::startLoading(bool wait)
 	Encoding const & enc = buffer_.params().encoding();
 	ofdocstream of;
 	try { of.reset(enc.iconvName()); }
-	catch (iconv_codecvt_facet_exception & e) {
+	catch (iconv_codecvt_facet_exception const & e) {
 		LYXERR0("Caught iconv exception: " << e.what()
 			<< "\nUnable to create LaTeX file: " << latexfile);
 		return;
@@ -589,33 +583,35 @@ void PreviewLoader::Impl::startLoading(bool wait)
 	double const font_scaling_factor = 
 		buffer_.isExporting() ? 75.0 * buffer_.params().html_math_img_scale 
 			: 0.01 * lyxrc.dpi * lyxrc.zoom * lyxrc.preview_scale_factor;
+	
+	// The conversion command.
+	ostringstream cs;
+	cs << pconverter_->command
+	   << " " << quoteName(latexfile.toFilesystemEncoding())
+	   << " --dpi " << int(font_scaling_factor);
 
 	// FIXME XHTML 
 	// The colors should be customizable.
-	ColorCode const bg = buffer_.isExporting() 
-		? Color_white : PreviewLoader::backgroundColor();
-	ColorCode const fg = buffer_.isExporting() 
-		? Color_black : PreviewLoader::foregroundColor();
-	std::string bg_name, fg_name;
-	if (theApp()) {
-		fg_name = theApp()->hexName(fg);
-		bg_name = theApp()->hexName(bg);
-	} else {
-		ColorCache cc;
-		fg_name = ltrim(fromqstr(cc.get(fg).name()), "#");
-		bg_name = ltrim(fromqstr(cc.get(bg).name()), "#");
+	if (!buffer_.isExporting()) {
+		ColorCode const fg = PreviewLoader::foregroundColor();
+		ColorCode const bg = PreviewLoader::backgroundColor();
+		cs << " --fg " << theApp()->hexName(fg) 
+		   << " --bg " << theApp()->hexName(bg);
 	}
 
-	// The conversion command.
-	ostringstream cs;
-	cs << pconverter_->command << ' ' << pconverter_->to << ' '
-	   << quoteName(latexfile.toFilesystemEncoding()) << ' '
-	   << int(font_scaling_factor) << ' '
-	   << fg_name << ' '
-	   << bg_name;
 	// FIXME what about LuaTeX?
 	if (buffer_.params().useNonTeXFonts)
-		cs << " xelatex";
+		cs << " --latex=xelatex";
+	if (buffer_.params().encoding().package() == Encoding::japanese)
+		cs << " --latex=platex";
+	if (buffer_.params().bibtex_command != "default")
+		cs << " --bibtex=" << quoteName(buffer_.params().bibtex_command);
+	else if (buffer_.params().encoding().package() == Encoding::japanese)
+		cs << " --bibtex=" << quoteName(lyxrc.jbibtex_command);
+	else
+		cs << " --bibtex=" << quoteName(lyxrc.bibtex_command);
+	if (buffer_.params().bufferFormat() == "lilypond-book")
+		cs << " --lilypond";
 
 	string const command = libScriptSearch(cs.str());
 
@@ -748,26 +744,7 @@ void PreviewLoader::Impl::dumpPreamble(otexstream & os) const
 	// Also support PDF output (automatically generated e.g. when
 	// \usepackage[pdftex]{hyperref} is used and XeTeX.
 	os << "\n"
-	   << "\\newif\\ifxetex\n"
-	   << "\\expandafter\\ifx\\csname XeTeXrevision\\endcsname\\relax\n"
-	   << "   \\xetexfalse\n"
-	   << "\\else\n"
-	   << "    \\xetextrue\n"
-	   << "\\fi\n"
-	   << "\\newif\\ifpdf\n"
-	   << "\\ifx\\pdfoutput\\undefined\n"
-	   << "\\else\\ifx\\pdfoutput\\relax\n"
-	   << "\\else\\ifnum0=\\pdfoutput\n"
-	   << "\\else\\pdftrue\\fi\\fi\\fi\n"
-	   << "\\ifxetex\n"
-	   << "  \\usepackage[active,delayed,tightpage,showlabels,lyx,xetex]{preview}\n"
-	   << "\\else\n"
-	   << "\\ifpdf\n"
-	   << "  \\usepackage[active,delayed,tightpage,showlabels,lyx,pdftex]{preview}\n"
-	   << "\\else\n"
-	   << "  \\usepackage[active,delayed,showlabels,lyx,dvips]{preview}\n"
-	   << "\\fi\n"
-	   << "\\fi\n"
+	   << "\\usepackage[active,delayed,showlabels,lyx]{preview}\n"
 	   << "\n";
 }
 
