@@ -19,10 +19,10 @@
 #include "Cursor.h"
 #include "DispatchResult.h"
 #include "Encoding.h"
+#include "Exporter.h"
 #include "Format.h"
 #include "FuncRequest.h"
 #include "FuncStatus.h"
-#include "Language.h"
 #include "LaTeXFeatures.h"
 #include "output_xhtml.h"
 #include "OutputParams.h"
@@ -40,7 +40,7 @@
 #include "support/gettext.h"
 #include "support/lstrings.h"
 #include "support/os.h"
-#include "support/Path.h"
+#include "support/PathChanger.h"
 #include "support/textutils.h"
 
 #include <limits>
@@ -57,7 +57,7 @@ namespace os = support::os;
 InsetBibtex::InsetBibtex(Buffer * buf, InsetCommandParams const & p)
 	: InsetCommand(buf, p)
 {
-	buffer().invalidateBibinfoCache();
+	buffer().invalidateBibfileCache();
 	buffer().removeBiblioTempFiles();
 }
 
@@ -102,7 +102,7 @@ void InsetBibtex::doDispatch(Cursor & cur, FuncRequest & cmd)
 			if (message.type_ == WarningException) {
 				Alert::warning(message.title_, message.details_);
 				cur.noScreenUpdate();
-			} else 
+			} else
 				throw message;
 			break;
 		}
@@ -214,7 +214,7 @@ docstring InsetBibtex::toolTip(BufferView const & /*bv*/, int /*x*/, int /*y*/)
 			tip += _("all uncited references");
 		else
 			tip += _("all cited references");
-	
+
 	if (toc) {
 		tip += ", ";
 		tip += _("included in TOC");
@@ -267,6 +267,9 @@ void InsetBibtex::latex(otexstream & os, OutputParams const & runparams) const
 	odocstringstream dbs;
 	bool didone = false;
 
+	// determine the export format
+	string const tex_format = flavor2format(runparams.flavor);
+
 	for (; it != en; ++it) {
 		string utf8input = to_utf8(*it);
 		string database =
@@ -282,7 +285,6 @@ void InsetBibtex::latex(otexstream & os, OutputParams const & runparams) const
 			database = removeExtension(in_file.mangledFileName());
 			FileName const out_file = makeAbsPath(database + ".bib",
 					buffer().masterBuffer()->temppath());
-
 			bool const success = in_file.copyTo(out_file);
 			if (!success) {
 				lyxerr << "Failed to copy '" << in_file
@@ -290,6 +292,7 @@ void InsetBibtex::latex(otexstream & os, OutputParams const & runparams) const
 				       << endl;
 			}
 		} else if (!runparams.inComment && runparams.nice && not_from_texmf) {
+			runparams.exportdata->addExternalFile(tex_format, try_in_file, database + ".bib");
 			if (!isValidLaTeXFileName(database)) {
 				frontend::Alert::warning(_("Invalid filename"),
 				         _("The following filename will cause troubles "
@@ -307,7 +310,7 @@ void InsetBibtex::latex(otexstream & os, OutputParams const & runparams) const
 
 		if (didone)
 			dbs << ',';
-		else 
+		else
 			didone = true;
 		// FIXME UNICODE
 		dbs << from_utf8(latex_path(database));
@@ -332,9 +335,12 @@ void InsetBibtex::latex(otexstream & os, OutputParams const & runparams) const
 			style = split(style, bibtotoc, ',');
 	}
 
+	if (style == "default")
+		style = buffer().params().biblio_style;
+
 	if (!style.empty()) {
 		string base = normalizeName(buffer(), runparams, style, ".bst");
-		FileName const try_in_file = 
+		FileName const try_in_file =
 			makeAbsPath(base + ".bst", buffer().filePath());
 		bool const not_from_texmf = try_in_file.isReadableFile();
 		// If this style does not come from texmf and we are not
@@ -405,9 +411,9 @@ support::FileNameList InsetBibtex::getBibFiles() const
 {
 	FileName path(buffer().filePath());
 	support::PathChanger p(path);
-	
+
 	support::FileNameList vec;
-	
+
 	vector<docstring> bibfilelist = getVectorFromString(getParam("bibfiles"));
 	vector<docstring>::const_iterator it = bibfilelist.begin();
 	vector<docstring>::const_iterator en = bibfilelist.end();
@@ -419,7 +425,7 @@ support::FileNameList InsetBibtex::getBibFiles() const
 		else
 			LYXERR0("Couldn't find " + to_utf8(*it) + " in InsetBibtex::getBibFiles()!");
 	}
-	
+
 	return vec;
 
 }
@@ -478,7 +484,7 @@ namespace {
 	/// @return true if a string of length > 0 could be read.
 	///
 	bool readTypeOrKey(docstring & val, ifdocstream & ifs,
-		docstring const & delimChars, docstring const & illegalChars, 
+		docstring const & delimChars, docstring const & illegalChars,
 		charCase chCase) {
 
 		char_type ch;
@@ -498,10 +504,10 @@ namespace {
 
 		// read value
 		bool legalChar = true;
-		while (ifs && !isSpace(ch) && 
+		while (ifs && !isSpace(ch) &&
 						 delimChars.find(ch) == docstring::npos &&
 						 (legalChar = (illegalChars.find(ch) == docstring::npos))
-					) 
+					)
 		{
 			if (chCase == makeLowerCase)
 				val += lowercase(ch);
@@ -509,7 +515,7 @@ namespace {
 				val += ch;
 			ifs.get(ch);
 		}
-		
+
 		if (!legalChar) {
 			ifs.putback(ch);
 			return false;
@@ -570,33 +576,33 @@ namespace {
 				do {
 					ifs.get(ch);
 				} while (ifs && isSpace(ch));
-				
+
 				if (!ifs)
 					return false;
-				
+
 				// We now have the first non-whitespace character
 				// We'll collapse adjacent whitespace.
 				bool lastWasWhiteSpace = false;
-				
+
  				// inside this delimited text braces must match.
  				// Thus we can have a closing delimiter only
  				// when nestLevel == 0
 				int nestLevel = 0;
- 
+
 				while (ifs && (nestLevel > 0 || ch != delim)) {
 					if (isSpace(ch)) {
 						lastWasWhiteSpace = true;
 						ifs.get(ch);
 						continue;
 					}
-					// We output the space only after we stop getting 
+					// We output the space only after we stop getting
 					// whitespace so as not to output any whitespace
 					// at the end of the value.
 					if (lastWasWhiteSpace) {
 						lastWasWhiteSpace = false;
 						val += ' ';
 					}
-					
+
 					val += ch;
 
 					// update nesting level
@@ -606,7 +612,7 @@ namespace {
 							break;
 						case '}':
 							--nestLevel;
-							if (nestLevel < 0) 
+							if (nestLevel < 0)
 								return false;
 							break;
 					}
@@ -763,13 +769,13 @@ void InsetBibtex::parseBibTeXFiles() const
 				// next char must be an equal sign
 				ifs.get(ch);
 				if (!ifs || ch != '=') {
-					lyxerr << "BibTeX Parser: No `=' after string name: " << 
+					lyxerr << "BibTeX Parser: No `=' after string name: " <<
 							name << "." << std::endl;
 					continue;
 				}
 
 				if (!readValue(value, ifs, strings)) {
-					lyxerr << "BibTeX Parser: Unable to read value for string: " << 
+					lyxerr << "BibTeX Parser: Unable to read value for string: " <<
 							name << "." << std::endl;
 					continue;
 				}
@@ -793,7 +799,7 @@ void InsetBibtex::parseBibTeXFiles() const
 				docstring key;
 
 				if (!readTypeOrKey(key, ifs, from_ascii(","), from_ascii("}"), keepCase)) {
-					lyxerr << "BibTeX Parser: Unable to read key for entry type:" << 
+					lyxerr << "BibTeX Parser: Unable to read key for entry type:" <<
 							entryType << "." << std::endl;
 					continue;
 				}
@@ -804,7 +810,7 @@ void InsetBibtex::parseBibTeXFiles() const
 				}
 
 				/////////////////////////////////////////////
-				// now we have a key, so we will add an entry 
+				// now we have a key, so we will add an entry
  				// (even if it's empty, as bibtex does)
 				//
 				// we now read the field = value pairs.
@@ -815,13 +821,13 @@ void InsetBibtex::parseBibTeXFiles() const
 				docstring value;
 				docstring data;
 				BibTeXInfo keyvalmap(key, entryType);
-				
+
 				bool readNext = removeWSAndComma(ifs);
- 
+
 				while (ifs && readNext) {
 
 					// read field name
-					if (!readTypeOrKey(name, ifs, from_ascii("="), 
+					if (!readTypeOrKey(name, ifs, from_ascii("="),
 					                   from_ascii("{}(),"), makeLowerCase) || !ifs)
 						break;
 
@@ -867,14 +873,14 @@ void InsetBibtex::parseBibTeXFiles() const
 FileName InsetBibtex::getBibTeXPath(docstring const & filename, Buffer const & buf)
 {
 	string texfile = changeExtension(to_utf8(filename), "bib");
-	// note that, if the filename can be found directly from the path, 
+	// note that, if the filename can be found directly from the path,
 	// findtexfile will just return a FileName object for that path.
 	FileName file(findtexfile(texfile, "bib"));
 	if (file.empty())
 		file = FileName(makeAbsPath(texfile, buf.filePath()));
 	return file;
 }
- 
+
 
 bool InsetBibtex::addDatabase(docstring const & db)
 {
@@ -916,30 +922,66 @@ void InsetBibtex::validate(LaTeXFeatures & features) const
 	// It'd be better to be able to get this from an InsetLayout, but at present
 	// InsetLayouts do not seem really to work for things that aren't InsetTexts.
 	if (features.runparams().flavor == OutputParams::HTML)
-		features.addPreambleSnippet("<style type=\"text/css\">\n"
-			"div.bibtexentry { margin-left: 2em; text-indent: -2em; }\n"
+		features.addCSSSnippet("div.bibtexentry { margin-left: 2em; text-indent: -2em; }\n"
 			"span.bibtexlabel:before{ content: \"[\"; }\n"
-			"span.bibtexlabel:after{ content: \"] \"; }\n"
-			"</style>");
+			"span.bibtexlabel:after{ content: \"] \"; }");
 }
 
 
-// FIXME 
+int InsetBibtex::plaintext(odocstringstream & os,
+       OutputParams const & op, size_t max_length) const
+{
+	docstring const reflabel = buffer().B_("References");
+
+	// We could output more information here, e.g., what databases are included
+	// and information about options. But I don't necessarily see any reason to
+	// do this right now.
+	if (op.for_tooltip || op.for_toc || op.for_search) {
+		os << '[' << reflabel << ']' << '\n';
+		return PLAINTEXT_NEWLINE;
+	}
+
+	BiblioInfo bibinfo = buffer().masterBibInfo();
+	bibinfo.makeCitationLabels(buffer());
+	vector<docstring> const & cites = bibinfo.citedEntries();
+
+	size_t start_size = os.str().size();
+	docstring refoutput;
+	refoutput += reflabel + "\n\n";
+
+	// Now we loop over the entries
+	vector<docstring>::const_iterator vit = cites.begin();
+	vector<docstring>::const_iterator const ven = cites.end();
+	for (; vit != ven; ++vit) {
+		if (start_size + refoutput.size() >= max_length)
+			break;
+		BiblioInfo::const_iterator const biit = bibinfo.find(*vit);
+		if (biit == bibinfo.end())
+			continue;
+		BibTeXInfo const & entry = biit->second;
+		refoutput += "[" + entry.label() + "] ";
+		// FIXME Right now, we are calling BibInfo::getInfo on the key,
+		// which will give us all the cross-referenced info. But for every
+		// entry, so there's a lot of repitition. This should be fixed.
+		refoutput += bibinfo.getInfo(entry.key(), buffer(), false) + "\n\n";
+	}
+	os << refoutput;
+	return refoutput.size();
+}
+
+
+// FIXME
 // docstring InsetBibtex::entriesAsXHTML(vector<docstring> const & entries)
 // And then here just: entriesAsXHTML(buffer().masterBibInfo().citedEntries())
 docstring InsetBibtex::xhtml(XHTMLStream & xs, OutputParams const &) const
 {
 	BiblioInfo const & bibinfo = buffer().masterBibInfo();
-	vector<docstring> const & cites = bibinfo.citedEntries();
-	CiteEngine const engine = buffer().params().citeEngine();
-	bool const numbers = 
-		(engine == ENGINE_BASIC || engine == ENGINE_NATBIB_NUMERICAL);
+	bool const all_entries = getParam("btprint") == "btPrintAll";
+	vector<docstring> const & cites = 
+	    all_entries ? bibinfo.getKeys() : bibinfo.citedEntries();
+
+	docstring const reflabel = buffer().B_("References");
 
-	docstring reflabel = from_ascii("References");
-	Language const * l = buffer().params().language;
-	if (l)
-		reflabel = translateIfPossible(reflabel, l->code());
-		
 	xs << html::StartTag("h2", "class='bibtex'")
 		<< reflabel
 		<< html::EndTag("h2")
@@ -952,36 +994,23 @@ docstring InsetBibtex::xhtml(XHTMLStream & xs, OutputParams const &) const
 		BiblioInfo::const_iterator const biit = bibinfo.find(*vit);
 		if (biit == bibinfo.end())
 			continue;
+
 		BibTeXInfo const & entry = biit->second;
-		xs << html::StartTag("div", "class='bibtexentry'");
-		// FIXME XHTML
-		// The same name/id problem we have elsewhere.
-		string const attr = "id='LyXCite-" + 
-				to_utf8(html::cleanAttr(entry.key())) + "'";
-		xs << html::CompTag("a", attr);
-		docstring citekey;
-		if (numbers)
-			citekey = entry.citeNumber();
-		else {
-			docstring const auth = entry.getAbbreviatedAuthor(buffer());
-			// we do it this way so as to access the xref, if necessary
-			// note that this also gives us the modifier
-			docstring const year = bibinfo.getYear(*vit, buffer(), true);
-			if (!auth.empty() && !year.empty())
-				citekey = auth + ' ' + year;
+		string const attr = "class='bibtexentry' id='LyXCite-" 
+		    + to_utf8(html::cleanAttr(entry.key())) + "'";
+		xs << html::StartTag("div", attr);
+		
+		// don't print labels if we're outputting all entries
+		if (!all_entries) {
+			xs << html::StartTag("span", "class='bibtexlabel'")
+				<< entry.label()
+				<< html::EndTag("span");
 		}
-		if (citekey.empty()) {
-			citekey = entry.label();
-			if (citekey.empty())
-				citekey = entry.key();
-		}
-		xs << html::StartTag("span", "class='bibtexlabel'")
-			<< citekey
-			<< html::EndTag("span");
+		
 		// FIXME Right now, we are calling BibInfo::getInfo on the key,
 		// which will give us all the cross-referenced info. But for every
 		// entry, so there's a lot of repitition. This should be fixed.
-		xs << html::StartTag("span", "class='bibtexinfo'") 
+		xs << html::StartTag("span", "class='bibtexinfo'")
 		   << XHTMLStream::ESCAPE_AND
 		   << bibinfo.getInfo(entry.key(), buffer(), true)
 		   << html::EndTag("span")
@@ -993,9 +1022,9 @@ docstring InsetBibtex::xhtml(XHTMLStream & xs, OutputParams const &) const
 }
 
 
-docstring InsetBibtex::contextMenuName() const
+string InsetBibtex::contextMenuName() const
 {
-	return from_ascii("context-bibtex");
+	return "context-bibtex";
 }
 
 
