@@ -147,6 +147,10 @@ void InsetRef::latex(otexstream & os, OutputParams const & rp) const
 		os << "\\mbox{";
 
 	if (cmd == "eqref" && buffer().params().use_refstyle) {
+		// we advertise this as printing "(n)", so we'll do that, at least
+		// for refstyle, since refstlye's own \eqref prints, by default,
+		// "equation n". if one wants \eqref, one can get it by using a
+		// formatted label in this case.
 		os << '(' << from_ascii("\\ref{") << data << from_ascii("})");
 	} 
 	else if (cmd == "formatted") {
@@ -169,7 +173,8 @@ void InsetRef::latex(otexstream & os, OutputParams const & rp) const
 }
 
 
-int InsetRef::plaintext(odocstream & os, OutputParams const &) const
+int InsetRef::plaintext(odocstringstream & os,
+        OutputParams const &, size_t) const
 {
 	docstring const str = getParam("reference");
 	os << '[' << str << ']';
@@ -223,10 +228,15 @@ docstring InsetRef::xhtml(XHTMLStream & xs, OutputParams const & op) const
 			        op.local_font->language()->lang());
 		else if (cmd == "eqref")
 			display_string = '(' + value + ')';
-		else if (cmd == "formatted"
-		         // we don't really have the ability to handle these 
-		         // properly in XHTML output
-		         || cmd == "nameref")
+		else if (cmd == "formatted")
+			display_string = il->prettyCounter();
+		else if (cmd == "nameref")
+			// FIXME We don't really have the ability to handle these
+			// properly in XHTML output yet (bug #8599).
+			// It might not be that hard to do. We have the InsetLabel,
+			// and we can presumably find its paragraph using the TOC.
+			// We could then output the contents of the paragraph using
+			// something?
 			display_string = il->prettyCounter();
 	} else 
 			display_string = ref;
@@ -244,7 +254,9 @@ docstring InsetRef::xhtml(XHTMLStream & xs, OutputParams const & op) const
 
 void InsetRef::toString(odocstream & os) const
 {
-	plaintext(os, OutputParams(0));
+	odocstringstream ods;
+	plaintext(ods, OutputParams(0));
+	os << ods.str();
 }
 
 
@@ -260,7 +272,7 @@ void InsetRef::updateBuffer(ParIterator const & it, UpdateType)
 {
 	docstring const & ref = getParam("reference");
 	// register this inset into the buffer reference cache.
-	buffer().references(ref).push_back(make_pair(this, it));
+	buffer().addReference(ref, this, it);
 
 	docstring label;
 	for (int i = 0; !types[i].latex_name.empty(); ++i) {
@@ -291,7 +303,7 @@ void InsetRef::updateBuffer(ParIterator const & it, UpdateType)
 }
 
 
-void InsetRef::addToToc(DocIterator const & cpit) const
+void InsetRef::addToToc(DocIterator const & cpit, bool output_active) const
 {
 	docstring const & label = getParam("reference");
 	if (buffer().insetLabel(label))
@@ -301,7 +313,7 @@ void InsetRef::addToToc(DocIterator const & cpit) const
 	// It seems that this reference does not point to any valid label.
 	screen_label_ = _("BROKEN: ") + screen_label_;
 	Toc & toc = buffer().tocBackend().toc("label");
-	toc.push_back(TocItem(cpit, 0, screen_label_));
+	toc.push_back(TocItem(cpit, 0, screen_label_, output_active));
 }
 
 
@@ -332,7 +344,7 @@ void InsetRef::validate(LaTeXFeatures & features) const
 				features.addPreambleSnippet("\\let\\pr@chap=\\pr@cha");
 		}
 	} else if (cmd == "eqref" && !buffer().params().use_refstyle)
-		// refstyle defines its own version
+		// with refstyle, we simply output "(\ref{label})"
 		features.require("amsmath");
 	else if (cmd == "nameref")
 		features.require("nameref");
