@@ -45,6 +45,7 @@
 #include "support/convert.h"
 #include "support/debug.h"
 #include "support/docstream.h"
+#include "support/FileName.h"
 #include "support/gettext.h"
 #include "support/lassert.h"
 #include "support/lstrings.h"
@@ -94,7 +95,7 @@ private:
 
 
 int findForward(DocIterator & cur, MatchString const & match,
-		 bool find_del = true)
+		bool find_del = true)
 {
 	for (; cur; cur.forwardChar())
 		if (cur.inTexted()) {
@@ -107,7 +108,7 @@ int findForward(DocIterator & cur, MatchString const & match,
 
 
 int findBackwards(DocIterator & cur, MatchString const & match,
-		 bool find_del = true)
+		  bool find_del = true)
 {
 	while (cur) {
 		cur.backwardChar();
@@ -121,23 +122,6 @@ int findBackwards(DocIterator & cur, MatchString const & match,
 }
 
 
-bool findChange(DocIterator & cur, bool next)
-{
-	if (!next)
-		cur.backwardPos();
-	for (; cur; next ? cur.forwardPos() : cur.backwardPos())
-		if (cur.inTexted() && cur.paragraph().isChanged(cur.pos())) {
-			if (!next)
-				// if we search backwards, take a step forward
-				// to correctly set the anchor
-				cur.forwardPos();
-			return true;
-		}
-
-	return false;
-}
-
-
 bool searchAllowed(docstring const & str)
 {
 	if (str.empty()) {
@@ -149,7 +133,7 @@ bool searchAllowed(docstring const & str)
 
 
 bool findOne(BufferView * bv, docstring const & searchstr,
-	bool case_sens, bool whole, bool forward, bool find_del = true)
+	     bool case_sens, bool whole, bool forward, bool find_del = true)
 {
 	if (!searchAllowed(searchstr))
 		return false;
@@ -160,8 +144,9 @@ bool findOne(BufferView * bv, docstring const & searchstr,
 
 	MatchString const match(searchstr, case_sens, whole);
 
-	int match_len = forward ? findForward(cur, match, find_del) :
-			  findBackwards(cur, match, find_del);
+	int match_len = forward
+		? findForward(cur, match, find_del)
+		: findBackwards(cur, match, find_del);
 
 	if (match_len > 0)
 		bv->putSelectionAt(cur, match_len, !forward);
@@ -195,11 +180,13 @@ int replaceAll(BufferView * bv,
 		pos_type const pos = cur.pos();
 		Font const font = cur.paragraph().getFontSettings(buf.params(), pos);
 		cur.recordUndo();
-		int striked = ssize - cur.paragraph().eraseChars(pos, pos + match_len,
-							    buf.params().trackChanges);
+		int striked = ssize -
+			cur.paragraph().eraseChars(pos, pos + match_len,
+						   buf.params().trackChanges);
 		cur.paragraph().insert(pos, replacestr, font,
-				       Change(buf.params().trackChanges ?
-					      Change::INSERTED : Change::UNCHANGED));
+				       Change(buf.params().trackChanges
+					      ? Change::INSERTED
+					      : Change::UNCHANGED));
 		for (int i = 0; i < rsize + striked; ++i)
 			cur.forwardChar();
 		++num;
@@ -231,21 +218,23 @@ int replaceAll(BufferView * bv,
 // returns the number of replacements made (one, if any) and 
 // whether anything at all was done.
 pair<bool, int> replaceOne(BufferView * bv, docstring searchstr,
-	    docstring const & replacestr, bool case_sens, 
-			bool whole, bool forward, bool findnext)
+			   docstring const & replacestr, bool case_sens,
+			   bool whole, bool forward, bool findnext)
 {
 	Cursor & cur = bv->cursor();
 	if (!cur.selection()) {
 		// no selection, non-empty search string: find it
 		if (!searchstr.empty()) {
 			findOne(bv, searchstr, case_sens, whole, forward);
-			return pair<bool, int>(true, 0);
+			return make_pair(true, 0);
 		}
 		// empty search string
 		if (!cur.inTexted())
 			// bail in math
-			return pair<bool, int>(false, 0);
-		// select current word and treat it as the search string
+			return make_pair(false, 0);
+		// select current word and treat it as the search string.
+		// This causes a minor bug as undo will restore this selection,
+		// which the user did not create (#8986).
 		cur.innerText()->selectWord(cur, WHOLE_WORD);
 		searchstr = cur.selectionAsString(false);
 	}
@@ -253,42 +242,44 @@ pair<bool, int> replaceOne(BufferView * bv, docstring searchstr,
 	// if we still don't have a search string, report the error
 	// and abort.
 	if (!searchAllowed(searchstr))
-		return pair<bool, int>(false, 0);
+		return make_pair(false, 0);
 	
 	bool have_selection = cur.selection();
 	docstring const selected = cur.selectionAsString(false);
 	bool match = 
-		case_sens ? searchstr == selected
-	            : compare_no_case(searchstr, selected) == 0;
+		case_sens
+		? searchstr == selected
+		: compare_no_case(searchstr, selected) == 0;
 
 	// no selection or current selection is not search word:
 	// just find the search word
 	if (!have_selection || !match) {
 		findOne(bv, searchstr, case_sens, whole, forward);
-		return pair<bool, int>(true, 0);
+		return make_pair(true, 0);
 	}
 
 	// we're now actually ready to replace. if the buffer is
 	// read-only, we can't, though.
 	if (bv->buffer().isReadonly())
-		return pair<bool, int>(false, 0);
+		return make_pair(false, 0);
 
 	cap::replaceSelectionWithString(cur, replacestr);
 	if (forward) {
 		cur.pos() += replacestr.length();
-		LASSERT(cur.pos() <= cur.lastpos(), /* */);
+		LASSERT(cur.pos() <= cur.lastpos(),
+		        cur.pos() = cur.lastpos());
 	}
 	if (findnext)
 		findOne(bv, searchstr, case_sens, whole, forward, false);
 
-	return pair<bool, int>(true, 1);
+	return make_pair(true, 1);
 }
 
 } // namespace anon
 
 
 docstring const find2string(docstring const & search,
-			 bool casesensitive, bool matchword, bool forward)
+			    bool casesensitive, bool matchword, bool forward)
 {
 	odocstringstream ss;
 	ss << search << '\n'
@@ -300,8 +291,9 @@ docstring const find2string(docstring const & search,
 
 
 docstring const replace2string(docstring const & replace,
-	docstring const & search, bool casesensitive, bool matchword,
-	bool all, bool forward, bool findnext)
+			       docstring const & search,
+			       bool casesensitive, bool matchword,
+			       bool all, bool forward, bool findnext)
 {
 	odocstringstream ss;
 	ss << replace << '\n'
@@ -365,7 +357,7 @@ bool lyxreplace(BufferView * bv,
 			replace_count = replaceAll(bv, search, rplc, casesensitive, matchword);
 			update = replace_count > 0;
 		} else {
-			pair<bool, int> rv = 
+			pair<bool, int> rv =
 				replaceOne(bv, search, rplc, casesensitive, matchword, forward, findnext);
 			update = rv.first;
 			replace_count = rv.second;
@@ -374,7 +366,7 @@ bool lyxreplace(BufferView * bv,
 		Buffer const & buf = bv->buffer();
 		if (!update) {
 			// emit message signal.
-			buf.message(_("String not found!"));
+			buf.message(_("String not found."));
 		} else {
 			if (replace_count == 0) {
 				buf.message(_("String found."));
@@ -392,21 +384,27 @@ bool lyxreplace(BufferView * bv,
 		if (findOne(bv, search, casesensitive, matchword, forward))
 			update = true;
 		else
-			bv->message(_("String not found!"));
+			bv->message(_("String not found."));
 	}
 	return update;
 }
 
 
-bool findNextChange(BufferView * bv)
+namespace {
+bool findChange(DocIterator & cur, bool next)
 {
-	return findChange(bv, true);
-}
+	if (!next)
+		cur.backwardPos();
+	for (; cur; next ? cur.forwardPos() : cur.backwardPos())
+		if (cur.inTexted() && cur.paragraph().isChanged(cur.pos())) {
+			if (!next)
+				// if we search backwards, take a step forward
+				// to correctly set the anchor
+				cur.forwardPos();
+			return true;
+		}
 
-
-bool findPreviousChange(BufferView * bv)
-{
-	return findChange(bv, false);
+	return false;
 }
 
 
@@ -480,6 +478,20 @@ bool findChange(BufferView * bv, bool next)
 
 	return true;
 }
+}
+
+
+bool findNextChange(BufferView * bv)
+{
+	return findChange(bv, true);
+}
+
+
+bool findPreviousChange(BufferView * bv)
+{
+	return findChange(bv, false);
+}
+
 
 namespace {
 
@@ -489,54 +501,62 @@ typedef vector<pair<string, string> > Escapes;
 /// @note Beware of order
 Escapes const & get_regexp_escapes()
 {
+	typedef std::pair<std::string, std::string> P;
+
 	static Escapes escape_map;
 	if (escape_map.empty()) {
-		escape_map.push_back(pair<string, string>("$", "_x_$"));
-		escape_map.push_back(pair<string, string>("{", "_x_{"));
-		escape_map.push_back(pair<string, string>("}", "_x_}"));
-		escape_map.push_back(pair<string, string>("[", "_x_["));
-		escape_map.push_back(pair<string, string>("]", "_x_]"));
-		escape_map.push_back(pair<string, string>("(", "_x_("));
-		escape_map.push_back(pair<string, string>(")", "_x_)"));
-		escape_map.push_back(pair<string, string>("+", "_x_+"));
-		escape_map.push_back(pair<string, string>("*", "_x_*"));
-		escape_map.push_back(pair<string, string>(".", "_x_."));
-		escape_map.push_back(pair<string, string>("\\", "(?:\\\\|\\\\backslash)"));
-		escape_map.push_back(pair<string, string>("~", "(?:\\\\textasciitilde|\\\\sim)"));
-		escape_map.push_back(pair<string, string>("^", "(?:\\^|\\\\textasciicircum\\{\\}|\\\\mathcircumflex)"));
-		escape_map.push_back(pair<string, string>("_x_", "\\"));
+		escape_map.push_back(P("$", "_x_$"));
+		escape_map.push_back(P("{", "_x_{"));
+		escape_map.push_back(P("}", "_x_}"));
+		escape_map.push_back(P("[", "_x_["));
+		escape_map.push_back(P("]", "_x_]"));
+		escape_map.push_back(P("(", "_x_("));
+		escape_map.push_back(P(")", "_x_)"));
+		escape_map.push_back(P("+", "_x_+"));
+		escape_map.push_back(P("*", "_x_*"));
+		escape_map.push_back(P(".", "_x_."));
+		escape_map.push_back(P("\\", "(?:\\\\|\\\\backslash)"));
+		escape_map.push_back(P("~", "(?:\\\\textasciitilde|\\\\sim)"));
+		escape_map.push_back(P("^", "(?:\\^|\\\\textasciicircum\\{\\}|\\\\mathcircumflex)"));
+		escape_map.push_back(P("_x_", "\\"));
 	}
 	return escape_map;
 }
 
 /// A map of lyx escaped strings and their unescaped equivalent.
-Escapes const & get_lyx_unescapes() {
+Escapes const & get_lyx_unescapes()
+{
+	typedef std::pair<std::string, std::string> P;
+
 	static Escapes escape_map;
 	if (escape_map.empty()) {
-		escape_map.push_back(pair<string, string>("\\%", "%"));
-		escape_map.push_back(pair<string, string>("\\mathcircumflex ", "^"));
-		escape_map.push_back(pair<string, string>("\\mathcircumflex", "^"));
-		escape_map.push_back(pair<string, string>("\\backslash ", "\\"));
-		escape_map.push_back(pair<string, string>("\\backslash", "\\"));
-		escape_map.push_back(pair<string, string>("\\\\{", "_x_<"));
-		escape_map.push_back(pair<string, string>("\\\\}", "_x_>"));
-		escape_map.push_back(pair<string, string>("\\sim ", "~"));
-		escape_map.push_back(pair<string, string>("\\sim", "~"));
+		escape_map.push_back(P("\\%", "%"));
+		escape_map.push_back(P("\\mathcircumflex ", "^"));
+		escape_map.push_back(P("\\mathcircumflex", "^"));
+		escape_map.push_back(P("\\backslash ", "\\"));
+		escape_map.push_back(P("\\backslash", "\\"));
+		escape_map.push_back(P("\\\\{", "_x_<"));
+		escape_map.push_back(P("\\\\}", "_x_>"));
+		escape_map.push_back(P("\\sim ", "~"));
+		escape_map.push_back(P("\\sim", "~"));
 	}
 	return escape_map;
 }
 
 /// A map of escapes turning a regexp matching text to one matching latex.
-Escapes const & get_regexp_latex_escapes() {
+Escapes const & get_regexp_latex_escapes()
+{
+	typedef std::pair<std::string, std::string> P;
+
 	static Escapes escape_map;
 	if (escape_map.empty()) {
-		escape_map.push_back(pair<string, string>("\\\\", "(?:\\\\\\\\|\\\\backslash|\\\\textbackslash\\{\\})"));
-		escape_map.push_back(pair<string, string>("(<?!\\\\\\\\textbackslash)\\{", "\\\\\\{"));
-		escape_map.push_back(pair<string, string>("(<?!\\\\\\\\textbackslash\\\\\\{)\\}", "\\\\\\}"));
-		escape_map.push_back(pair<string, string>("\\[", "\\{\\[\\}"));
-		escape_map.push_back(pair<string, string>("\\]", "\\{\\]\\}"));
-		escape_map.push_back(pair<string, string>("\\^", "(?:\\^|\\\\textasciicircum\\{\\}|\\\\mathcircumflex)"));
-		escape_map.push_back(pair<string, string>("%", "\\\\\\%"));
+		escape_map.push_back(P("\\\\", "(?:\\\\\\\\|\\\\backslash|\\\\textbackslash\\{\\})"));
+		escape_map.push_back(P("(<?!\\\\\\\\textbackslash)\\{", "\\\\\\{"));
+		escape_map.push_back(P("(<?!\\\\\\\\textbackslash\\\\\\{)\\}", "\\\\\\}"));
+		escape_map.push_back(P("\\[", "\\{\\[\\}"));
+		escape_map.push_back(P("\\]", "\\{\\]\\}"));
+		escape_map.push_back(P("\\^", "(?:\\^|\\\\textasciicircum\\{\\}|\\\\mathcircumflex)"));
+		escape_map.push_back(P("%", "\\\\\\%"));
 	}
 	return escape_map;
 }
@@ -562,6 +582,7 @@ string apply_escapes(string s, Escapes const & escape_map)
 	return s;
 }
 
+
 /// Within \regexp{} apply get_lyx_unescapes() only (i.e., preserve regexp semantics of the string),
 /// while outside apply get_lyx_unescapes()+get_regexp_escapes().
 /// If match_latex is true, then apply regexp_latex_escapes() to \regexp{} contents as well.
@@ -583,7 +604,8 @@ string escape_for_regex(string s, bool match_latex)
 		LYXERR(Debug::FIND, "new_pos: " << new_pos);
 		if (new_pos == s.size())
 			break;
-		size_t end_pos = s.find("\\endregexp{}}", new_pos + 8); // find_matching_brace(s, new_pos + 7);
+		// Might fail if \\endregexp{} is preceeded by unexpected stuff (weird escapes)
+		size_t end_pos = s.find("\\endregexp{}}", new_pos + 8);
 		LYXERR(Debug::FIND, "end_pos: " << end_pos);
 		t = s.substr(new_pos + 8, end_pos - (new_pos + 8));
 		LYXERR(Debug::FIND, "t in regexp      : " << t);
@@ -607,9 +629,10 @@ string escape_for_regex(string s, bool match_latex)
 	return s;
 }
 
+
 /// Wrapper for lyx::regex_replace with simpler interface
 bool regex_replace(string const & s, string & t, string const & searchstr,
-	string const & replacestr)
+		   string const & replacestr)
 {
 	lyx::regex e(searchstr);
 	ostringstream oss;
@@ -621,6 +644,7 @@ bool regex_replace(string const & s, string & t, string const & searchstr,
 	return rv;
 }
 
+
 /** Checks if supplied string segment is well-formed from the standpoint of matching open-closed braces.
  **
  ** Verify that closed braces exactly match open braces. This avoids that, for example,
@@ -662,6 +686,7 @@ bool braces_match(string::const_iterator const & beg,
 	return true;
 }
 
+
 /** The class performing a match between a position in the document and the FindAdvOptions.
  **/
 class MatchStringAdv {
@@ -746,7 +771,8 @@ static docstring buffer_to_latex(Buffer & buffer)
 }
 
 
-static docstring stringifySearchBuffer(Buffer & buffer, FindAndReplaceOptions const & opt) {
+static docstring stringifySearchBuffer(Buffer & buffer, FindAndReplaceOptions const & opt)
+{
 	docstring str;
 	if (!opt.ignoreformat) {
 		str = buffer_to_latex(buffer);
@@ -759,9 +785,9 @@ static docstring stringifySearchBuffer(Buffer & buffer, FindAndReplaceOptions co
 		for (pos_type pit = pos_type(0); pit < (pos_type)buffer.paragraphs().size(); ++pit) {
 			Paragraph const & par = buffer.paragraphs().at(pit);
 			LYXERR(Debug::FIND, "Adding to search string: '"
-				<< par.stringify(pos_type(0), par.size(),
-						 AS_STR_INSETS, runparams)
-				<< "'");
+			       << par.stringify(pos_type(0), par.size(),
+						AS_STR_INSETS, runparams)
+			       << "'");
 			str += par.stringify(pos_type(0), par.size(),
 					     AS_STR_INSETS, runparams);
 		}
@@ -771,7 +797,8 @@ static docstring stringifySearchBuffer(Buffer & buffer, FindAndReplaceOptions co
 
 
 /// Return separation pos between the leading material and the rest
-static size_t identifyLeading(string const & s)  {
+static size_t identifyLeading(string const & s)
+{
 	string t = s;
 	// @TODO Support \item[text]
 	while (regex_replace(t, t, "^\\\\(emph|textbf|subsubsection|subsection|section|subparagraph|paragraph|part)\\*?\\{", "")
@@ -785,7 +812,8 @@ static size_t identifyLeading(string const & s)  {
 
 
 // Remove trailing closure of math, macros and environments, so to catch parts of them.
-static int identifyClosing(string & t) {
+static int identifyClosing(string & t)
+{
 	int open_braces = 0;
 	do {
 		LYXERR(Debug::FIND, "identifyClosing(): t now is '" << t << "'");
@@ -817,7 +845,16 @@ MatchStringAdv::MatchStringAdv(lyx::Buffer & buf, FindAndReplaceOptions const &
 	close_wildcards = 0;
 
 	size_t lead_size = 0;
-	if (!opt.ignoreformat) {
+	if (opt.ignoreformat) {
+                if (!use_regexp) {
+                        // if par_as_string_nolead were emty, 
+                        // the following call to findAux will always *find* the string
+                        // in the checked data, and thus always using the slow
+                        // examining of the current text part.
+                        par_as_string_nolead = par_as_string;
+                }
+        }
+        else {
 		lead_size = identifyLeading(par_as_string);
 		lead_as_string = par_as_string.substr(0, lead_size);
 		par_as_string_nolead = par_as_string.substr(lead_size, par_as_string.size() - lead_size);
@@ -843,14 +880,14 @@ MatchStringAdv::MatchStringAdv(lyx::Buffer & buf, FindAndReplaceOptions const &
 		if (
 			// Insert .* before trailing '\$' ('$' has been escaped by escape_for_regex)
 			regex_replace(par_as_string, par_as_string, "(.*[^\\\\])(\\\\\\$)\\'", "$1(.*?)$2")
-				// Insert .* before trailing '\\\]' ('\]' has been escaped by escape_for_regex)
-				|| regex_replace(par_as_string, par_as_string, "(.*[^\\\\])( \\\\\\\\\\\\\\])\\'", "$1(.*?)$2")
-				// Insert .* before trailing '\\end\{...}' ('\end{...}' has been escaped by escape_for_regex)
-				|| regex_replace(par_as_string, par_as_string, 
-					"(.*[^\\\\])( \\\\\\\\end\\\\\\{[a-zA-Z_]*)(\\\\\\*)?(\\\\\\})\\'", "$1(.*?)$2$3$4")
-				// Insert .* before trailing '\}' ('}' has been escaped by escape_for_regex)
-				|| regex_replace(par_as_string, par_as_string, "(.*[^\\\\])(\\\\\\})\\'", "$1(.*?)$2")
-		) {
+			// Insert .* before trailing '\\\]' ('\]' has been escaped by escape_for_regex)
+			|| regex_replace(par_as_string, par_as_string, "(.*[^\\\\])( \\\\\\\\\\\\\\])\\'", "$1(.*?)$2")
+			// Insert .* before trailing '\\end\{...}' ('\end{...}' has been escaped by escape_for_regex)
+			|| regex_replace(par_as_string, par_as_string,
+					 "(.*[^\\\\])( \\\\\\\\end\\\\\\{[a-zA-Z_]*)(\\\\\\*)?(\\\\\\})\\'", "$1(.*?)$2$3$4")
+			// Insert .* before trailing '\}' ('}' has been escaped by escape_for_regex)
+			|| regex_replace(par_as_string, par_as_string, "(.*[^\\\\])(\\\\\\})\\'", "$1(.*?)$2")
+			) {
 			++close_wildcards;
 		}
 		LYXERR(Debug::FIND, "par_as_string now is '" << par_as_string << "'");
@@ -872,6 +909,9 @@ MatchStringAdv::MatchStringAdv(lyx::Buffer & buf, FindAndReplaceOptions const &
 
 int MatchStringAdv::findAux(DocIterator const & cur, int len, bool at_begin) const
 {
+	if (at_begin &&
+		(opt.restr == FindAndReplaceOptions::R_ONLY_MATHS && !cur.inMathed()) )
+		return 0;
 	docstring docstr = stringifyFromForSearch(opt, cur, len);
 	LYXERR(Debug::FIND, "Matching against     '" << lyx::to_utf8(docstr) << "'");
 	string str = normalize(docstr, true);
@@ -922,14 +962,18 @@ int MatchStringAdv::operator()(DocIterator const & cur, int len, bool at_begin)
 {
 	int res = findAux(cur, len, at_begin);
 	LYXERR(Debug::FIND,
-	       "res=" << res << ", at_begin=" << at_begin << ", matchword=" << opt.matchword << ", inTexted=" << cur.inTexted());
+	       "res=" << res << ", at_begin=" << at_begin
+	       << ", matchword=" << opt.matchword
+	       << ", inTexted=" << cur.inTexted());
 	if (res == 0 || !at_begin || !opt.matchword || !cur.inTexted())
 		return res;
 	Paragraph const & par = cur.paragraph();
-	bool ws_left = cur.pos() > 0 ?
-		par.isWordSeparator(cur.pos() - 1) : true;
-	bool ws_right = cur.pos() + res < par.size() ?
-		par.isWordSeparator(cur.pos() + res) : true;
+	bool ws_left = (cur.pos() > 0)
+		? par.isWordSeparator(cur.pos() - 1)
+		: true;
+	bool ws_right = (cur.pos() + res < par.size())
+		? par.isWordSeparator(cur.pos() + res)
+		: true;
 	LYXERR(Debug::FIND,
 	       "cur.pos()=" << cur.pos() << ", res=" << res
 	       << ", separ: " << ws_left << ", " << ws_right
@@ -948,10 +992,10 @@ string MatchStringAdv::normalize(docstring const & s, bool hack_braces) const
 	else
 		t = lyx::to_utf8(s);
 	// Remove \n at begin
-	while (t.size() > 0 && t[0] == '\n')
+	while (!t.empty() && t[0] == '\n')
 		t = t.substr(1);
 	// Remove \n at end
-	while (t.size() > 0 && t[t.size() - 1] == '\n')
+	while (!t.empty() && t[t.size() - 1] == '\n')
 		t = t.substr(0, t.size() - 1);
 	size_t pos;
 	// Replace all other \n with spaces
@@ -982,32 +1026,34 @@ docstring stringifyFromCursor(DocIterator const & cur, int len)
 {
 	LYXERR(Debug::FIND, "Stringifying with len=" << len << " from cursor at pos: " << cur);
 	if (cur.inTexted()) {
-			Paragraph const & par = cur.paragraph();
-			// TODO what about searching beyond/across paragraph breaks ?
-			// TODO Try adding a AS_STR_INSERTS as last arg
-			pos_type end = ( len == -1 || cur.pos() + len > int(par.size()) ) ?
-				int(par.size()) : cur.pos() + len;
-			OutputParams runparams(&cur.buffer()->params().encoding());
-			odocstringstream os;
-			runparams.nice = true;
-			runparams.flavor = OutputParams::LATEX;
-			runparams.linelen = 100000; //lyxrc.plaintext_linelen;
-			// No side effect of file copying and image conversion
-			runparams.dryrun = true;
-			LYXERR(Debug::FIND, "Stringifying with cur: " 
-				<< cur << ", from pos: " << cur.pos() << ", end: " << end);
-			return par.stringify(cur.pos(), end, AS_STR_INSETS, runparams);
+		Paragraph const & par = cur.paragraph();
+		// TODO what about searching beyond/across paragraph breaks ?
+		// TODO Try adding a AS_STR_INSERTS as last arg
+		pos_type end = ( len == -1 || cur.pos() + len > int(par.size()) ) ?
+			int(par.size()) : cur.pos() + len;
+		OutputParams runparams(&cur.buffer()->params().encoding());
+		odocstringstream os;
+		runparams.nice = true;
+		runparams.flavor = OutputParams::LATEX;
+		runparams.linelen = 100000; //lyxrc.plaintext_linelen;
+		// No side effect of file copying and image conversion
+		runparams.dryrun = true;
+		LYXERR(Debug::FIND, "Stringifying with cur: "
+		       << cur << ", from pos: " << cur.pos() << ", end: " << end);
+		return par.stringify(cur.pos(), end, AS_STR_INSETS, runparams);
 	} else if (cur.inMathed()) {
-			docstring s;
-			CursorSlice cs = cur.top();
-			MathData md = cs.cell();
-			MathData::const_iterator it_end = 
-				( ( len == -1 || cs.pos() + len > int(md.size()) )
-					? md.end() : md.begin() + cs.pos() + len );
-			for (MathData::const_iterator it = md.begin() + cs.pos(); it != it_end; ++it)
-				s = s + asString(*it);
-			LYXERR(Debug::FIND, "Stringified math: '" << s << "'");
-			return s;
+		docstring s;
+		CursorSlice cs = cur.top();
+		MathData md = cs.cell();
+		MathData::const_iterator it_end =
+			(( len == -1 || cs.pos() + len > int(md.size()))
+			 ? md.end()
+			 : md.begin() + cs.pos() + len );
+		for (MathData::const_iterator it = md.begin() + cs.pos();
+		     it != it_end; ++it)
+			s = s + asString(*it);
+		LYXERR(Debug::FIND, "Stringified math: '" << s << "'");
+		return s;
 	}
 	LYXERR(Debug::FIND, "Don't know how to stringify from here: " << cur);
 	return docstring();
@@ -1022,9 +1068,9 @@ docstring latexifyFromCursor(DocIterator const & cur, int len)
 {
 	LYXERR(Debug::FIND, "Latexifying with len=" << len << " from cursor at pos: " << cur);
 	LYXERR(Debug::FIND, "  with cur.lastpost=" << cur.lastpos() << ", cur.lastrow="
-		<< cur.lastrow() << ", cur.lastcol=" << cur.lastcol());
+	       << cur.lastrow() << ", cur.lastcol=" << cur.lastcol());
 	Buffer const & buf = *cur.buffer();
-	LASSERT(buf.params().isLatex(), /* */);
+	LBUFERR(buf.params().isLatex());
 
 	TexRow texrow;
 	odocstringstream ods;
@@ -1042,24 +1088,27 @@ docstring latexifyFromCursor(DocIterator const & cur, int len)
 		if (len != -1 && endpos > cur.pos() + len)
 			endpos = cur.pos() + len;
 		TeXOnePar(buf, *cur.innerText(), cur.pit(), os, runparams,
-			string(), cur.pos(), endpos);
+			  string(), cur.pos(), endpos);
 		LYXERR(Debug::FIND, "Latexified text: '" << lyx::to_utf8(ods.str()) << "'");
 	} else if (cur.inMathed()) {
 		// Retrieve the math environment type, and add '$' or '$[' or others (\begin{equation}) accordingly
 		for (int s = cur.depth() - 1; s >= 0; --s) {
-				CursorSlice const & cs = cur[s];
-				if (cs.asInsetMath() && cs.asInsetMath() && cs.asInsetMath()->asHullInset()) {
-						WriteStream ws(ods);
-						cs.asInsetMath()->asHullInset()->header_write(ws);
-						break;
-				}
+			CursorSlice const & cs = cur[s];
+			if (cs.asInsetMath() && cs.asInsetMath() && cs.asInsetMath()->asHullInset()) {
+				WriteStream ws(ods);
+				cs.asInsetMath()->asHullInset()->header_write(ws);
+				break;
+			}
 		}
 
 		CursorSlice const & cs = cur.top();
 		MathData md = cs.cell();
-		MathData::const_iterator it_end = ( ( len == -1 || cs.pos() + len > int(md.size()) )
-			? md.end() : md.begin() + cs.pos() + len );
-		for (MathData::const_iterator it = md.begin() + cs.pos(); it != it_end; ++it)
+		MathData::const_iterator it_end =
+			((len == -1 || cs.pos() + len > int(md.size()))
+			 ? md.end()
+			 : md.begin() + cs.pos() + len);
+		for (MathData::const_iterator it = md.begin() + cs.pos();
+		     it != it_end; ++it)
 			ods << asString(*it);
 
 		// Retrieve the math environment type, and add '$' or '$]'
@@ -1098,7 +1147,7 @@ int findAdvFinalize(DocIterator & cur, MatchStringAdv const & match)
 		cur.forwardPos();
 	} while (cur && cur.depth() > d && match(cur) > 0);
 	cur = old_cur;
-	LASSERT(match(cur) > 0, /* */);
+	LASSERT(match(cur) > 0, return 0);
 	LYXERR(Debug::FIND, "Ok");
 
 	// Compute the match length
@@ -1128,12 +1177,12 @@ int findForwardAdv(DocIterator & cur, MatchStringAdv & match)
 {
 	if (!cur)
 		return 0;
-	while (cur) {
+	while (!theApp()->longOperationCancelled() && cur) {
 		LYXERR(Debug::FIND, "findForwardAdv() cur: " << cur);
 		int match_len = match(cur, -1, false);
 		LYXERR(Debug::FIND, "match_len: " << match_len);
 		if (match_len) {
-			for (; cur; cur.forwardPos()) {
+			for (; !theApp()->longOperationCancelled() && cur; cur.forwardPos()) {
 				LYXERR(Debug::FIND, "Advancing cur: " << cur);
 				int match_len = match(cur);
 				LYXERR(Debug::FIND, "match_len: " << match_len);
@@ -1186,7 +1235,8 @@ int findMostBackwards(DocIterator & cur, MatchStringAdv const & match)
 
 
 /// Finds backwards
-int findBackwardsAdv(DocIterator & cur, MatchStringAdv & match) {
+int findBackwardsAdv(DocIterator & cur, MatchStringAdv & match)
+{
 	if (! cur)
 		return 0;
 	// Backup of original position
@@ -1230,7 +1280,7 @@ int findBackwardsAdv(DocIterator & cur, MatchStringAdv & match) {
 		else
 			cur.backwardPos();
 		pit_changed = true;
-	} while (true);
+	} while (!theApp()->longOperationCancelled());
 	return 0;
 }
 
@@ -1239,9 +1289,10 @@ int findBackwardsAdv(DocIterator & cur, MatchStringAdv & match) {
 
 
 docstring stringifyFromForSearch(FindAndReplaceOptions const & opt,
-	DocIterator const & cur, int len)
+				 DocIterator const & cur, int len)
 {
-	LASSERT(cur.pos() >= 0 && cur.pos() <= cur.lastpos(), /* */);
+	LASSERT(cur.pos() >= 0 && cur.pos() <= cur.lastpos(),
+	        return docstring());
 	if (!opt.ignoreformat)
 		return latexifyFromCursor(cur, len);
 	else
@@ -1249,13 +1300,14 @@ docstring stringifyFromForSearch(FindAndReplaceOptions const & opt,
 }
 
 
-FindAndReplaceOptions::FindAndReplaceOptions(docstring const & find_buf_name, bool casesensitive,
+FindAndReplaceOptions::FindAndReplaceOptions(
+	docstring const & find_buf_name, bool casesensitive,
 	bool matchword, bool forward, bool expandmacros, bool ignoreformat,
 	docstring const & repl_buf_name, bool keep_case,
-	SearchScope scope)
+	SearchScope scope, SearchRestriction restr)
 	: find_buf_name(find_buf_name), casesensitive(casesensitive), matchword(matchword),
-	forward(forward), expandmacros(expandmacros), ignoreformat(ignoreformat),
-	repl_buf_name(repl_buf_name), keep_case(keep_case), scope(scope)
+	  forward(forward), expandmacros(expandmacros), ignoreformat(ignoreformat),
+	  repl_buf_name(repl_buf_name), keep_case(keep_case), scope(scope), restr(restr)
 {
 }
 
@@ -1264,9 +1316,15 @@ namespace {
 
 
 /** Check if 'len' letters following cursor are all non-lowercase */
-static bool allNonLowercase(DocIterator const & cur, int len) {
-	pos_type end_pos = cur.pos() + len;
-	for (pos_type pos = cur.pos(); pos != end_pos; ++pos)
+static bool allNonLowercase(Cursor const & cur, int len)
+{
+	pos_type beg_pos = cur.selectionBegin().pos();
+	pos_type end_pos = cur.selectionBegin().pos() + len;
+	if (len > cur.lastpos() + 1 - beg_pos) {
+		LYXERR(Debug::FIND, "This should not happen, more debug needed");
+		len = cur.lastpos() + 1 - beg_pos;
+	}
+	for (pos_type pos = beg_pos; pos != end_pos; ++pos)
 		if (isLowerCase(cur.paragraph().getChar(pos)))
 			return false;
 	return true;
@@ -1274,14 +1332,16 @@ static bool allNonLowercase(DocIterator const & cur, int len) {
 
 
 /** Check if first letter is upper case and second one is lower case */
-static bool firstUppercase(DocIterator const & cur) {
+static bool firstUppercase(Cursor const & cur)
+{
 	char_type ch1, ch2;
-	if (cur.pos() >= cur.lastpos() - 1) {
+	pos_type pos = cur.selectionBegin().pos();
+	if (pos >= cur.lastpos() - 1) {
 		LYXERR(Debug::FIND, "No upper-case at cur: " << cur);
 		return false;
 	}
-	ch1 = cur.paragraph().getChar(cur.pos());
-	ch2 = cur.paragraph().getChar(cur.pos()+1);
+	ch1 = cur.paragraph().getChar(pos);
+	ch2 = cur.paragraph().getChar(pos + 1);
 	bool result = isUpperCase(ch1) && isLowerCase(ch2);
 	LYXERR(Debug::FIND, "firstUppercase(): "
 	       << "ch1=" << ch1 << "(" << char(ch1) << "), ch2=" 
@@ -1295,13 +1355,16 @@ static bool firstUppercase(DocIterator const & cur) {
  **
  ** \fixme What to do with possible further paragraphs in replace buffer ?
  **/
-static void changeFirstCase(Buffer & buffer, TextCase first_case, TextCase others_case) {
+static void changeFirstCase(Buffer & buffer, TextCase first_case, TextCase others_case)
+{
 	ParagraphList::iterator pit = buffer.paragraphs().begin();
+	LASSERT(pit->size() >= 1, /**/);
 	pos_type right = pos_type(1);
 	pit->changeCase(buffer.params(), pos_type(0), right, first_case);
-	right = pit->size() + 1;
-	pit->changeCase(buffer.params(), right, right, others_case);
+	right = pit->size();
+	pit->changeCase(buffer.params(), pos_type(1), right, others_case);
 }
+
 } // anon namespace
 
 ///
@@ -1321,7 +1384,7 @@ static void findAdvReplace(BufferView * bv, FindAndReplaceOptions const & opt, M
 	       << ", sel_len: " << sel_len << endl);
 	if (sel_len == 0)
 		return;
-	LASSERT(sel_len > 0, /**/);
+	LASSERT(sel_len > 0, return);
 
 	if (!matchAdv(sel_beg, sel_len))
 		return;
@@ -1333,8 +1396,9 @@ static void findAdvReplace(BufferView * bv, FindAndReplaceOptions const & opt, M
 	string lyx = oss.str();
 	Buffer repl_buffer("", false);
 	repl_buffer.setUnnamed(true);
-	LASSERT(repl_buffer.readString(lyx), /**/);
+	LASSERT(repl_buffer.readString(lyx), return);
 	if (opt.keep_case && sel_len >= 2) {
+		LYXERR(Debug::FIND, "keep_case true: cur.pos()=" << cur.pos() << ", sel_len=" << sel_len);
 		if (cur.inTexted()) {
 			if (firstUppercase(cur))
 				changeFirstCase(repl_buffer, text_uppercase, text_lowercase);
@@ -1343,7 +1407,7 @@ static void findAdvReplace(BufferView * bv, FindAndReplaceOptions const & opt, M
 		}
 	}
 	cap::cutSelection(cur, false, false);
-	if (!cur.inMathed()) {
+	if (cur.inTexted()) {
 		repl_buffer.changeLanguage(
 			repl_buffer.language(),
 			cur.getFont().language());
@@ -1354,7 +1418,7 @@ static void findAdvReplace(BufferView * bv, FindAndReplaceOptions const & opt, M
 					bv->buffer().errorList("Paste"));
 		LYXERR(Debug::FIND, "After pasteParagraphList() cur=" << cur << endl);
 		sel_len = repl_buffer.paragraphs().begin()->size();
-	} else {
+	} else if (cur.inMathed()) {
 		TexRow texrow;
 		odocstringstream ods;
 		otexstream os(ods, texrow);
@@ -1403,9 +1467,9 @@ bool findAdv(BufferView * bv, FindAndReplaceOptions const & opt)
 		findAdvReplace(bv, opt, matchAdv);
 		cur = bv->cursor();
 		if (opt.forward)
-				match_len = findForwardAdv(cur, matchAdv);
+			match_len = findForwardAdv(cur, matchAdv);
 		else
-				match_len = findBackwardsAdv(cur, matchAdv);
+			match_len = findBackwardsAdv(cur, matchAdv);
 	} catch (...) {
 		// This may only be raised by lyx::regex()
 		bv->message(_("Invalid regular expression!"));
@@ -1436,7 +1500,8 @@ ostringstream & operator<<(ostringstream & os, FindAndReplaceOptions const & opt
 	   << opt.ignoreformat << ' '
 	   << to_utf8(opt.repl_buf_name) << "\nEOSS\n"
 	   << opt.keep_case << ' '
-	   << int(opt.scope);
+	   << int(opt.scope) << ' '
+	   << int(opt.restr);
 
 	LYXERR(Debug::FIND, "built: " << os.str());
 
@@ -1452,10 +1517,10 @@ istringstream & operator>>(istringstream & is, FindAndReplaceOptions & opt)
 	getline(is, line);
 	while (line != "EOSS") {
 		if (! s.empty())
-				s = s + "\n";
+			s = s + "\n";
 		s = s + line;
 		if (is.eof())	// Tolerate malformed request
-				break;
+			break;
 		getline(is, line);
 	}
 	LYXERR(Debug::FIND, "file_buf_name: '" << s << "'");
@@ -1466,10 +1531,10 @@ istringstream & operator>>(istringstream & is, FindAndReplaceOptions & opt)
 	getline(is, line);
 	while (line != "EOSS") {
 		if (! s.empty())
-				s = s + "\n";
+			s = s + "\n";
 		s = s + line;
 		if (is.eof())	// Tolerate malformed request
-				break;
+			break;
 		getline(is, line);
 	}
 	LYXERR(Debug::FIND, "repl_buf_name: '" << s << "'");
@@ -1478,8 +1543,12 @@ istringstream & operator>>(istringstream & is, FindAndReplaceOptions & opt)
 	int i;
 	is >> i;
 	opt.scope = FindAndReplaceOptions::SearchScope(i);
+	is >> i;
+	opt.restr = FindAndReplaceOptions::SearchRestriction(i);
+
 	LYXERR(Debug::FIND, "parsed: " << opt.casesensitive << ' ' << opt.matchword << ' ' << opt.forward << ' '
-		   << opt.expandmacros << ' ' << opt.ignoreformat << ' ' << opt.keep_case);
+	       << opt.expandmacros << ' ' << opt.ignoreformat << ' ' << opt.keep_case << ' '
+	       << opt.scope << ' ' << opt.restr);
 	return is;
 }
 
