@@ -18,7 +18,7 @@
 #include "MetricsInfo.h"
 
 #include "Dimension.h"
-#include "Encoding.h"
+#include "BufferEncodings.h"
 #include "LaTeXFeatures.h"
 #include "TextPainter.h"
 
@@ -140,8 +140,8 @@ void InsetMathChar::write(WriteStream & os) const
 
 void InsetMathChar::validate(LaTeXFeatures & features) const
 {
-	if (char_ >= 0x80)
-		encodings.validate(char_, features, true);
+	if (!isASCII(char_))
+		BufferEncodings::validate(char_, features, true);
 }
 
 
