@@ -225,6 +225,8 @@ void InsetMathHull::setBuffer(Buffer & buffer)
 }
 
 
+// FIXME This should really be controlled by the TOC level, or
+// something of the sort.
 namespace {
 	const char * counters_to_save[] = {"section", "chapter"};
 	unsigned int const numcnts = sizeof(counters_to_save)/sizeof(char *);
@@ -282,7 +284,7 @@ void InsetMathHull::updateBuffer(ParIterator const & it, UpdateType utype)
 }
 
 
-void InsetMathHull::addToToc(DocIterator const & pit) const
+void InsetMathHull::addToToc(DocIterator const & pit, bool output_active) const
 {
 	if (!buffer_) {
 		//FIXME: buffer_ should be set at creation for this inset! Problem is
@@ -297,8 +299,8 @@ void InsetMathHull::addToToc(DocIterator const & pit) const
 		if (!numbered_[row])
 			continue;
 		if (label_[row])
-			label_[row]->addToToc(pit);
-		toc.push_back(TocItem(pit, 0, nicelabel(row)));
+			label_[row]->addToToc(pit, output_active);
+		toc.push_back(TocItem(pit, 0, nicelabel(row), output_active));
 	}
 }
 
@@ -342,6 +344,8 @@ char InsetMathHull::defaultColAlign(col_type col)
 {
 	if (type_ == hullEqnArray)
 		return "rcl"[col];
+	if (type_ == hullMultline)
+		return 'c';
 	if (type_ == hullGather)
 		return 'c';
 	if (type_ >= hullAlign)
@@ -350,6 +354,18 @@ char InsetMathHull::defaultColAlign(col_type col)
 }
 
 
+char InsetMathHull::displayColAlign(col_type col, row_type row) const
+{
+	if (type_ == hullMultline) {
+		if (row == 0)
+			return 'l';
+		if (row == nrows() - 1)
+			return 'r';
+	}
+	return InsetMathGrid::displayColAlign(col, row);
+}
+
+
 int InsetMathHull::defaultColSpace(col_type col)
 {
 	if (type_ == hullAlign || type_ == hullAlignAt)
@@ -396,7 +412,9 @@ ColorCode InsetMathHull::standardColor() const
 
 bool InsetMathHull::previewState(BufferView * bv) const
 {
-	if (!editing(bv) && RenderPreview::status() == LyXRC::PREVIEW_ON) {
+	if (!editing(bv) && RenderPreview::status() == LyXRC::PREVIEW_ON
+		&& type_ != hullRegexp)
+	{
 		graphics::PreviewImage const * pimage =
 			preview_->getPreviewImage(bv->buffer());
 		return pimage && pimage->image();
@@ -569,13 +587,13 @@ void InsetMathHull::addPreview(DocIterator const & inset_pos,
 
 
 void InsetMathHull::preparePreview(DocIterator const & pos,
-                                   bool forexport) const  
+                                   bool forexport) const
 {
 	// there is no need to do all the macro stuff if we're not
 	// actually going to generate the preview.
 	if (RenderPreview::status() != LyXRC::PREVIEW_ON && !forexport)
 		return;
-	
+
 	Buffer const * buffer = pos.buffer();
 
 	// collect macros at this position
@@ -647,7 +665,7 @@ bool InsetMathHull::notifyCursorLeaves(Cursor const & old, Cursor & cur)
 
 docstring InsetMathHull::label(row_type row) const
 {
-	LASSERT(row < nrows(), /**/);
+	LASSERT(row < nrows(), return docstring());
 	if (InsetLabel * il = label_[row])
 		return il->screenLabel();
 	return docstring();
@@ -734,7 +752,7 @@ void InsetMathHull::validate(LaTeXFeatures & features) const
 	if (features.runparams().isLaTeX()) {
 		if (ams())
 			features.require("amsmath");
-	
+
 		if (type_ == hullRegexp) {
 			features.require("color");
 			string frcol = lcolor.getLaTeXName(Color_regexpframe);
@@ -746,23 +764,22 @@ void InsetMathHull::validate(LaTeXFeatures & features) const
 			features.addPreambleSnippet(
 				string("\\newcommand{\\endregexp}{}"));
 		}
-	
+
 		// Validation is necessary only if not using AMS math.
 		// To be safe, we will always run mathedvalidate.
 		//if (features.amsstyle)
 		//  return;
-	
+
 		//features.binom      = true;
 	} else if (features.runparams().math_flavor == OutputParams::MathAsHTML) {
 		// it would be better to do this elsewhere, but we can't validate in
 		// InsetMathMatrix and we have no way, outside MathExtern, to know if
 		// we even have any matrices.
-				features.addPreambleSnippet("<style type=\"text/css\">\n"
+				features.addCSSSnippet(
 					"table.matrix{display: inline-block; vertical-align: middle; text-align:center;}\n"
 					"table.matrix td{padding: 0.25px;}\n"
 					"td.ldelim{width: 0.5ex; border: thin solid black; border-right: none;}\n"
-					"td.rdelim{width: 0.5ex; border: thin solid black; border-left: none;}\n"
-					"</style>");
+					"td.rdelim{width: 0.5ex; border: thin solid black; border-left: none;}");
 	}
 	InsetMathGrid::validate(features);
 }
@@ -1005,7 +1022,7 @@ void InsetMathHull::glueall(HullType type)
 
 void InsetMathHull::splitTo2Cols()
 {
-	LASSERT(ncols() == 1, /**/);
+	LASSERT(ncols() == 1, return);
 	InsetMathGrid::addCol(1);
 	for (row_type row = 0; row < nrows(); ++row) {
 		idx_type const i = 2 * row;
@@ -1018,13 +1035,13 @@ void InsetMathHull::splitTo2Cols()
 
 void InsetMathHull::splitTo3Cols()
 {
-	LASSERT(ncols() < 3, /**/);
+	LASSERT(ncols() < 3, return);
 	if (ncols() < 2)
 		splitTo2Cols();
 	InsetMathGrid::addCol(2);
 	for (row_type row = 0; row < nrows(); ++row) {
 		idx_type const i = 3 * row + 1;
-		if (cell(i).size()) {
+		if (!cell(i).empty()) {
 			cell(i + 1) = MathData(buffer_, cell(i).begin() + 1, cell(i).end());
 			cell(i).erase(1, cell(i).size());
 		}
@@ -1248,9 +1265,9 @@ void InsetMathHull::infoize(odocstream & os) const
 
 void InsetMathHull::check() const
 {
-	LASSERT(numbered_.size() == nrows(), /**/);
-	LASSERT(numbers_.size() == nrows(), /**/);
-	LASSERT(label_.size() == nrows(), /**/);
+	LATTEST(numbered_.size() == nrows());
+	LATTEST(numbers_.size() == nrows());
+	LATTEST(label_.size() == nrows());
 }
 
 
@@ -1294,7 +1311,7 @@ void InsetMathHull::doExtern(Cursor & cur, FuncRequest & func)
 		MathData ar;
 		if (cur.inMathed() && cur.selection()) {
 			asArray(grabAndEraseSelection(cur), ar);
-		} else if (pos == cur.cell().size()) {
+		} else if (!pos == cur.cell().empty()) {
 			ar = cur.cell();
 			lyxerr << "use whole cell: " << ar << endl;
 		} else {
@@ -1353,7 +1370,7 @@ void InsetMathHull::doDispatch(Cursor & cur, FuncRequest & cmd)
 		cur.undispatched();
 		break;
 
-	case LFUN_BREAK_PARAGRAPH:
+	case LFUN_PARAGRAPH_BREAK:
 		// just swallow this
 		break;
 
@@ -1362,7 +1379,7 @@ void InsetMathHull::doDispatch(Cursor & cur, FuncRequest & cmd)
 		if (type_ == hullSimple || type_ == hullEquation) {
 			cur.recordUndoInset();
 			bool const align =
-				cur.bv().buffer().params().use_amsmath == BufferParams::package_on;
+				cur.bv().buffer().params().use_package("amsmath") == BufferParams::package_on;
 			mutate(align ? hullAlign : hullEqnArray);
 			// mutate() may change labels and such.
 			cur.forceBufferUpdate();
@@ -1417,7 +1434,7 @@ void InsetMathHull::doDispatch(Cursor & cur, FuncRequest & cmd)
 		break;
 	}
 
-	case LFUN_LABEL_COPY_AS_REF: {
+	case LFUN_LABEL_COPY_AS_REFERENCE: {
 		row_type row;
 		if (cmd.argument().empty() && &cur.inset() == this)
 			// if there is no argument and we're inside math, we retrieve
@@ -1555,7 +1572,7 @@ bool InsetMathHull::getStatus(Cursor & cur, FuncRequest const & cmd,
 
 	// we never allow this in math, and we want to bind enter
 	// to another actions in command-alternatives
-	case LFUN_BREAK_PARAGRAPH:
+	case LFUN_PARAGRAPH_BREAK:
 		status.setEnabled(false);
 		return true;
 	case LFUN_MATH_MUTATE: {
@@ -1613,7 +1630,7 @@ bool InsetMathHull::getStatus(Cursor & cur, FuncRequest const & cmd,
 		status.setEnabled(type_ != hullSimple);
 		return true;
 
-	case LFUN_LABEL_COPY_AS_REF: {
+	case LFUN_LABEL_COPY_AS_REFERENCE: {
 		bool enabled = false;
 		row_type row;
 		if (cmd.argument().empty() && &cur.inset() == this) {
@@ -1625,7 +1642,7 @@ bool InsetMathHull::getStatus(Cursor & cur, FuncRequest const & cmd,
 			// if there is an argument, find the corresponding label, else
 			// check whether there is at least one label.
 			for (row_type row = 0; row != nrows(); ++row) {
-				if (numbered_[row] && label_[row] && 
+				if (numbered_[row] && label_[row] &&
 					(cmd.argument().empty() || label(row) == cmd.argument())) {
 						enabled = true;
 						break;
@@ -1880,7 +1897,8 @@ bool InsetMathHull::readQuiet(Lexer & lex)
 }
 
 
-int InsetMathHull::plaintext(odocstream & os, OutputParams const & op) const
+int InsetMathHull::plaintext(odocstringstream & os,
+        OutputParams const & op, size_t max_length) const
 {
 	// disables ASCII-art for export of equations. See #2275.
 	if (0 && display()) {
@@ -1908,7 +1926,7 @@ int InsetMathHull::plaintext(odocstream & os, OutputParams const & op) const
 				wi << (c == 0 ? "" : "\t") << cell(index(r, c));
 			// if it's for the TOC, we write just the first line
 			// and do not include the newline.
-			if (op.for_toc)
+			if (op.for_toc || op.for_tooltip || oss.str().size() >= max_length)
 				break;
 			wi << "\n";
 		}
@@ -2104,7 +2122,7 @@ void InsetMathHull::mathAsLatex(WriteStream & os) const
 
 docstring InsetMathHull::xhtml(XHTMLStream & xs, OutputParams const & op) const
 {
-	BufferParams::MathOutput const mathtype = 
+	BufferParams::MathOutput const mathtype =
 		buffer().masterBuffer()->params().html_math_output;
 
 	bool success = false;
@@ -2128,10 +2146,10 @@ docstring InsetMathHull::xhtml(XHTMLStream & xs, OutputParams const & op) const
 		} catch (MathExportException const &) {}
 		if (success) {
 			if (getType() == hullSimple)
-				xs << html::StartTag("math", 
+				xs << html::StartTag("math",
 							"xmlns=\"http://www.w3.org/1998/Math/MathML\"", true);
-			else 
-				xs << html::StartTag("math", 
+			else
+				xs << html::StartTag("math",
 				      "display=\"block\" xmlns=\"http://www.w3.org/1998/Math/MathML\"", true);
 			xs << XHTMLStream::ESCAPE_NONE
 				 << os.str()
@@ -2152,14 +2170,14 @@ docstring InsetMathHull::xhtml(XHTMLStream & xs, OutputParams const & op) const
 			   << html::EndTag(tag);
 		}
 	}
-	
+
 	// what we actually want is this:
 	// if (
-	//     ((mathtype == BufferParams::MathML || mathtype == BufferParams::HTML) 
+	//     ((mathtype == BufferParams::MathML || mathtype == BufferParams::HTML)
 	//       && !success)
 	//     || mathtype == BufferParams::Images
 	//    )
-	// but what follows is equivalent, since we'll enter only if either (a) we 
+	// but what follows is equivalent, since we'll enter only if either (a) we
 	// tried and failed with MathML or HTML or (b) didn't try yet at all but
 	// aren't doing LaTeX, in which case we are doing Images.
 	if (!success && mathtype != BufferParams::LaTeX) {
@@ -2195,7 +2213,7 @@ docstring InsetMathHull::xhtml(XHTMLStream & xs, OutputParams const & op) const
 			success = true;
 		}
 	}
-	
+
 	// so we'll pass this test if we've failed everything else, or
 	// if mathtype was LaTeX, since we won't have entered any of the
 	// earlier branches
@@ -2207,14 +2225,14 @@ docstring InsetMathHull::xhtml(XHTMLStream & xs, OutputParams const & op) const
 		ModeSpecifier specifier(wi, MATH_MODE);
 		mathAsLatex(wi);
 		docstring const latex = ls.str();
-		
+
 		// class='math' allows for use of jsMath
 		// http://www.math.union.edu/~dpvc/jsMath/
 		// FIXME XHTML
 		// probably should allow for some kind of customization here
 		string const tag = (getType() == hullSimple) ? "span" : "div";
 		xs << html::StartTag(tag, "class='math'")
-		   << latex 
+		   << latex
 		   << html::EndTag(tag)
 		   << html::CR();
 	}
@@ -2224,7 +2242,9 @@ docstring InsetMathHull::xhtml(XHTMLStream & xs, OutputParams const & op) const
 
 void InsetMathHull::toString(odocstream & os) const
 {
-	plaintext(os, OutputParams(0));
+	odocstringstream ods;
+	plaintext(ods, OutputParams(0));
+	os << ods.str();
 }
 
 
@@ -2238,9 +2258,9 @@ void InsetMathHull::forToc(docstring & os, size_t) const
 }
 
 
-docstring InsetMathHull::contextMenuName() const
+string InsetMathHull::contextMenuName() const
 {
-	return from_ascii("context-math");
+	return "context-math";
 }
 
 
