@@ -139,7 +139,7 @@ void InsetMathNest::cursorPos(BufferView const & bv,
 // to touch all (math)inset's draw() methods. Right now, we'll store
 // absolute value, and make them here relative, only to make them
 // absolute again when actually drawing the cursor. What a mess.
-	LASSERT(&sl.inset() == this, /**/);
+	LASSERT(&sl.inset() == this, return);
 	MathData const & ar = sl.cell();
 	CoordCache const & coord_cache = bv.coordCache();
 	if (!coord_cache.getArrays().has(&ar)) {
@@ -192,7 +192,7 @@ void InsetMathNest::updateBuffer(ParIterator const & it, UpdateType utype)
 
 bool InsetMathNest::idxNext(Cursor & cur) const
 {
-	LASSERT(&cur.inset() == this, /**/);
+	LASSERT(&cur.inset() == this, return false);
 	if (cur.idx() == cur.lastidx())
 		return false;
 	++cur.idx();
@@ -209,7 +209,7 @@ bool InsetMathNest::idxForward(Cursor & cur) const
 
 bool InsetMathNest::idxPrev(Cursor & cur) const
 {
-	LASSERT(&cur.inset() == this, /**/);
+	LASSERT(&cur.inset() == this, return false);
 	if (cur.idx() == 0)
 		return false;
 	--cur.idx();
@@ -226,7 +226,7 @@ bool InsetMathNest::idxBackward(Cursor & cur) const
 
 bool InsetMathNest::idxFirst(Cursor & cur) const
 {
-	LASSERT(&cur.inset() == this, /**/);
+	LASSERT(&cur.inset() == this, return false);
 	if (nargs() == 0)
 		return false;
 	cur.idx() = 0;
@@ -237,7 +237,7 @@ bool InsetMathNest::idxFirst(Cursor & cur) const
 
 bool InsetMathNest::idxLast(Cursor & cur) const
 {
-	LASSERT(&cur.inset() == this, /**/);
+	LASSERT(&cur.inset() == this, return false);
 	if (nargs() == 0)
 		return false;
 	cur.idx() = cur.lastidx();
@@ -576,7 +576,7 @@ void InsetMathNest::doDispatch(Cursor & cur, FuncRequest & cmd)
 		replaceSelection(cur);
 		docstring topaste;
 		if (cmd.argument().empty() && !theClipboard().isInternal())
-			topaste = theClipboard().getAsText();
+			topaste = theClipboard().getAsText(Clipboard::PlainTextType);
 		else {
 			size_t n = 0;
 			idocstringstream is(cmd.argument());
@@ -706,6 +706,10 @@ void InsetMathNest::doDispatch(Cursor & cur, FuncRequest & cmd)
 			act == LFUN_UP_SELECT;
 		cur.selHandle(select);
 
+		// handle autocorrect:
+		cur.autocorrect() = false;
+		cur.message(_("Autocorrect Off ('!' to enter)"));
+
 		// go up/down
 		bool up = act == LFUN_UP || act == LFUN_UP_SELECT;
 		bool successful = cur.upDownInMath(up);
@@ -732,7 +736,6 @@ void InsetMathNest::doDispatch(Cursor & cur, FuncRequest & cmd)
 		cur.setSelection(true);
 		cur.idx() = cur.lastidx();
 		cur.pos() = cur.lastpos();
-		cur.bv().cursor() = cur;
 		break;
 
 	case LFUN_PARAGRAPH_UP:
@@ -808,7 +811,7 @@ void InsetMathNest::doDispatch(Cursor & cur, FuncRequest & cmd)
 		if (cur.pos() == 0)
 			// May affect external cell:
 			cur.recordUndoInset();
-		else
+		else if (!cur.inMacroMode())
 			cur.recordUndoSelection();
 		// if the inset can not be removed from within, delete it
 		if (!cur.backspace()) {
@@ -874,7 +877,7 @@ void InsetMathNest::doDispatch(Cursor & cur, FuncRequest & cmd)
 		// if relevant. Think typing "\frac<space>".
 		if (cmd.argument()[0] == ' '
 		    && cur.inMacroMode() && cur.macroName() != "\\"
-		    && cur.macroModeClose()) {
+		    && cur.macroModeClose() && cur.pos() > 0) {
 			MathAtom const atom = cur.prevAtom();
 			if (atom->asNestInset() && atom->isActive()) {
 				cur.posBackward();
@@ -1001,7 +1004,6 @@ void InsetMathNest::doDispatch(Cursor & cur, FuncRequest & cmd)
 		cur.macroModeClose();
 		docstring const save_selection = grabAndEraseSelection(cur);
 		selClearOrDel(cur);
-		//cur.plainInsert(MathAtom(new InsetMathMBox(cur.bv())));
 		if (currentMode() <= Inset::TEXT_MODE)
 			cur.plainInsert(MathAtom(new InsetMathEnsureMath(buffer_)));
 		else
@@ -1142,10 +1144,34 @@ void InsetMathNest::doDispatch(Cursor & cur, FuncRequest & cmd)
 		break;
 	}
 
-	case LFUN_SPACE_INSERT:
+	case LFUN_SPACE_INSERT: {
 		cur.recordUndoSelection();
-		cur.insert(MathAtom(new InsetMathSpace));
+		string const name = cmd.getArg(0);
+		if (name == "normal")
+			cur.insert(MathAtom(new InsetMathSpace(" ", "")));
+		else if (name == "protected")
+			cur.insert(MathAtom(new InsetMathSpace("~", "")));
+		else if (name == "thin" || name == "med" || name == "thick")
+			cur.insert(MathAtom(new InsetMathSpace(name + "space", "")));
+		else if (name == "hfill*")
+			cur.insert(MathAtom(new InsetMathSpace("hspace*{\\fill}", "")));
+		else if (name == "quad" || name == "qquad" ||
+		         name == "enspace" || name == "enskip" ||
+		         name == "negthinspace" || name == "negmedspace" ||
+		         name == "negthickspace" || name == "hfill")
+			cur.insert(MathAtom(new InsetMathSpace(name, "")));
+		else if (name == "hspace" || name == "hspace*") {
+			string const len = cmd.getArg(1);
+			if (len.empty() || !isValidLength(len)) {
+				lyxerr << "LyX function 'space-insert " << name << "' "
+				          "needs a valid length argument." << endl;
+				break;
+			}
+			cur.insert(MathAtom(new InsetMathSpace(name, len)));
+		} else
+			cur.insert(MathAtom(new InsetMathSpace));
 		break;
+	}
 
 	case LFUN_MATH_SPACE:
 		cur.recordUndoSelection();
@@ -1425,9 +1451,13 @@ bool InsetMathNest::getStatus(Cursor & cur, FuncRequest const & cmd,
 		flag.setEnabled(false);
 		break;
 
+	case LFUN_CAPTION_INSERT:
+		flag.setEnabled(false);
+		break;
+	
 	case LFUN_SPACE_INSERT: {
 		docstring const & name = cmd.argument();
-		if (name == "protected" || name == "normal")
+		if (name == "visible")
 			flag.setEnabled(false);
 		break;
 	}
@@ -1436,6 +1466,13 @@ bool InsetMathNest::getStatus(Cursor & cur, FuncRequest const & cmd,
 		flag.setEnabled(!asHullInset());
 		break;
 
+	case LFUN_PASTE: {
+		docstring const & name = cmd.argument();
+		if (name == "html" || name == "latex")
+			flag.setEnabled(false);
+		break;
+	}
+
 	default:
 		ret = false;
 		break;
@@ -1612,7 +1649,7 @@ bool InsetMathNest::interpretChar(Cursor & cur, char_type const c)
 				cur.backspace();
 				cur.niceInsert(MathAtom(new InsetMathComment(buf)));
 			} else if (c == '#') {
-				LASSERT(cur.activeMacro(), /**/);
+				LASSERT(cur.activeMacro(), return false);
 				cur.activeMacro()->setName(name + docstring(1, c));
 			} else {
 				cur.backspace();
@@ -1649,6 +1686,13 @@ bool InsetMathNest::interpretChar(Cursor & cur, char_type const c)
 					new InsetMathBig(name.substr(1), delim)));
 				return true;
 			}
+		} else if (name == "\\smash" && c == '[') {
+			// We can't use cur.macroModeClose() because
+			// it would create an InsetMathPhantom
+			InsetMathUnknown * p = cur.activeMacro();
+			p->finalize();
+			interpretChar(cur, c);
+			return true;
 		}
 
 		// leave macro mode and try again if necessary
@@ -1689,6 +1733,7 @@ bool InsetMathNest::interpretChar(Cursor & cur, char_type const c)
 		//lyxerr << "starting with macro" << endl;
 		bool reduced = cap::reduceSelectionToOneCell(cur);
 		if (reduced || !cur.selection()) {
+			cur.recordUndoInset();
 			docstring const safe = cap::grabAndEraseSelection(cur);
 			if (!cur.inRegexped())
 				cur.insert(MathAtom(new InsetMathUnknown(from_ascii("\\"), safe, false)));
@@ -2082,6 +2127,7 @@ MathCompletionList::MathCompletionList(Cursor const & cur)
 	globals.push_back(from_ascii("\\root"));
 	globals.push_back(from_ascii("\\tabular"));
 	globals.push_back(from_ascii("\\stackrel"));
+	globals.push_back(from_ascii("\\stackrelthree"));
 	globals.push_back(from_ascii("\\binom"));
 	globals.push_back(from_ascii("\\choose"));
 	globals.push_back(from_ascii("\\brace"));
@@ -2111,6 +2157,14 @@ MathCompletionList::MathCompletionList(Cursor const & cur)
 	globals.push_back(from_ascii("\\hphantom"));
 	globals.push_back(from_ascii("\\phantom"));
 	globals.push_back(from_ascii("\\vphantom"));
+	globals.push_back(from_ascii("\\cancel"));
+	globals.push_back(from_ascii("\\bcancel"));
+	globals.push_back(from_ascii("\\xcancel"));
+	globals.push_back(from_ascii("\\cancelto"));
+	globals.push_back(from_ascii("\\smash"));
+	globals.push_back(from_ascii("\\mathclap"));
+	globals.push_back(from_ascii("\\mathllap"));
+	globals.push_back(from_ascii("\\mathrlap"));
 	MathWordList const & words = mathedWordList();
 	MathWordList::const_iterator it2;
 	//lyxerr << "Globals completion commands: ";
