@@ -25,15 +25,17 @@
 #include "ParagraphParameters.h"
 #include "TextClass.h"
 #include "TexRow.h"
-#include "VSpace.h"
 
 #include "insets/InsetBibitem.h"
 #include "insets/InsetArgument.h"
 
 #include "support/lassert.h"
+#include "support/convert.h"
 #include "support/debug.h"
 #include "support/lstrings.h"
+#include "support/textutils.h"
 
+#include <algorithm>
 #include <boost/next_prior.hpp>
 #include <list>
 
@@ -46,9 +48,9 @@ namespace lyx {
 namespace {
 
 enum OpenEncoding {
-		none,
-		inputenc,
-		CJK
+	none,
+	inputenc,
+	CJK
 };
 
 static int open_encoding_ = none;
@@ -66,7 +68,7 @@ string const getPolyglossiaEnvName(Language const * lang)
 }
 
 
-struct TeXEnvironementData
+struct TeXEnvironmentData
 {
 	bool cjk_nested;
 	Layout const * style;
@@ -76,13 +78,13 @@ struct TeXEnvironementData
 };
 
 
-static TeXEnvironementData prepareEnvironement(Buffer const & buf,
+static TeXEnvironmentData prepareEnvironment(Buffer const & buf,
 					Text const & text,
 					ParagraphList::const_iterator pit,
 					otexstream & os,
 					OutputParams const & runparams)
 {
-	TeXEnvironementData data;
+	TeXEnvironmentData data;
 
 	BufferParams const & bparams = buf.params();
 
@@ -109,19 +111,18 @@ static TeXEnvironementData prepareEnvironement(Buffer const & buf,
 		? (use_prev_env_language ? prev_env_language_
 					 : priorpit->getParLanguage(bparams))
 		: doc_language;
-	string par_lang = data.par_language->babel();
-	string prev_par_lang = prev_par_language->babel();
-	string doc_lang = doc_language->babel();
-	string lang_begin_command = lyxrc.language_command_begin;
-	string lang_end_command = lyxrc.language_command_end;
 
-	if (runparams.use_polyglossia) {
-		par_lang = getPolyglossiaEnvName(data.par_language);
-		prev_par_lang = getPolyglossiaEnvName(prev_par_language);
-		doc_lang = getPolyglossiaEnvName(doc_language);
-		lang_begin_command = "\\begin{$$lang}";
-		lang_end_command = "\\end{$$lang}";
-	}
+	bool const use_polyglossia = runparams.use_polyglossia;
+	string const par_lang = use_polyglossia ?
+		getPolyglossiaEnvName(data.par_language) : data.par_language->babel();
+	string const prev_par_lang = use_polyglossia ?
+		getPolyglossiaEnvName(prev_par_language) : prev_par_language->babel();
+	string const doc_lang = use_polyglossia ?
+		getPolyglossiaEnvName(doc_language) : doc_language->babel();
+	string const lang_begin_command = use_polyglossia ?
+		"\\begin{$$lang}" : lyxrc.language_command_begin;
+	string const lang_end_command = use_polyglossia ?
+		"\\end{$$lang}" : lyxrc.language_command_end;
 
 	if (par_lang != prev_par_lang) {
 		if (!lang_end_command.empty() &&
@@ -142,7 +143,7 @@ static TeXEnvironementData prepareEnvironement(Buffer const & buf,
 				lang_begin_command,
 				"$$lang",
 				par_lang));
-			if (runparams.use_polyglossia
+			if (use_polyglossia
 			    && !data.par_language->polyglossiaOpts().empty())
 					os << "["
 					   << from_ascii(data.par_language->polyglossiaOpts())
@@ -162,8 +163,11 @@ static TeXEnvironementData prepareEnvironement(Buffer const & buf,
 
 	if (style.isEnvironment()) {
 		os << "\\begin{" << from_ascii(style.latexname()) << '}';
-		if (style.optargs != 0 || style.reqargs != 0)
-			latexArgInsets(*pit, os, runparams, style.reqargs, style.optargs);
+		if (!style.latexargs().empty()) {
+			OutputParams rp = runparams;
+			rp.local_font = &pit->getFirstFontSettings(bparams);
+			latexArgInsets(paragraphs, pit, os, rp, style.latexargs());
+		}
 		if (style.latextype == LATEX_LIST_ENVIRONMENT) {
 			os << '{'
 			   << pit->params().labelWidthString()
@@ -194,8 +198,8 @@ static TeXEnvironementData prepareEnvironement(Buffer const & buf,
 }
 
 
-static void finishEnvironement(otexstream & os, OutputParams const & runparams,
-			       TeXEnvironementData const & data)
+static void finishEnvironment(otexstream & os, OutputParams const & runparams,
+			       TeXEnvironmentData const & data)
 {
 	if (open_encoding_ == CJK && data.cjk_nested) {
 		// We need to close the encoding even if it does not change
@@ -270,7 +274,7 @@ void TeXEnvironment(Buffer const & buf, Text const & text,
 		// Or     par->params().leftIndent() != current_left_indent)
 
 		if (par->layout().isParagraph()) {
-			// FIXME (Lgb): How to handle this? 
+			// FIXME (Lgb): How to handle this?
 			//&& !suffixIs(os, "\n\n")
 
 			// (ARRae) There should be at least one '\n' already but we need there to
@@ -298,70 +302,183 @@ void TeXEnvironment(Buffer const & buf, Text const & text,
 		}
 
 		// This is a new environment.
-		TeXEnvironementData const data =
-			prepareEnvironement(buf, text, par, os, runparams);
+		TeXEnvironmentData const data =
+			prepareEnvironment(buf, text, par, os, runparams);
 		// Recursive call to TeXEnvironment!
 		TeXEnvironment(buf, text, runparams, pit, os);
-		finishEnvironement(os, runparams, data);
+		finishEnvironment(os, runparams, data);
 	}
 
 	if (pit != runparams.par_end)
 		LYXERR(Debug::LATEX, "TeXEnvironment for paragraph " << par_begin << " done.");
 }
 
+
+void getArgInsets(otexstream & os, OutputParams const & runparams, Layout::LaTeXArgMap const & latexargs,
+		  map<int, lyx::InsetArgument const *> ilist, vector<string> required, string const & prefix)
+{
+	unsigned int const argnr = latexargs.size();
+	if (argnr == 0)
+		return;
+
+	for (unsigned int i = 1; i <= argnr; ++i) {
+		map<int, InsetArgument const *>::const_iterator lit = ilist.find(i);
+		bool inserted = false;
+		if (lit != ilist.end()) {
+			InsetArgument const * ins = (*lit).second;
+			if (ins) {
+				Layout::LaTeXArgMap::const_iterator const lait =
+						latexargs.find(ins->name());
+				if (lait != latexargs.end()) {
+					Layout::latexarg arg = (*lait).second;
+					docstring ldelim = arg.mandatory ?
+							from_ascii("{") : from_ascii("[");
+					docstring rdelim = arg.mandatory ?
+							from_ascii("}") : from_ascii("]");
+					if (!arg.ldelim.empty())
+						ldelim = arg.ldelim;
+					if (!arg.rdelim.empty())
+						rdelim = arg.rdelim;
+					ins->latexArgument(os, runparams, ldelim, rdelim, arg.presetarg);
+					inserted = true;
+				}
+			}
+		}
+		if (!inserted) {
+			Layout::LaTeXArgMap::const_iterator lait = latexargs.begin();
+			Layout::LaTeXArgMap::const_iterator const laend = latexargs.end();
+			for (; lait != laend; ++lait) {
+				string const name = prefix + convert<string>(i);
+				if ((*lait).first == name) {
+					Layout::latexarg arg = (*lait).second;
+					docstring preset = arg.presetarg;
+					if (!arg.defaultarg.empty()) {
+						if (!preset.empty())
+							preset += ",";
+						preset += arg.defaultarg;
+					}
+					if (arg.mandatory) {
+						docstring ldelim = arg.ldelim.empty() ?
+								from_ascii("{") : arg.ldelim;
+						docstring rdelim = arg.rdelim.empty() ?
+								from_ascii("}") : arg.rdelim;
+						os << ldelim << preset << rdelim;
+					} else if (!preset.empty()) {
+						docstring ldelim = arg.ldelim.empty() ?
+								from_ascii("[") : arg.ldelim;
+						docstring rdelim = arg.rdelim.empty() ?
+								from_ascii("]") : arg.rdelim;
+						os << ldelim << preset << rdelim;
+					} else if (find(required.begin(), required.end(),
+						   (*lait).first) != required.end()) {
+						docstring ldelim = arg.ldelim.empty() ?
+								from_ascii("[") : arg.ldelim;
+						docstring rdelim = arg.rdelim.empty() ?
+								from_ascii("]") : arg.rdelim;
+						os << ldelim << rdelim;
+					} else
+						break;
+				}
+			}
+		}
+	}
+}
+
+
 } // namespace anon
 
 
 void latexArgInsets(Paragraph const & par, otexstream & os,
-	OutputParams const & runparams, unsigned int reqargs,
-	unsigned int optargs)
+	OutputParams const & runparams, Layout::LaTeXArgMap const & latexargs, string const & prefix)
 {
-	unsigned int totalargs = reqargs + optargs;
-	list<InsetArgument const *> ilist;
+	map<int, InsetArgument const *> ilist;
+	vector<string> required;
 
 	InsetList::const_iterator it = par.insetList().begin();
 	InsetList::const_iterator end = par.insetList().end();
 	for (; it != end; ++it) {
 		if (it->inset->lyxCode() == ARG_CODE) {
-			if (ilist.size() >= totalargs) {
-				LYXERR0("WARNING: Found extra argument inset.");
-				continue;
-			}
 			InsetArgument const * ins =
 				static_cast<InsetArgument const *>(it->inset);
-			ilist.push_back(ins);
+			if (ins->name().empty())
+				LYXERR0("Error: Unnamed argument inset!");
+			else {
+				string const name = prefix.empty() ? ins->name() : split(ins->name(), ':');
+				unsigned int const nr = convert<unsigned int>(name);
+				ilist[nr] = ins;
+				Layout::LaTeXArgMap::const_iterator const lit =
+						latexargs.find(ins->name());
+				if (lit != latexargs.end()) {
+					Layout::latexarg const & arg = (*lit).second;
+					if (!arg.requires.empty()) {
+						vector<string> req = getVectorFromString(arg.requires);
+						required.insert(required.end(), req.begin(), req.end());
+					}
+				}
+			}
 		}
 	}
+	getArgInsets(os, runparams, latexargs, ilist, required, prefix);
+}
 
-	if (!reqargs && ilist.empty())
-		return;
 
-	bool const have_optional_args = ilist.size() > reqargs;
-	if (have_optional_args) {
-		unsigned int todo = ilist.size() - reqargs;
-		for (unsigned int i = 0; i < todo; ++i) {
-			InsetArgument const * ins = ilist.front();
-			ilist.pop_front();
-			ins->latexArgument(os, runparams, true);
-		}
+void latexArgInsets(ParagraphList const & pars, ParagraphList::const_iterator pit,
+	otexstream & os, OutputParams const & runparams, Layout::LaTeXArgMap const & latexargs,
+	string const & prefix)
+{
+	map<int, InsetArgument const *> ilist;
+	vector<string> required;
+
+	depth_type const current_depth = pit->params().depth();
+	Layout const current_layout = pit->layout();
+
+	// get the first paragraph in sequence with this layout and depth
+	pit_type offset = 0;
+	while (true) {
+		if (boost::prior(pit, offset) == pars.begin())
+			break;
+		ParagraphList::const_iterator priorpit = boost::prior(pit, offset + 1);
+		if (priorpit->layout() == current_layout
+		    && priorpit->params().depth() == current_depth)
+			++offset;
+		else
+			break;
 	}
 
-	// we should now have no more insets than there are required
-	// arguments.
-	LASSERT(ilist.size() <= reqargs, /* */);
-	if (!reqargs)
-		return;
+	ParagraphList::const_iterator spit = boost::prior(pit, offset);
 
-	for (unsigned int i = 0; i < reqargs; ++i) {
-		if (ilist.empty())
-			// a required argument wasn't given, so we output {}
-			os << "{}";
-		else {
-			InsetArgument const * ins = ilist.front();
-			ilist.pop_front();
-			ins->latexArgument(os, runparams, false);
+	for (; spit != pars.end(); ++spit) {
+		if (spit->layout() != current_layout || spit->params().depth() < current_depth)
+			break;
+		if (spit->params().depth() > current_depth)
+			continue;
+		InsetList::const_iterator it = spit->insetList().begin();
+		InsetList::const_iterator end = spit->insetList().end();
+		for (; it != end; ++it) {
+			if (it->inset->lyxCode() == ARG_CODE) {
+				InsetArgument const * ins =
+					static_cast<InsetArgument const *>(it->inset);
+				if (ins->name().empty())
+					LYXERR0("Error: Unnamed argument inset!");
+				else {
+					string const name = prefix.empty() ? ins->name() : split(ins->name(), ':');
+					unsigned int const nr = convert<unsigned int>(name);
+					if (ilist.find(nr) == ilist.end())
+						ilist[nr] = ins;
+					Layout::LaTeXArgMap::const_iterator const lit =
+							latexargs.find(ins->name());
+					if (lit != latexargs.end()) {
+						Layout::latexarg const & arg = (*lit).second;
+						if (!arg.requires.empty()) {
+							vector<string> req = getVectorFromString(arg.requires);
+							required.insert(required.end(), req.begin(), req.end());
+						}
+					}
+				}
+			}
 		}
 	}
+	getArgInsets(os, runparams, latexargs, ilist, required, prefix);
 }
 
 namespace {
@@ -374,15 +491,18 @@ void parStartCommand(Paragraph const & par, otexstream & os,
 	case LATEX_COMMAND:
 		os << '\\' << from_ascii(style.latexname());
 
-		// Separate handling of optional argument inset.
-		if (style.optargs != 0 || style.reqargs != 0)
-			latexArgInsets(par, os, runparams, style.reqargs, style.optargs);
-		else
-			os << from_ascii(style.latexparam());
+		// Command arguments
+		if (!style.latexargs().empty())
+			latexArgInsets(par, os, runparams, style.latexargs());
+		os << from_ascii(style.latexparam());
 		break;
 	case LATEX_ITEM_ENVIRONMENT:
 	case LATEX_LIST_ENVIRONMENT:
-		os << "\\item ";
+		os << "\\" + style.itemcommand();
+		// Item arguments
+		if (!style.itemargs().empty())
+			latexArgInsets(par, os, runparams, style.itemargs(), "item:");
+		os << " ";
 		break;
 	case LATEX_BIB_ENVIRONMENT:
 		// ignore this, the inset will write itself
@@ -403,7 +523,8 @@ void TeXOnePar(Buffer const & buf,
 	       string const & everypar,
 	       int start_pos, int end_pos)
 {
-	BufferParams const & bparams = buf.params();
+	BufferParams const & bparams = runparams_in.is_child
+		? buf.masterParams() : buf.params();
 	ParagraphList const & paragraphs = text.paragraphs();
 	Paragraph const & par = paragraphs.at(pit);
 	// FIXME This check should not really be needed.
@@ -434,7 +555,7 @@ void TeXOnePar(Buffer const & buf,
 		open_encoding_ = none;
 	}
 
-	if (text.inset().getLayout().isPassThru()) {
+	if (text.inset().isPassThru()) {
 		Font const outerfont = text.outerFont(pit);
 
 		// No newline before first paragraph in this lyxtext
@@ -466,7 +587,7 @@ void TeXOnePar(Buffer const & buf,
 			os << '\n';
 		if (!style.parbreak_is_newline) {
 			os << '\n';
-		} else if (nextpar) {
+		} else if (nextpar && !style.isEnvironment()) {
 			Layout const nextstyle = text.inset().forcePlainLayout()
 				? bparams.documentClass().plainLayout()
 				: nextpar->layout();
@@ -506,21 +627,32 @@ void TeXOnePar(Buffer const & buf,
 					 : priorpar->getParLanguage(bparams))
 		: outer_language;
 
-	string par_lang = par_language->babel();
-	string prev_lang = prev_language->babel();
-	string doc_lang = doc_language->babel();
-	string outer_lang = outer_language->babel();
-	string lang_begin_command = lyxrc.language_command_begin;
-	string lang_end_command = lyxrc.language_command_end;
 
-	if (runparams.use_polyglossia) {
-		par_lang = getPolyglossiaEnvName(par_language);
-		prev_lang = getPolyglossiaEnvName(prev_language);
-		doc_lang = getPolyglossiaEnvName(doc_language);
-		outer_lang = getPolyglossiaEnvName(outer_language);
-		lang_begin_command = "\\begin{$$lang}";
-		lang_end_command = "\\end{$$lang}";
+	bool const use_polyglossia = runparams.use_polyglossia;
+	string const par_lang = use_polyglossia ?
+		getPolyglossiaEnvName(par_language): par_language->babel();
+	string const prev_lang = use_polyglossia ?
+		getPolyglossiaEnvName(prev_language) : prev_language->babel();
+	string const doc_lang = use_polyglossia ?
+		getPolyglossiaEnvName(doc_language) : doc_language->babel();
+	string const outer_lang = use_polyglossia ?
+		getPolyglossiaEnvName(outer_language) : outer_language->babel();
+	string lang_begin_command = use_polyglossia ?
+		"\\begin{$$lang}" : lyxrc.language_command_begin;
+	string lang_end_command = use_polyglossia ?
+		"\\end{$$lang}" : lyxrc.language_command_end;
+	// the '%' is necessary to prevent unwanted whitespace
+	string lang_command_termination = "%\n";
+
+	// In some insets (such as Arguments), we cannot use \selectlanguage
+	bool const localswitch = !use_polyglossia
+		&& text.inset().getLayout().forcelocalfontswitch();
+	if (localswitch) {
+		lang_begin_command = lyxrc.language_command_local;
+		lang_end_command = "}";
+		lang_command_termination.clear();
 	}
+
 	if (par_lang != prev_lang
 		// check if we already put language command in TeXEnvironment()
 		&& !(style.isEnvironment()
@@ -535,8 +667,7 @@ void TeXOnePar(Buffer const & buf,
 			os << from_ascii(subst(lang_end_command,
 				"$$lang",
 				prev_lang))
-			   // the '%' is necessary to prevent unwanted whitespace
-			   << "%\n";
+			   << lang_command_termination;
 		}
 
 		// We need to open a new language if we couldn't close the previous
@@ -551,7 +682,7 @@ void TeXOnePar(Buffer const & buf,
 			// language paragraph should appear within an \L or \R (in addition
 			// to, outside of, the normal language switch commands).
 			// This behavior is not correct for ArabTeX, though.
-			if (!runparams.use_polyglossia
+			if (!use_polyglossia
 			    // not for ArabTeX
 				&& par_language->lang() != "arabic_arabtex"
 				&& outer_language->lang() != "arabic_arabtex"
@@ -589,13 +720,12 @@ void TeXOnePar(Buffer const & buf,
 					lang_begin_command,
 					"$$lang",
 					par_lang));
-				if (runparams.use_polyglossia
+				if (use_polyglossia
 				    && !par_language->polyglossiaOpts().empty())
 						os << "["
 						  << from_ascii(par_language->polyglossiaOpts())
 						  << "]";
-				   // the '%' is necessary to prevent unwanted whitespace
-				os << "%\n";
+				os << lang_command_termination;
 			}
 		}
 	}
@@ -617,7 +747,7 @@ void TeXOnePar(Buffer const & buf,
 				par.getFontSettings(bparams, i).language()->encoding();
 			if (encoding->package() != Encoding::CJK
 				&& runparams.encoding->package() == Encoding::inputenc
-				&& c < 0x80)
+				&& isASCII(c))
 				continue;
 			if (par.isInset(i))
 				break;
@@ -651,8 +781,7 @@ void TeXOnePar(Buffer const & buf,
 						lang_begin_command,
 						"$$lang",
 						par_lang))
-					// the '%' is necessary to prevent unwanted whitespace
-					<< "%\n";
+					<< lang_command_termination;
 				}
 				runparams.encoding = encoding;
 			}
@@ -682,7 +811,6 @@ void TeXOnePar(Buffer const & buf,
 	}
 
 	parStartCommand(par, os, runparams, style);
-
 	Font const outerfont = text.outerFont(pit);
 
 	// FIXME UNICODE
@@ -712,6 +840,8 @@ void TeXOnePar(Buffer const & buf,
 		os << "\\" << from_ascii(font.latexSize()) << " \\par}";
 	} else if (is_command) {
 		os << '}';
+		if (!style.postcommandargs().empty())
+			latexArgInsets(par, os, runparams, style.postcommandargs(), "post:");
 		if (runparams.encoding != prev_encoding) {
 			runparams.encoding = prev_encoding;
 			if (!runparams.isFullUnicode())
@@ -756,11 +886,11 @@ void TeXOnePar(Buffer const & buf,
 	// Closing the language is needed for the last paragraph; it is also
 	// needed if we're within an \L or \R that we may have opened above (not
 	// necessarily in this paragraph) and are about to close.
-	bool closing_rtl_ltr_environment = !runparams.use_polyglossia
+	bool closing_rtl_ltr_environment = !use_polyglossia
 		// not for ArabTeX
 		&& (par_language->lang() != "arabic_arabtex"
 		    && outer_language->lang() != "arabic_arabtex")
-		     // have we opened and \L or \R environment?
+		// have we opened an \L or \R environment?
 		&& runparams.local_font != 0
 		&& runparams.local_font->isRightToLeft() != par_language->rightToLeft()
 		// are we about to close the language?
@@ -768,7 +898,9 @@ void TeXOnePar(Buffer const & buf,
 		   || (runparams.isLastPar && par_language->babel() != outer_language->babel()));
 
 	if (closing_rtl_ltr_environment
-		|| (runparams.isLastPar && par_language->babel() != outer_language->babel())) {
+	    || (runparams.isLastPar
+	        && ((!use_polyglossia && par_language->babel() != outer_language->babel())
+		    || (use_polyglossia && par_language->polyglossia() != outer_language->polyglossia())))) {
 		// Since \selectlanguage write the language to the aux file,
 		// we need to reset the language at the end of footnote or
 		// float.
@@ -785,24 +917,24 @@ void TeXOnePar(Buffer const & buf,
 					(runparams.isLastPar && runparams.master_language)
 						? runparams.master_language
 						: outer_language;
-				string const current_lang = runparams.use_polyglossia ?
-					getPolyglossiaEnvName(current_language)
+				string const current_lang = use_polyglossia
+					? getPolyglossiaEnvName(current_language)
 					: current_language->babel();
 				if (!current_lang.empty()) {
 					os << from_ascii(subst(
 						lang_begin_command,
 						"$$lang",
 						current_lang));
-					pending_newline = true;
-					unskip_newline = true;
+					pending_newline = !localswitch;
+					unskip_newline = !localswitch;
 				}
 			} else if (!par_lang.empty()) {
 				os << from_ascii(subst(
 					lang_end_command,
 					"$$lang",
 					par_lang));
-				pending_newline = true;
-				unskip_newline = true;
+				pending_newline = !localswitch;
+				unskip_newline = !localswitch;
 			}
 		}
 	}
@@ -908,6 +1040,9 @@ void latexParagraphs(Buffer const & buf,
 		     OutputParams const & runparams,
 		     string const & everypar)
 {
+	LASSERT(runparams.par_begin <= runparams.par_end,
+		{ os << "% LaTeX Output Error\n"; return; } );
+
 	BufferParams const & bparams = buf.params();
 
 	bool const maintext = text.isMainText();
@@ -924,8 +1059,8 @@ void latexParagraphs(Buffer const & buf,
 	}
 	// if "auto begin" is switched off, explicitly switch the
 	// language on at start
-	string const mainlang = runparams.use_polyglossia ?
-		getPolyglossiaEnvName(bparams.language)
+	string const mainlang = runparams.use_polyglossia
+		? getPolyglossiaEnvName(bparams.language)
 		: bparams.language->babel();
 	string const lang_begin_command = runparams.use_polyglossia ?
 		"\\begin{$$lang}" : lyxrc.language_command_begin;
@@ -945,7 +1080,6 @@ void latexParagraphs(Buffer const & buf,
 	}
 
 	ParagraphList const & paragraphs = text.paragraphs();
-	LASSERT(runparams.par_begin <= runparams.par_end, /**/);
 
 	if (runparams.par_begin == runparams.par_end) {
 		// The full doc will be exported but it is easier to just rely on
@@ -1004,11 +1138,11 @@ void latexParagraphs(Buffer const & buf,
 			continue;
 		}
 		
-		TeXEnvironementData const data =
-			prepareEnvironement(buf, text, par, os, runparams);
+		TeXEnvironmentData const data =
+			prepareEnvironment(buf, text, par, os, runparams);
 		// pit can be changed in TeXEnvironment.
 		TeXEnvironment(buf, text, runparams, pit, os);
-		finishEnvironement(os, runparams, data);
+		finishEnvironment(os, runparams, data);
 	}
 
 	if (pit == runparams.par_end) {
@@ -1063,8 +1197,12 @@ pair<bool, int> switchEncoding(odocstream & os, BufferParams const & bparams,
 {
 	Encoding const & oldEnc = *runparams.encoding;
 	bool moving_arg = runparams.moving_arg;
-	if (!force && ((bparams.inputenc != "auto" && bparams.inputenc != "default")
-		|| moving_arg))
+	// If we switch from/to CJK, we need to switch anyway, despite custom inputenc
+	bool const from_to_cjk = 
+		(oldEnc.package() == Encoding::CJK && newEnc.package() != Encoding::CJK)
+		|| (oldEnc.package() != Encoding::CJK && newEnc.package() == Encoding::CJK);
+	if (!force && !from_to_cjk
+	    && ((bparams.inputenc != "auto" && bparams.inputenc != "default") || moving_arg))
 		return make_pair(false, 0);
 
 	// Do nothing if the encoding is unchanged.
