@@ -133,8 +133,7 @@ namespace {
 /*
  * This is a parser that (mostly) mimics the behavior of a posix shell as
  * regards quoting, but its output is tailored for being processed by QProcess.
- * Note that shell metacharacters are not parsed and only output redirection
- * is taken into account.
+ * Note that shell metacharacters are not parsed.
  *
  * The escape character is the backslash.
  * A backslash that is not quoted preserves the literal value of the following
@@ -167,13 +166,14 @@ namespace {
  *    "\a"   ->  "\a"
  *    "a\"b" ->  "a"""b"
  */
-string const parsecmd(string const & incmd, string & outfile, string & errfile)
+string const parsecmd(string const & incmd, string & infile, string & outfile,
+                     string & errfile)
 {
 	bool in_single_quote = false;
 	bool in_double_quote = false;
 	bool escaped = false;
 	string const python_call = "python -tt";
-	vector<string> outcmd(3);
+	vector<string> outcmd(4);
 	size_t start = 0;
 
 	if (prefixIs(incmd, python_call)) {
@@ -218,6 +218,8 @@ string const parsecmd(string const & incmd, string & outfile, string & errfile)
 					outcmd[o] = rtrim(outcmd[o], "1");
 				o = 1;
 			}
+		} else if (c == '<' && !(in_double_quote || escaped)) {
+			o = 3;
 		} else {
 			if (escaped && in_double_quote)
 				outcmd[o] += '\\';
@@ -225,6 +227,7 @@ string const parsecmd(string const & incmd, string & outfile, string & errfile)
 			escaped = false;
 		}
 	}
+	infile  = trim(outcmd[3], " \"");
 	outfile = trim(outcmd[1], " \"");
 	errfile = trim(outcmd[2], " \"");
 	return trim(outcmd[0]);
@@ -237,28 +240,44 @@ string const parsecmd(string const & incmd, string & outfile, string & errfile)
 int Systemcall::startscript(Starttype how, string const & what,
 			    string const & path, bool process_events)
 {
-	lyxerr << "\nRunning: " << what << endl;
+	LYXERR(Debug::INFO,"Running: " << what);
 
+	string infile;
 	string outfile;
 	string errfile;
-	QString cmd = QString::fromLocal8Bit(
-			parsecmd(what, outfile, errfile).c_str());
+	QString const cmd = QString::fromLocal8Bit(
+			parsecmd(what, infile, outfile, errfile).c_str());
 
-	SystemcallPrivate d(outfile, errfile);
+	SystemcallPrivate d(infile, outfile, errfile);
 
-
-	d.startProcess(cmd, path);
+#ifdef Q_OS_WIN32
+	// QProcess::startDetached cannot provide environment variables. When the
+	// environment variables are set using the latexEnvCmdPrefix and the process
+	// is started with QProcess::startDetached, a console window is shown every 
+	// time a viewer is started. To avoid this, we fall back on Windows to the 
+	// original implementation that creates a QProcess object.
+	d.startProcess(cmd, path, false);
 	if (!d.waitWhile(SystemcallPrivate::Starting, process_events, -1)) {
 		LYXERR0("Systemcall: '" << cmd << "' did not start!");
 		LYXERR0("error " << d.errorMessage());
 		return 10;
 	}
-
 	if (how == DontWait) {
-		QProcess* released = d.releaseProcess();
-		(void) released; // TODO who deletes it?
+		d.releaseProcess();
 		return 0;
 	}
+#else
+	d.startProcess(cmd, path, how == DontWait);
+	if (how == DontWait && d.state == SystemcallPrivate::Running)
+		return 0;
+
+	if (d.state == SystemcallPrivate::Error
+			|| !d.waitWhile(SystemcallPrivate::Starting, process_events, -1)) {
+		LYXERR0("Systemcall: '" << cmd << "' did not start!");
+		LYXERR0("error " << d.errorMessage());
+		return 10;
+	}
+#endif
 
 	if (!d.waitWhile(SystemcallPrivate::Running, process_events,
 			 os::timeout_min() * 60 * 1000)) {
@@ -277,15 +296,19 @@ int Systemcall::startscript(Starttype how, string const & what,
 }
 
 
-SystemcallPrivate::SystemcallPrivate(std::string const & of,
+SystemcallPrivate::SystemcallPrivate(std::string const & sf,
+				     std::string const & of,
 				     std::string const & ef) :
 				process_(new QProcess), 
 				out_index_(0),
 				err_index_(0),
+				in_file_(sf),
 				out_file_(of), 
 				err_file_(ef), 
 				process_events_(false)
 {
+	if (!in_file_.empty())
+		process_->setStandardInputFile(QString::fromLocal8Bit(in_file_.c_str()));
 	if (!out_file_.empty()) {
 		if (out_file_[0] == '&') {
 			if (subst(out_file_, " ", "") == "&2"
@@ -341,10 +364,18 @@ SystemcallPrivate::SystemcallPrivate(std::string const & of,
 }
 
 
-void SystemcallPrivate::startProcess(QString const & cmd, string const & path)
+void SystemcallPrivate::startProcess(QString const & cmd, string const & path, bool detached)
 {
 	cmd_ = cmd;
-	if (process_) {
+	if (detached) {
+		state = SystemcallPrivate::Running;
+		if (!QProcess::startDetached(toqstr(latexEnvCmdPrefix(path)) + cmd_)) {
+			state = SystemcallPrivate::Error;
+			return;
+		}
+		QProcess* released = releaseProcess();
+		delete released;
+	} else if (process_) {
 		state = SystemcallPrivate::Starting;
 		process_->start(toqstr(latexEnvCmdPrefix(path)) + cmd_);
 	}
@@ -353,7 +384,7 @@ void SystemcallPrivate::startProcess(QString const & cmd, string const & path)
 
 void SystemcallPrivate::processEvents()
 {
-	if(process_events_) {
+	if (process_events_) {
 		QCoreApplication::processEvents(/*QEventLoop::ExcludeUserInputEvents*/);
 	}
 }
@@ -422,7 +453,7 @@ bool SystemcallPrivate::waitWhile(State waitwhile, bool process_events, int time
 		return state != Error;
 	} 
 
-	// process events while waiting whith timeout
+	// process events while waiting with timeout
 	QTime timer;
 	timer.start();
 	while (state == waitwhile && state != Error && !timedout) {
