@@ -26,7 +26,7 @@
 #include <stdlib.h>
 
 #ifdef __APPLE__
-#include <Carbon/Carbon.h>
+#include <CoreServices/CoreServices.h>
 #endif
 
 using namespace std;
@@ -51,7 +51,7 @@ void init(int argc, char * argv[])
 
 string utf8_argv(int i)
 {
-	LASSERT(i < argc_, /**/);
+	LASSERT(i < argc_, return "");
 	return to_utf8(from_local8bit(argv_[i]));
 }
 
